@@ -96,13 +96,13 @@
 //! #include <error.h>
 //!
 //! pgp_error_t err;
-//! pgp_tpk_t tpk;
+//! pgp_cert_t cert;
 //!
-//! tpk = pgp_tpk_from_file (&err, "../openpgp/tests/data/keys/testy.pgp");
-//! if (tpk == NULL)
-//!   error (1, 0, "pgp_tpk_from_bytes: %s", pgp_error_to_string (err));
+//! cert = pgp_cert_from_file (&err, "../openpgp/tests/data/keys/testy.pgp");
+//! if (cert == NULL)
+//!   error (1, 0, "pgp_cert_from_bytes: %s", pgp_error_to_string (err));
 //!
-//! pgp_tpk_free (tpk);
+//! pgp_cert_free (cert);
 //! ```
 
 #![warn(missing_docs)]
@@ -113,7 +113,6 @@ extern crate lazy_static;
 extern crate libc;
 extern crate native_tls;
 extern crate memsec;
-extern crate nettle;
 
 extern crate sequoia_ffi_macros;
 use sequoia_ffi_macros::{
@@ -140,6 +139,8 @@ pub(crate) use crate::openpgp::{
     MoveIntoRaw,
     MoveResultIntoRaw,
     Maybe,
+    maybe_time,
+    to_time_t,
 };
 
 /* Error handling with implicit context.  */
