@@ -93,7 +93,7 @@ pub trait Handler {
 /// A factory for handlers.
 pub type HandlerFactory = fn(descriptor: Descriptor,
                              handle: tokio_core::reactor::Handle)
-                             -> Result<Box<Handler>>;
+                             -> Result<Box<dyn Handler>>;
 
 /// A descriptor is used to connect to a service.
 #[derive(Clone)]
@@ -275,6 +275,8 @@ impl Descriptor {
             .arg(format!("{}", self.ctx.ephemeral()))
             // l will be closed here if the exec fails.
             .stdin(unsafe { Stdio::from_raw_fd(fd) })
+            .stdout(Stdio::null())
+            .stderr(Stdio::null())
             .spawn()?;
         Ok(())
     }
