@@ -8,33 +8,60 @@
 //! See the [get example].
 //!
 //! [draft-koch]: https://datatracker.ietf.org/doc/html/draft-koch-openpgp-webkey-service/#section-3.1
-//! [get example]: get#example
+//! [get example]: fn.get.html#example
 //!
 
 
 // XXX: We might want to merge the 2 structs in the future and move the
 // functions to methods.
+extern crate tempfile;
 extern crate tokio_core;
 
+use std::collections::HashMap;
 use std::fmt;
+use std::fs;
+use std::path::{Path, PathBuf};
 
+use failure::ResultExt;
 use futures::{future, Future, Stream};
 use hyper::{Uri, Client};
 use hyper_tls::HttpsConnector;
-// Hash implements the traits for Sha1
-// Sha1 is used to obtain a 20 bytes digest that after zbase32 encoding can
-// be used as file name
-use nettle::{
-    Hash, hash::insecure_do_not_use::Sha1,
-};
 use url;
 
-use crate::openpgp::TPK;
+use crate::openpgp::{
+    Fingerprint,
+    Cert,
+};
 use crate::openpgp::parse::Parse;
-use crate::openpgp::tpk::TPKParser;
+use crate::openpgp::serialize::Serialize;
+use crate::openpgp::types::HashAlgorithm;
+use crate::openpgp::cert::CertParser;
 
 use super::{Result, Error};
 
+/// WKD variants.
+///
+/// There are two variants of the URL scheme.  `Advanced` should be
+/// preferred.
+#[derive(Clone, Copy, Debug, PartialEq, Eq)]
+pub enum Variant {
+    /// Advanced variant.
+    ///
+    /// This method uses a separate subdomain and is more flexible.
+    /// This method should be preferred.
+    Advanced,
+    /// Direct variant.
+    ///
+    /// This method is deprecated.
+    Direct,
+}
+
+impl Default for Variant {
+    fn default() -> Self {
+        Variant::Advanced
+    }
+}
+
 
 /// Stores the local_part and domain of an email address.
 pub struct EmailAddress {
@@ -82,7 +109,7 @@ impl EmailAddress {
 ///
 /// NOTE: This is a different `Url` than [`url::Url`] (`url` crate) that is
 /// actually returned with the method [to_url](#method.to_url)
-#[derive(Clone)]
+#[derive(Debug, Clone)]
 pub struct Url {
     domain: String,
     local_encoded: String,
@@ -109,10 +136,11 @@ impl Url {
     }
 
     /// Returns an URL string from a [`Url`].
-    pub fn build<T>(&self, direct_method: T) -> String
-            where T: Into<Option<bool>> {
-        let direct_method = direct_method.into().unwrap_or(false);
-        if direct_method {
+    pub fn build<V>(&self, variant: V) -> String
+        where V: Into<Option<Variant>>
+    {
+        let variant = variant.into().unwrap_or_default();
+        if variant == Variant::Direct {
             format!("https://{}/.well-known/openpgpkey/hu/{}?l={}",
                     self.domain, self.local_encoded, self.local_part)
         } else {
@@ -123,20 +151,43 @@ impl Url {
     }
 
     /// Returns an [`url::Url`].
-    pub fn to_url<T>(&self, direct_method: T) -> Result<url::Url>
-            where T: Into<Option<bool>> {
-        let url_string = self.build(direct_method);
+    pub fn to_url<V>(&self, variant: V) -> Result<url::Url>
+            where V: Into<Option<Variant>> {
+        let url_string = self.build(variant);
         let url_url = url::Url::parse(url_string.as_str())?;
         Ok(url_url)
     }
 
     /// Returns an [`hyper::Uri`].
-    pub fn to_uri<T>(&self, direct_method: T) -> Result<Uri>
-            where T: Into<Option<bool>> {
-        let url_string = self.build(direct_method);
+    pub fn to_uri<V>(&self, variant: V) -> Result<Uri>
+            where V: Into<Option<Variant>> {
+        let url_string = self.build(variant);
         let uri = url_string.as_str().parse::<Uri>()?;
         Ok(uri)
     }
+
+    /// Returns a [`PathBuf`].
+    pub fn to_file_path<V>(&self, variant: V) -> Result<PathBuf>
+        where V: Into<Option<Variant>>
+    {
+        // Create the directories string.
+        let variant = variant.into().unwrap_or_default();
+        let url = self.to_url(variant)?;
+        // Can not create path_buf as:
+        // let path_buf: PathBuf = [url.domain().unwrap(), url.path()]
+        //    .iter().collect();
+        // or:
+        // let mut path_buf = PathBuf::new();
+        // path_buf.push(url.domain().unwrap());
+        // path_buf.push(url.path());
+        // Because the domain part will disappear, dunno why.
+        // url.to_file_path() would not create the directory with the domain,
+        // but expect the hostname to match the domain.
+        // Ignore the query part of the url, take only the domain and path.
+        let string = format!("{}{}", url.domain().unwrap(), url.path());
+        let path_buf = PathBuf::from(string);
+        Ok(path_buf)
+    }
 }
 
 
@@ -148,17 +199,19 @@ impl Url {
 ///     described in [RFC6189], section 5.1.6. The resulting string has a
 ///     fixed length of 32 octets.
 fn encode_local_part<S: AsRef<str>>(local_part: S) -> String {
-    let mut hasher = Sha1::default();
-    hasher.update(local_part.as_ref().as_bytes());
-    // Declare and assign a 20 bytes length vector to use in hasher.result
-    let mut local_hash = vec![0; 20];
-    hasher.digest(&mut local_hash);
+    let local_part = local_part.as_ref();
+
+    let mut digest = vec![0; 20];
+    let mut ctx = HashAlgorithm::SHA1.context().expect("must be implemented");
+    ctx.update(local_part.as_bytes());
+    ctx.digest(&mut digest);
+
     // After z-base-32 encoding 20 bytes, it will be 32 bytes long.
-    zbase32::encode_full_bytes(&local_hash[..])
+    zbase32::encode_full_bytes(&digest[..])
 }
 
 
-/// Parse an HTTP response body that may contain TPKs and filter them based on
+/// Parse an HTTP response body that may contain Certs and filter them based on
 /// whether they contain a userid with the given email address.
 ///
 /// From [draft-koch]:
@@ -167,36 +220,38 @@ fn encode_local_part<S: AsRef<str>>(local_part: S) -> String {
 /// The key needs to carry a User ID packet ([RFC4880]) with that mail
 /// address.
 /// ```
-pub(crate) fn parse_body<S: AsRef<str>>(body: &[u8], email_address: S)
-        -> Result<Vec<TPK>> {
+fn parse_body<S: AsRef<str>>(body: &[u8], email_address: S)
+        -> Result<Vec<Cert>> {
     let email_address = email_address.as_ref();
     // This will fail on the first packet that can not be parsed.
-    let packets = TPKParser::from_bytes(&body)?;
+    let packets = CertParser::from_bytes(&body)?;
     // Collect only the correct packets.
-    let tpks: Vec<TPK> = packets.flatten().collect();
-    // Collect only the TPKs that contain the email in any of their userids
-    let valid_tpks: Vec<TPK> = tpks.iter()
-        // XXX: This filter could become a TPK method, but it adds other API
+    let certs: Vec<Cert> = packets.flatten().collect();
+    if certs.is_empty() {
+        return Err(Error::NotFound.into());
+    }
+
+    // Collect only the Certs that contain the email in any of their userids
+    let valid_certs: Vec<Cert> = certs.iter()
+        // XXX: This filter could become a Cert method, but it adds other API
         // method to maintain
-        .filter(|tpk| {tpk.userids()
+        .filter(|cert| {cert.userids().bundles()
             .any(|uidb|
-                if let Ok(Some(a)) = uidb.userid().address() {
+                if let Ok(Some(a)) = uidb.userid().email() {
                     a == email_address
                 } else { false })
         }).cloned().collect();
-    if valid_tpks.is_empty() {
+    if valid_certs.is_empty() {
         Err(Error::EmailNotInUserids(email_address.into()).into())
     } else {
-        Ok(valid_tpks)
+        Ok(valid_certs)
     }
 }
 
 
-/// Retrieves the TPKs that contain userids with a given email address
+/// Retrieves the Certs that contain userids with a given email address
 /// from a Web Key Directory URL.
 ///
-/// This function is call by [net::wkd::get](../../wkd/fn.get.html).
-///
 /// From [draft-koch]:
 ///
 /// ```text
@@ -230,13 +285,13 @@ pub(crate) fn parse_body<S: AsRef<str>>(body: &[u8], email_address: S)
 ///
 /// let email_address = "foo@bar.baz";
 /// let mut core = Core::new().unwrap();
-/// let tpks = core.run(wkd::get(&email_address)).unwrap();
+/// let certs = core.run(wkd::get(&email_address)).unwrap();
 /// ```
 
 // XXX: Maybe the direct method should be tried on other errors too.
 // https://mailarchive.ietf.org/arch/msg/openpgp/6TxZc2dQFLKXtS0Hzmrk963EteE
 pub fn get<S: AsRef<str>>(email_address: S)
-                          -> impl Future<Item=Vec<TPK>, Error=failure::Error> {
+                          -> impl Future<Item=Vec<Cert>, Error=failure::Error> {
     let email = email_address.as_ref().to_string();
     future::lazy(move || -> Result<_> {
         // First, prepare URIs and client.
@@ -246,7 +301,8 @@ pub fn get<S: AsRef<str>>(email_address: S)
         let https = HttpsConnector::new(4)?;
         let client = Client::builder().build::<_, hyper::Body>(https);
 
-        Ok((email, client, wkd_url.to_uri(false)?, wkd_url.to_uri(true)?))
+        use self::Variant::*;
+        Ok((email, client, wkd_url.to_uri(Advanced)?, wkd_url.to_uri(Direct)?))
     }).and_then(|(email, client, advanced_uri, direct_uri)| {
         // First, try the Advanced Method.
         client.get(advanced_uri)
@@ -266,13 +322,112 @@ pub fn get<S: AsRef<str>>(email_address: S)
     })
 }
 
+/// Inserts a key into a Web Key Directory.
+///
+/// Creates a WKD hierarchy at `base_path` for `domain`, and inserts
+/// the given `cert`.  If `cert` already exists in the WKD, it is
+/// updated.  Any existing Certs are left in place.
+///
+/// # Errors
+///
+/// If the Cert does not have a well-formed UserID with `domain`,
+/// `Error::InvalidArgument` is returned.
+pub fn insert<P, S, V>(base_path: P, domain: S, variant: V,
+                       cert: &Cert)
+                       -> Result<()>
+    where P: AsRef<Path>,
+          S: AsRef<str>,
+          V: Into<Option<Variant>>
+{
+    let base_path = base_path.as_ref();
+    let domain = domain.as_ref();
+    let variant = variant.into().unwrap_or_default();
+
+    // First, check which UserIDs are in `domain`.
+    let addresses = cert.userids().bundles().filter_map(|uidb| {
+        uidb.userid().email().unwrap_or(None).and_then(|addr| {
+            if EmailAddress::from(&addr).ok().map(|e| e.domain == domain)
+                .unwrap_or(false)
+            {
+                Url::from(&addr).ok()
+            } else {
+                None
+            }
+        })
+    }).collect::<Vec<_>>();
+
+    // Any?
+    if addresses.len() == 0 {
+        return Err(openpgp::Error::InvalidArgument(
+            format!("Key {} does not have a UserID in {}", cert, domain)
+        ).into());
+    }
+
+    // Finally, create the files.
+    for address in addresses.into_iter() {
+        let path = base_path.join(address.to_file_path(variant)?);
+        fs::create_dir_all(path.parent().expect("by construction"))?;
+        let mut keyring = KeyRing::default();
+        if path.is_file() {
+            for t in CertParser::from_file(&path).context(
+                format!("Error parsing existing file {:?}", path))?
+            {
+                keyring.insert(t.context(
+                    format!("Malformed Cert in existing {:?}", path))?)?;
+            }
+        }
+        keyring.insert(cert.clone())?;
+        let mut file = fs::File::create(&path)?;
+        keyring.export(&mut file)?;
+    }
+
+    Ok(())
+}
+
+struct KeyRing(HashMap<Fingerprint, Cert>);
+
+impl Default for KeyRing {
+    fn default() -> Self {
+        Self(Default::default())
+    }
+}
+
+impl KeyRing {
+    fn insert(&mut self, cert: Cert) -> Result<()> {
+        let fp = cert.fingerprint();
+        if let Some(existing) = self.0.get_mut(&fp) {
+            *existing = existing.clone().merge(cert)?;
+        } else {
+            self.0.insert(fp, cert);
+        }
+        Ok(())
+    }
+}
+
+impl Serialize for KeyRing {
+    fn serialize(&self, o: &mut dyn std::io::Write) -> Result<()> {
+        for cert in self.0.values() {
+            cert.serialize(o)?;
+        }
+        Ok(())
+    }
+
+    fn export(&self, o: &mut dyn std::io::Write) -> Result<()> {
+        for cert in self.0.values() {
+            cert.export(o)?;
+        }
+        Ok(())
+    }
+}
+
 
 #[cfg(test)]
 mod tests {
     use crate::openpgp::serialize::Serialize;
-    use crate::openpgp::tpk::TPKBuilder;
+    use crate::openpgp::cert::CertBuilder;
 
     use super::*;
+    use self::Variant::*;
 
     #[test]
     fn encode_local_part_succed() {
@@ -309,35 +464,93 @@ mod tests {
             "https://example.com/\
              .well-known/openpgpkey/hu/\
              stnkabub89rpcphiz4ppbxixkwyt1pic?l=test1";
-        assert_eq!(expected_url, wkd_url.clone().build(true));
+        assert_eq!(expected_url, wkd_url.clone().build(Direct));
         assert_eq!(url::Url::parse(expected_url).unwrap(),
-                   wkd_url.clone().to_url(true).unwrap());
+                   wkd_url.clone().to_url(Direct).unwrap());
         assert_eq!(expected_url.parse::<Uri>().unwrap(),
-                   wkd_url.to_uri(true).unwrap());
+                   wkd_url.to_uri(Direct).unwrap());
+    }
+
+    #[test]
+    fn url_to_file_path() {
+        // Advanced method
+        let expected_path =
+            "openpgpkey.example.com/\
+             .well-known/openpgpkey/example.com/hu/\
+             stnkabub89rpcphiz4ppbxixkwyt1pic";
+        let wkd_url = Url::from("test1@example.com").unwrap();
+        assert_eq!(expected_path,
+            wkd_url.clone().to_file_path(None).unwrap().to_str().unwrap());
+
+        // Direct method
+        let expected_path =
+            "example.com/\
+             .well-known/openpgpkey/hu/\
+             stnkabub89rpcphiz4ppbxixkwyt1pic";
+        assert_eq!(expected_path,
+            wkd_url.to_file_path(Direct).unwrap().to_str().unwrap());
     }
 
     #[test]
     fn test_parse_body() {
-        let (tpk, _) = TPKBuilder::new()
+        let (cert, _) = CertBuilder::new()
             .add_userid("test@example.example")
             .generate()
             .unwrap();
         let mut buffer: Vec<u8> = Vec::new();
-        tpk.serialize(&mut buffer).unwrap();
-        let valid_tpks = parse_body(&buffer, "juga@sequoia-pgp.org");
-        // The userid is not in the TPK
-        assert!(valid_tpks.is_err());
-        // XXX: add userid to the tpk, instead of creating a new one
-        // tpk.add_userid("juga@sequoia.org");
-        let (tpk, _) = TPKBuilder::new()
+        cert.serialize(&mut buffer).unwrap();
+        let valid_certs = parse_body(&buffer, "juga@sequoia-pgp.org");
+        // The userid is not in the Cert
+        assert!(valid_certs.is_err());
+        // XXX: add userid to the cert, instead of creating a new one
+        // cert.add_userid("juga@sequoia.org");
+        let (cert, _) = CertBuilder::new()
             .add_userid("test@example.example")
             .add_userid("juga@sequoia-pgp.org")
             .generate()
             .unwrap();
-        tpk.serialize(&mut buffer).unwrap();
-        let valid_tpks = parse_body(&buffer, "juga@sequoia-pgp.org");
-        assert!(valid_tpks.is_ok());
-        assert!(valid_tpks.unwrap().len() == 1);
-        // XXX: Test with more TPKs
+        cert.serialize(&mut buffer).unwrap();
+        let valid_certs = parse_body(&buffer, "juga@sequoia-pgp.org");
+        assert!(valid_certs.is_ok());
+        assert!(valid_certs.unwrap().len() == 1);
+        // XXX: Test with more Certs
+    }
+
+    #[test]
+    fn wkd_generate() {
+       let (cert, _) = CertBuilder::new()
+            .add_userid("test1@example.example")
+            .add_userid("juga@sequoia-pgp.org")
+            .generate()
+            .unwrap();
+        let (cert2, _) = CertBuilder::new()
+            .add_userid("justus@sequoia-pgp.org")
+            .generate()
+            .unwrap();
+
+        let dir = tempfile::tempdir().unwrap();
+        let dir_path = dir.path();
+        insert(&dir_path, "sequoia-pgp.org", None, &cert).unwrap();
+        insert(&dir_path, "sequoia-pgp.org", None, &cert2).unwrap();
+
+        // justus and juga files will be generated, but not test one.
+        let path = dir_path.join(
+            "openpgpkey.sequoia-pgp.org/\
+             .well-known/openpgpkey/sequoia-pgp.org/hu\
+             /jwp7xjqkdujgz5op6bpsoypg34pnrgmq");
+        // Check that justus file was created
+        assert!(path.is_file());
+        let path = dir_path.join(
+            "openpgpkey.sequoia-pgp.org/\
+             .well-known/openpgpkey/sequoia-pgp.org/hu\
+             /7t1uqk9cwh1955776rc4z1gqf388566j");
+        // Check that juga file was created.
+        assert!(path.is_file());
+        // Check that the file for test uid is not created.
+        let path = dir_path.join(
+            "openpgpkey.example.com/\
+             .well-known/openpgpkey/example.com/hu/\
+             stnkabub89rpcphiz4ppbxixkwyt1pic");
+        assert!(!path.is_file());
     }
 }
