@@ -2,19 +2,19 @@
 //!
 //! Wraps [`sequoia-openpgp::armor`].
 //!
-//! [`sequoia-openpgp::armor`]: ../../sequoia_openpgp/armor/index.html
+//! [`sequoia-openpgp::armor`]: ../../../sequoia_openpgp/armor/index.html
 
 use std::mem::size_of;
 use std::ptr;
 use std::slice;
-use std::io;
 use libc::{self, c_char, c_int, size_t};
 
 extern crate sequoia_openpgp;
 use self::sequoia_openpgp::armor;
 
-use super::io::{Reader, ReaderKind};
+use super::io::{Reader, ReaderKind, WriterKind};
 use crate::Maybe;
+use crate::MoveFromRaw;
 use crate::MoveIntoRaw;
 use crate::MoveResultIntoRaw;
 use crate::RefRaw;
@@ -303,6 +303,9 @@ pub extern "C" fn pgp_armor_reader_headers(errp: Option<&mut *mut crate::error::
 ///
 /// A filter that applies ASCII Armor to the data written to it.
 ///
+/// Note: You must call `pgp_armor_writer_finalize` to deallocate this
+/// writer.
+///
 /// # Example
 ///
 /// ```c
@@ -322,7 +325,7 @@ pub extern "C" fn pgp_armor_reader_headers(errp: Option<&mut *mut crate::error::
 ///   size_t len = 0;
 ///   pgp_writer_t alloc;
 ///   pgp_writer_t armor;
-///   pgp_error_t err;
+///   pgp_error_t err = NULL;
 ///
 ///   char *message = "Hello world!";
 ///   struct pgp_armor_header header[] = {
@@ -337,8 +340,9 @@ pub extern "C" fn pgp_armor_reader_headers(errp: Option<&mut *mut crate::error::
 ///
 ///   if (pgp_writer_write (&err, armor, (uint8_t *) message, strlen (message)) < 0)
 ///     error (1, 0, "Writing failed: %s", pgp_error_to_string (err));
-//
-///   pgp_writer_free (armor);
+///
+///   pgp_armor_writer_finalize (&err, armor);
+///   assert (err == NULL);
 ///   pgp_writer_free (alloc);
 ///
 ///   assert (len == 114);
@@ -386,7 +390,28 @@ pub extern "C" fn pgp_armor_writer_new
         header_.iter().map(|h| (h.0.as_ref(), h.1.as_ref())).collect();
 
     armor::Writer::new(inner, kind, &header)
-        .map(|w| -> Box<io::Write> { Box::new(w) })
+        .map(|w| WriterKind::Armored(w))
         .map_err(|e| ::failure::Error::from(e))
         .move_into_raw(errp)
 }
+
+/// Finalizes the armor writer.
+///
+/// Consumes the writer.  No further deallocation of the writer is
+/// required.
+#[::sequoia_ffi_macros::extern_fn] #[no_mangle]
+pub extern "C" fn pgp_armor_writer_finalize
+    (errp: Option<&mut *mut crate::error::Error>,
+     writer: *mut super::io::Writer)
+     -> crate::error::Status
+{
+    ffi_make_fry_from_errp!(errp);
+    let writer = if let WriterKind::Armored(writer) = writer.move_from_raw() {
+        writer
+    } else {
+        panic!("FFI contract violation: Wrong parameter type: \
+                expected armor writer");
+    };
+
+    ffi_try_status!(writer.finalize().map_err(|e| e.into()))
+}
