@@ -162,6 +162,30 @@ macro_rules! ffi_make_fry_from_errp {
             };
         }
 
+        /// Like try! for ffi glue.
+        ///
+        /// Evaluates the given expression.  `Ok(v)` evaluates to `v`.
+        /// On failure, stashes the error in the context and returns
+        /// the appropriate Status code.
+        #[allow(unused_macros)]
+        macro_rules! ffi_try_or_status {
+            ($expr:expr) => {
+                match $expr {
+                    Ok(v) => v,
+                    Err(e) => {
+                        use crate::MoveIntoRaw;
+                        use failure::Error;
+                        let status = crate::error::Status::from(&e);
+                        if let Some(errp) = $errp {
+                            let e : Error = e.into();
+                            *errp = e.move_into_raw();
+                        }
+                        return status;
+                    },
+                }
+            };
+        }
+
         /// Like try! for ffi glue.
         ///
         /// Unwraps the given expression.  On failure, stashes the
@@ -279,6 +303,32 @@ pub(crate) fn build_hasher() -> DefaultHasher {
     RANDOM_STATE.build_hasher()
 }
 
+/* time_t support.  */
+
+/// Converts a time_t for use in Sequoia.
+pub(crate) fn maybe_time(t: libc::time_t) -> Option<std::time::SystemTime> {
+    if t == 0 {
+        None
+    } else {
+        Some(std::time::UNIX_EPOCH + std::time::Duration::new(t as u64, 0))
+    }
+}
+
+/// Converts a time_t for use in C.
+#[allow(dead_code)]
+pub(crate) fn to_time_t<T>(t: T) -> libc::time_t
+    where T: Into<Option<std::time::SystemTime>>
+{
+    if let Some(t) = t.into() {
+        match t.duration_since(std::time::UNIX_EPOCH) {
+            Ok(d) => d.as_secs() as libc::time_t,
+            Err(_) => 0, // Unrepresentable.
+        }
+    } else {
+        0
+    }
+}
+
 pub mod armor;
 pub mod crypto;
 pub mod error;
@@ -289,6 +339,7 @@ pub mod packet;
 pub mod packet_pile;
 pub mod parse;
 pub mod serialize;
-pub mod tpk;
+pub mod cert;
 pub mod tsk;
 pub mod revocation_status;
+pub mod policy;
