@@ -8,7 +8,7 @@
 //!
 //! Wraps [`sequoia-openpgp::Fingerprint`].
 //!
-//! [`sequoia-openpgp::Fingerprint`]: ../../sequoia_openpgp/enum.Fingerprint.html
+//! [`sequoia-openpgp::Fingerprint`]: ../../../sequoia_openpgp/enum.Fingerprint.html
 
 use std::slice;
 use libc::{c_char, size_t};
@@ -29,7 +29,7 @@ use crate::RefRaw;
 ///
 /// Wraps [`sequoia-openpgp::Fingerprint`].
 ///
-/// [`sequoia-openpgp::Fingerprint`]: ../../sequoia_openpgp/enum.Fingerprint.html
+/// [`sequoia-openpgp::Fingerprint`]: ../../../sequoia_openpgp/enum.Fingerprint.html
 #[crate::ffi_wrapper_type(prefix = "pgp_",
                      derive = "Clone, Debug, Display, Hash, PartialEq")]
 pub struct Fingerprint(openpgp::Fingerprint);
@@ -99,5 +99,5 @@ fn pgp_fingerprint_to_hex(fp: *const Fingerprint)
 #[::sequoia_ffi_macros::extern_fn] #[no_mangle] pub extern "C"
 fn pgp_fingerprint_to_keyid(fp: *const Fingerprint)
                             -> *mut KeyID {
-    fp.ref_raw().to_keyid().move_into_raw()
+    openpgp::KeyID::from(fp.ref_raw()).move_into_raw()
 }
