@@ -84,13 +84,13 @@
 //! #include <error.h>
 //!
 //! pgp_error_t err;
-//! pgp_tpk_t tpk = pgp_tpk_from_file (&err, "../openpgp/tests/data/keys/testy.pgp");
-//! if (tpk == NULL)
-//!   error (1, 0, "pgp_tpk_from_file: %s", pgp_error_to_string (err));
+//! pgp_cert_t cert = pgp_cert_from_file (&err, "../openpgp/tests/data/keys/testy.pgp");
+//! if (cert == NULL)
+//!   error (1, 0, "pgp_cert_from_file: %s", pgp_error_to_string (err));
 //!
 //!  /* XXX: Do something interesting.  */
 //!
-//! pgp_tpk_free (tpk);
+//! pgp_cert_free (cert);
 //! ```
 //!
 //! The examples are compiled and executed as part of the test suite,
@@ -134,7 +134,7 @@
 //! or the allocation for the string failed, which is considered
 //! undefined behavior.
 //!
-//! [`pgp_fingerprint_to_string`]: struct.Fingerprint.html#method.pgp_fingerprint_to_string
+//! [`pgp_fingerprint_to_string`]: fingerprint/struct.Fingerprint.html#method.pgp_fingerprint_to_string
 //!
 //! Failing functions signal failure either in-band (e.g. `NULL`, or
 //! -1), using `pgp_status_t`, and may store complex error information
@@ -145,7 +145,7 @@
 //! on the other hand, will return `NULL` and store a complex error at
 //! the location given using the `errp` parameter.
 //!
-//! [`pgp_fingerprint_from_hex`]: struct.Fingerprint.html#method.pgp_fingerprint_from_hex
+//! [`pgp_fingerprint_from_hex`]: fingerprint/struct.Fingerprint.html#method.pgp_fingerprint_from_hex
 //! [`pgp_packet_parser_from_bytes`]: parse/fn.pgp_packet_parser_from_bytes.html
 //!
 //! Errors may be inspected using [`pgp_error_status`], and formatted
@@ -320,7 +320,6 @@ extern crate failure;
 extern crate lazy_static;
 extern crate libc;
 extern crate memsec;
-extern crate nettle;
 
 extern crate sequoia_ffi_macros;
 use sequoia_ffi_macros::{
