@@ -8,17 +8,17 @@
 //! Wraps the streaming parsing functions, see
 //! [`sequoia-openpgp::parse::stream`].
 //!
-//! [`sequoia-openpgp::parse::stream`]: ../../../sequoia_openpgp/parse/stream/index.html
+//! [`sequoia-openpgp::parse::stream`]: ../../../../sequoia_openpgp/parse/stream/index.html
 
 use std::ptr;
 use libc::{c_int, c_void, time_t};
 
 extern crate sequoia_openpgp as openpgp;
-extern crate time;
 
 use self::openpgp::{
+    cert::components::Amalgamation,
     crypto::SessionKey,
-    constants::SymmetricAlgorithm,
+    types::SymmetricAlgorithm,
     packet::{
         PKESK,
         SKESK,
@@ -39,16 +39,18 @@ use crate::MoveIntoRaw;
 use crate::MoveResultIntoRaw;
 use crate::RefRaw;
 use crate::RefMutRaw;
+use crate::maybe_time;
 
 use super::super::{
     error::Status,
     crypto,
     io,
     keyid,
-    tpk::TPK,
+    cert::Cert,
     packet::signature::Signature,
     packet::key::Key,
     parse::PacketParser,
+    policy::Policy,
     revocation_status::RevocationStatus,
 };
 
@@ -169,9 +171,54 @@ fn pgp_verification_result_variant(result: *const VerificationResult)
 {
     use self::stream::VerificationResult::*;
     match result.ref_raw() {
-        GoodChecksum(..) => 1,
-        MissingKey(_) => 2,
-        BadChecksum(_) => 3,
+        GoodChecksum { .. } => 1,
+        MissingKey { .. } => 2,
+        NotAlive { .. } => 3,
+        Error { .. } => 4,
+    }
+}
+
+macro_rules! make_decomposition_fn {
+    ($fn_name:ident, $variant:path) => {
+#[::sequoia_ffi_macros::extern_fn] #[no_mangle] pub extern "C"
+fn $fn_name<'a>(
+    result: *const VerificationResult<'a>,
+    sig_r: Maybe<*mut Signature>,
+    cert_r: Maybe<*mut Cert>,
+    key_r: Maybe<*mut Key>,
+    binding_r: Maybe<*mut Signature>,
+    revocation_status_r:
+    Maybe<*mut RevocationStatus<'a>>)
+    -> bool
+{
+    use self::stream::VerificationResult::*;
+    if let $variant { sig, cert, ka } = result.ref_raw() {
+        if let Some(mut p) = sig_r {
+            *unsafe { p.as_mut() } = sig.move_into_raw();
+        }
+        if let Some(mut p) = cert_r {
+            *unsafe { p.as_mut() } = cert.move_into_raw();
+        }
+        if let Some(mut p) = key_r {
+            *unsafe { p.as_mut() } = {
+                let key = ka.key()
+                    .mark_parts_unspecified_ref()
+                    .mark_role_unspecified_ref();
+                key.move_into_raw()
+            };
+        }
+        if let Some(mut p) = binding_r {
+            *unsafe { p.as_mut() } =
+                ka.binding_signature().move_into_raw();
+        }
+        if let Some(mut p) = revocation_status_r {
+            *unsafe { p.as_mut() } = ka.revoked().move_into_raw();
+        }
+        true
+    } else {
+        false
+    }
+}
     }
 }
 
@@ -180,40 +227,24 @@ fn pgp_verification_result_variant(result: *const VerificationResult)
 /// Returns `true` iff the given value is a
 /// `VerificationResult::GoodChecksum`, and returns the variants members
 /// in `sig_r` and the like iff `sig_r != NULL`.
+make_decomposition_fn!(pgp_verification_result_good_checksum, GoodChecksum);
+
+/// Decomposes a `VerificationResult::NotAlive`.
+///
+/// Returns `true` iff the given value is a
+/// `VerificationResult::NotAlive`, and returns the variant's members
+/// in `sig_r` and the like iff `sig_r != NULL`.
 #[::sequoia_ffi_macros::extern_fn] #[no_mangle] pub extern "C"
-fn pgp_verification_result_good_checksum<'a>(
+fn pgp_verification_result_not_alive<'a>(
     result: *const VerificationResult<'a>,
-    sig_r: Maybe<*mut Signature>,
-    tpk_r: Maybe<*mut TPK>,
-    key_r: Maybe<*mut Key>,
-    binding_r: Maybe<Maybe<Signature>>,
-    revocation_status_r:
-    Maybe<*mut RevocationStatus<'a>>)
+    sig_r: Maybe<*mut Signature>)
     -> bool
 {
     use self::stream::VerificationResult::*;
-    if let GoodChecksum(ref sig, ref tpk, ref key, ref binding, ref revocation)
-        = result.ref_raw()
-    {
+    if let NotAlive { sig, .. } = result.ref_raw() {
         if let Some(mut p) = sig_r {
             *unsafe { p.as_mut() } = sig.move_into_raw();
         }
-        if let Some(mut p) = tpk_r {
-            *unsafe { p.as_mut() } = tpk.move_into_raw();
-        }
-        if let Some(mut p) = key_r {
-            *unsafe { p.as_mut() } = {
-                let key : &self::openpgp::packet::key::UnspecifiedKey
-                    = (*key).into();
-                key.move_into_raw()
-            };
-        }
-        if let Some(mut p) = binding_r {
-            *unsafe { p.as_mut() } = binding.move_into_raw();
-        }
-        if let Some(mut p) = revocation_status_r {
-            *unsafe { p.as_mut() } = revocation.move_into_raw();
-        }
         true
     } else {
         false
@@ -232,8 +263,7 @@ fn pgp_verification_result_missing_key<'a>(
     -> bool
 {
     use self::stream::VerificationResult::*;
-    if let MissingKey(ref sig) = result.ref_raw()
-    {
+    if let MissingKey { sig, .. } = result.ref_raw() {
         if let Some(mut p) = sig_r {
             *unsafe { p.as_mut() } = sig.move_into_raw();
         }
@@ -243,23 +273,26 @@ fn pgp_verification_result_missing_key<'a>(
     }
 }
 
-/// Decomposes a ``VerificationResult::BadChecksum`.
+/// Decomposes a `VerificationResult::Error`.
 ///
 /// Returns `true` iff the given value is a
-/// `VerificationResult::BadChecksum`, and returns the variants
-/// members in `sig_r` and the like iff `sig_r != NULL`.
+/// `VerificationResult::Error`, and returns the variants members
+/// in `sig_r` and the like iff `sig_r != NULL`.
 #[::sequoia_ffi_macros::extern_fn] #[no_mangle] pub extern "C"
-fn pgp_verification_result_bad_checksum<'a>(
+fn pgp_verification_result_error<'a>(
     result: *const VerificationResult<'a>,
-    sig_r: Maybe<*mut Signature>)
+    sig_r: Maybe<*mut Signature>,
+    err_r: Maybe<*mut crate::error::Error>)
     -> bool
 {
     use self::stream::VerificationResult::*;
-    if let BadChecksum(ref sig) = result.ref_raw()
-    {
+    if let Error { sig, error, .. } = result.ref_raw() {
         if let Some(mut p) = sig_r {
             *unsafe { p.as_mut() } = sig.move_into_raw();
         }
+        if let Some(mut p) = err_r {
+            *unsafe { p.as_mut() } = error.move_into_raw();
+        }
         true
     } else {
         false
@@ -274,13 +307,13 @@ pub struct HelperCookie {
 /// How to free the memory allocated by the callback.
 type FreeCallback = fn(*mut c_void);
 
-/// Returns the TPKs corresponding to the passed KeyIDs.
+/// Returns the Certs corresponding to the passed KeyIDs.
 ///
 /// If the free callback is not NULL, then it is called to free the
-/// returned array of TPKs.
+/// returned array of Certs.
 type GetPublicKeysCallback = fn(*mut HelperCookie,
                                 *const *mut keyid::KeyID, usize,
-                                &mut *mut *mut TPK, *mut usize,
+                                &mut *mut *mut Cert, *mut usize,
                                 *mut FreeCallback) -> Status;
 
 /// Inspect packets as they are decrypted.
@@ -339,23 +372,24 @@ impl VHelper {
 }
 
 impl VerificationHelper for VHelper {
-    fn get_public_keys(&mut self, ids: &[openpgp::KeyID])
-        -> Result<Vec<openpgp::TPK>, failure::Error>
+    fn get_public_keys(&mut self, ids: &[openpgp::KeyHandle])
+        -> Result<Vec<openpgp::Cert>, failure::Error>
     {
-        // The size of KeyID is not known in C.  Convert from an array
-        // of KeyIDs to an array of KeyID refs.
+        // The size of ID is not known in C.  Convert to KeyID, and
+        // move it to C.
         let ids : Vec<*mut keyid::KeyID> =
-            ids.iter().map(|k| k.move_into_raw()).collect();
+            ids.iter().map(|k| openpgp::KeyID::from(k.clone()).move_into_raw())
+            .collect();
 
-        let mut tpk_refs_raw : *mut *mut TPK = ptr::null_mut();
-        let mut tpk_refs_raw_len = 0usize;
+        let mut cert_refs_raw : *mut *mut Cert = ptr::null_mut();
+        let mut cert_refs_raw_len = 0usize;
 
         let mut free : FreeCallback = |_| {};
 
         let result = (self.get_public_keys_cb)(
             self.cookie,
             ids.as_ptr(), ids.len(),
-            &mut tpk_refs_raw, &mut tpk_refs_raw_len as *mut usize,
+            &mut cert_refs_raw, &mut cert_refs_raw_len as *mut usize,
             &mut free);
 
         // Free the KeyID wrappers.
@@ -370,20 +404,20 @@ impl VerificationHelper for VHelper {
                 format!("{:?}", result)).into());
         }
 
-        // Convert the array of references to TPKs to a Vec<TPK>
-        // (i.e., not a Vec<&TPK>).
-        let mut tpks : Vec<openpgp::TPK> = Vec::with_capacity(tpk_refs_raw_len);
-        for i in 0..tpk_refs_raw_len {
-            let tpk_raw = unsafe { *tpk_refs_raw.offset(i as isize) };
-            tpks.push(tpk_raw.move_from_raw());
+        // Convert the array of references to Certs to a Vec<Cert>
+        // (i.e., not a Vec<&Cert>).
+        let mut certs : Vec<openpgp::Cert> = Vec::with_capacity(cert_refs_raw_len);
+        for i in 0..cert_refs_raw_len {
+            let cert_raw = unsafe { *cert_refs_raw.offset(i as isize) };
+            certs.push(cert_raw.move_from_raw());
         }
 
-        (free)(tpk_refs_raw as *mut c_void);
+        (free)(cert_refs_raw as *mut c_void);
 
-        Ok(tpks)
+        Ok(certs)
     }
 
-    fn check(&mut self, structure: &stream::MessageStructure)
+    fn check(&mut self, structure: stream::MessageStructure)
         -> Result<(), failure::Error>
     {
         let result = (self.check_signatures_cb)(self.cookie,
@@ -420,21 +454,21 @@ impl VerificationHelper for VHelper {
 /// #include <sequoia/openpgp.h>
 ///
 /// struct verify_cookie {
-///   pgp_tpk_t key;
+///   pgp_cert_t key;
 /// };
 ///
 /// static pgp_status_t
 /// get_public_keys_cb (void *cookie_opaque,
 ///                     pgp_keyid_t *keyids, size_t keyids_len,
-///                     pgp_tpk_t **tpks, size_t *tpks_len,
+///                     pgp_cert_t **certs, size_t *certs_len,
 ///                     void (**our_free)(void *))
 /// {
-///   /* Feed the TPKs to the verifier here.  */
+///   /* Feed the Certs to the verifier here.  */
 ///   struct verify_cookie *cookie = cookie_opaque;
-///   *tpks = malloc (sizeof (pgp_tpk_t));
-///   assert (*tpks);
-///   *tpks[0] = cookie->key;
-///   *tpks_len = 1;
+///   *certs = malloc (sizeof (pgp_cert_t));
+///   assert (*certs);
+///   *certs[0] = cookie->key;
+///   *certs_len = 1;
 ///   *our_free = free;
 ///   return PGP_STATUS_SUCCESS;
 /// }
@@ -471,23 +505,24 @@ impl VerificationHelper for VHelper {
 /// int
 /// main (int argc, char **argv)
 /// {
-///   pgp_tpk_t tpk;
+///   pgp_cert_t cert;
 ///   pgp_reader_t source;
 ///   pgp_reader_t plaintext;
 ///   uint8_t buf[128];
 ///   ssize_t nread;
+///   pgp_policy_t policy = pgp_standard_policy ();
 ///
-///   tpk = pgp_tpk_from_file (NULL, "../openpgp/tests/data/keys/testy.pgp");
-///   assert(tpk);
+///   cert = pgp_cert_from_file (NULL, "../openpgp/tests/data/keys/testy.pgp");
+///   assert(cert);
 ///
 ///   source = pgp_reader_from_file (
 ///       NULL, "../openpgp/tests/data/messages/signed-1-sha256-testy.gpg");
 ///   assert (source);
 ///
 ///   struct verify_cookie cookie = {
-///     .key = tpk,  /* Move.  */
+///     .key = cert,  /* Move.  */
 ///   };
-///   plaintext = pgp_verifier_new (NULL, source,
+///   plaintext = pgp_verifier_new (NULL, policy, source,
 ///                                 get_public_keys_cb, check_cb,
 ///                                 &cookie, 1554542219);
 ///   assert (source);
@@ -499,11 +534,13 @@ impl VerificationHelper for VHelper {
 ///
 ///   pgp_reader_free (plaintext);
 ///   pgp_reader_free (source);
+///   pgp_policy_free (policy);
 ///   return 0;
 /// }
 /// ```
 #[::sequoia_ffi_macros::extern_fn] #[no_mangle] pub extern "C"
 fn pgp_verifier_new<'a>(errp: Option<&mut *mut crate::error::Error>,
+                        policy: *const Policy,
                         input: *mut io::Reader,
                         get_public_keys: GetPublicKeysCallback,
                         check: CheckCallback,
@@ -511,21 +548,14 @@ fn pgp_verifier_new<'a>(errp: Option<&mut *mut crate::error::Error>,
                         time: time_t)
                         -> Maybe<io::Reader>
 {
+    let policy = policy.ref_raw().as_ref();
     let helper = VHelper::new(get_public_keys, check, cookie);
 
-    Verifier::from_reader(input.ref_mut_raw(), helper, maybe_time(time))
+    Verifier::from_reader(policy, input.ref_mut_raw(), helper, maybe_time(time))
         .map(|r| io::ReaderKind::Generic(Box::new(r)))
         .move_into_raw(errp)
 }
 
-fn maybe_time(t: time_t) -> Option<time::Tm> {
-    if t == 0 {
-        None
-    } else {
-        Some(time::at(time::Timespec::new(t as i64, 0)))
-    }
-}
-
 /// Verifies a detached OpenPGP signature.
 ///
 /// # Example
@@ -542,21 +572,21 @@ fn maybe_time(t: time_t) -> Option<time::Tm> {
 /// #include <sequoia/openpgp.h>
 ///
 /// struct verify_cookie {
-///   pgp_tpk_t key;
+///   pgp_cert_t key;
 /// };
 ///
 /// static pgp_status_t
 /// get_public_keys_cb (void *cookie_opaque,
 ///                     pgp_keyid_t *keyids, size_t keyids_len,
-///                     pgp_tpk_t **tpks, size_t *tpks_len,
+///                     pgp_cert_t **certs, size_t *certs_len,
 ///                     void (**our_free)(void *))
 /// {
-///   /* Feed the TPKs to the verifier here.  */
+///   /* Feed the Certs to the verifier here.  */
 ///   struct verify_cookie *cookie = cookie_opaque;
-///   *tpks = malloc (sizeof (pgp_tpk_t));
-///   assert (*tpks);
-///   *tpks[0] = cookie->key;
-///   *tpks_len = 1;
+///   *certs = malloc (sizeof (pgp_cert_t));
+///   assert (*certs);
+///   *certs[0] = cookie->key;
+///   *certs_len = 1;
 ///   *our_free = free;
 ///   return PGP_STATUS_SUCCESS;
 /// }
@@ -589,16 +619,17 @@ fn maybe_time(t: time_t) -> Option<time::Tm> {
 /// int
 /// main (int argc, char **argv)
 /// {
-///   pgp_tpk_t tpk;
+///   pgp_cert_t cert;
 ///   pgp_reader_t signature;
 ///   pgp_reader_t source;
 ///   pgp_reader_t plaintext;
 ///   uint8_t buf[128];
 ///   ssize_t nread;
+///   pgp_policy_t policy = pgp_standard_policy ();
 ///
-///   tpk = pgp_tpk_from_file (NULL,
+///   cert = pgp_cert_from_file (NULL,
 ///     "../openpgp/tests/data/keys/emmelie-dorothea-dina-samantha-awina-ed25519.pgp");
-///   assert(tpk);
+///   assert(cert);
 ///
 ///   signature = pgp_reader_from_file (
 ///     NULL,
@@ -610,9 +641,9 @@ fn maybe_time(t: time_t) -> Option<time::Tm> {
 ///   assert (source);
 ///
 ///   struct verify_cookie cookie = {
-///     .key = tpk,  /* Move.  */
+///     .key = cert,  /* Move.  */
 ///   };
-///   plaintext = pgp_detached_verifier_new (NULL, signature, source,
+///   plaintext = pgp_detached_verifier_new (NULL, policy, signature, source,
 ///     get_public_keys_cb, check_cb,
 ///     &cookie, 1554542219);
 ///   assert (source);
@@ -625,11 +656,13 @@ fn maybe_time(t: time_t) -> Option<time::Tm> {
 ///   pgp_reader_free (plaintext);
 ///   pgp_reader_free (source);
 ///   pgp_reader_free (signature);
+///   pgp_policy_free (policy);
 ///   return 0;
 /// }
 /// ```
 #[::sequoia_ffi_macros::extern_fn] #[no_mangle] pub extern "C"
 fn pgp_detached_verifier_new<'a>(errp: Option<&mut *mut crate::error::Error>,
+                                 policy: *const Policy,
                                  signature_input: *mut io::Reader,
                                  input: *mut io::Reader,
                                  get_public_keys: GetPublicKeysCallback,
@@ -638,9 +671,11 @@ fn pgp_detached_verifier_new<'a>(errp: Option<&mut *mut crate::error::Error>,
                                  time: time_t)
                                  -> Maybe<io::Reader>
 {
+    let policy = policy.ref_raw().as_ref();
+
     let helper = VHelper::new(get_public_keys, check, cookie);
 
-    DetachedVerifier::from_reader(signature_input.ref_mut_raw(),
+    DetachedVerifier::from_reader(policy, signature_input.ref_mut_raw(),
                                   input.ref_mut_raw(), helper, maybe_time(time))
         .map(|r| io::ReaderKind::Generic(Box::new(r)))
         .move_into_raw(errp)
@@ -670,13 +705,13 @@ impl DHelper {
 }
 
 impl VerificationHelper for DHelper {
-    fn get_public_keys(&mut self, ids: &[openpgp::KeyID])
-        -> Result<Vec<openpgp::TPK>, failure::Error>
+    fn get_public_keys(&mut self, ids: &[openpgp::KeyHandle])
+        -> Result<Vec<openpgp::Cert>, failure::Error>
     {
         self.vhelper.get_public_keys(ids)
     }
 
-    fn check(&mut self, structure: &stream::MessageStructure)
+    fn check(&mut self, structure: stream::MessageStructure)
         -> Result<(), failure::Error>
     {
         self.vhelper.check(structure)
@@ -777,19 +812,19 @@ impl DecryptionHelper for DHelper {
 /// #include <sequoia/openpgp.h>
 ///
 /// struct decrypt_cookie {
-///   pgp_tpk_t key;
+///   pgp_cert_t key;
 ///   int decrypt_called;
 /// };
 ///
 /// static pgp_status_t
 /// get_public_keys_cb (void *cookie_raw,
 ///                     pgp_keyid_t *keyids, size_t keyids_len,
-///                     pgp_tpk_t **tpks, size_t *tpk_len,
+///                     pgp_cert_t **certs, size_t *cert_len,
 ///                     void (**our_free)(void *))
 /// {
-///   /* Feed the TPKs to the verifier here.  */
-///   *tpks = NULL;
-///   *tpk_len = 0;
+///   /* Feed the Certs to the verifier here.  */
+///   *certs = NULL;
+///   *cert_len = 0;
 ///   *our_free = free;
 ///   return PGP_STATUS_SUCCESS;
 /// }
@@ -827,9 +862,9 @@ impl DecryptionHelper for DHelper {
 ///     pgp_pkesk_t pkesk = pkesks[i];
 ///     pgp_keyid_t keyid = pgp_pkesk_recipient (pkesk);
 ///
-///     pgp_tpk_key_iter_t key_iter = pgp_tpk_key_iter_all (cookie->key);
+///     pgp_cert_key_iter_t key_iter = pgp_cert_key_iter (cookie->key);
 ///     pgp_key_t key;
-///     while ((key = pgp_tpk_key_iter_next (key_iter, NULL, NULL))) {
+///     while ((key = pgp_cert_key_iter_next (key_iter))) {
 ///       pgp_keyid_t this_keyid = pgp_key_keyid (key);
 ///       int match = pgp_keyid_equal (this_keyid, keyid);
 ///       pgp_keyid_free (this_keyid);
@@ -837,7 +872,7 @@ impl DecryptionHelper for DHelper {
 ///         break;
 ///       pgp_key_free (key);
 ///     }
-///     pgp_tpk_key_iter_free (key_iter);
+///     pgp_cert_key_iter_free (key_iter);
 ///     pgp_keyid_free (keyid);
 ///     if (! key)
 ///       continue;
@@ -857,7 +892,7 @@ impl DecryptionHelper for DHelper {
 ///     rc = decrypt (decrypt_cookie, algo, sk);
 ///     pgp_session_key_free (sk);
 ///
-///     *identity_out = pgp_tpk_fingerprint (cookie->key);
+///     *identity_out = pgp_cert_fingerprint (cookie->key);
 ///     return rc;
 ///   }
 ///
@@ -867,25 +902,26 @@ impl DecryptionHelper for DHelper {
 /// int
 /// main (int argc, char **argv)
 /// {
-///   pgp_tpk_t tpk;
+///   pgp_cert_t cert;
 ///   pgp_reader_t source;
 ///   pgp_reader_t plaintext;
 ///   uint8_t buf[128];
 ///   ssize_t nread;
+///   pgp_policy_t policy = pgp_standard_policy ();
 ///
-///   tpk = pgp_tpk_from_file (
+///   cert = pgp_cert_from_file (
 ///       NULL, "../openpgp/tests/data/keys/testy-private.pgp");
-///   assert(tpk);
+///   assert(cert);
 ///
 ///   source = pgp_reader_from_file (
 ///       NULL, "../openpgp/tests/data/messages/encrypted-to-testy.gpg");
 ///   assert (source);
 ///
 ///   struct decrypt_cookie cookie = {
-///     .key = tpk,
+///     .key = cert,
 ///     .decrypt_called = 0,
 ///   };
-///   plaintext = pgp_decryptor_new (NULL, source,
+///   plaintext = pgp_decryptor_new (NULL, policy, source,
 ///                                  get_public_keys_cb, decrypt_cb,
 ///                                  check_cb, NULL, &cookie, 1554542219);
 ///   assert (plaintext);
@@ -897,12 +933,14 @@ impl DecryptionHelper for DHelper {
 ///
 ///   pgp_reader_free (plaintext);
 ///   pgp_reader_free (source);
-///   pgp_tpk_free (tpk);
+///   pgp_cert_free (cert);
+///   pgp_policy_free (policy);
 ///   return 0;
 /// }
 /// ```
 #[::sequoia_ffi_macros::extern_fn] #[no_mangle] pub extern "C"
 fn pgp_decryptor_new<'a>(errp: Option<&mut *mut crate::error::Error>,
+                         policy: *const Policy,
                          input: *mut io::Reader,
                          get_public_keys: GetPublicKeysCallback,
                          decrypt: DecryptCallback,
@@ -912,10 +950,11 @@ fn pgp_decryptor_new<'a>(errp: Option<&mut *mut crate::error::Error>,
                          time: time_t)
                          -> Maybe<io::Reader>
 {
+    let policy = policy.ref_raw().as_ref();
     let helper = DHelper::new(
         get_public_keys, decrypt, check, inspect, cookie);
 
-    Decryptor::from_reader(input.ref_mut_raw(), helper, maybe_time(time))
+    Decryptor::from_reader(policy, input.ref_mut_raw(), helper, maybe_time(time))
         .map(|r| io::ReaderKind::Generic(Box::new(r)))
         .move_into_raw(errp)
 }
