@@ -6,14 +6,16 @@ use std::io;
 
 extern crate sequoia_openpgp as openpgp;
 use crate::openpgp::armor;
-use crate::openpgp::constants::DataFormat;
-use crate::openpgp::packet::KeyFlags;
+use crate::openpgp::types::KeyFlags;
 use crate::openpgp::parse::Parse;
 use crate::openpgp::serialize::stream::{
     Message, LiteralWriter, Encryptor,
 };
+use crate::openpgp::policy::StandardPolicy as P;
 
 fn main() {
+    let p = &P::new();
+
     let args: Vec<String> = env::args().collect();
     if args.len() < 3 {
         panic!("A simple encryption filter.\n\n\
@@ -22,24 +24,27 @@ fn main() {
     }
 
     let mode = match args[1].as_ref() {
-        "at-rest" => KeyFlags::default().set_encrypt_at_rest(true),
-        "for-transport" => KeyFlags::default().set_encrypt_for_transport(true),
+        "at-rest" => KeyFlags::default().set_storage_encryption(true),
+        "for-transport" => KeyFlags::default().set_transport_encryption(true),
         x => panic!("invalid mode: {:?}, \
                      must be either 'at-rest' or 'for-transport'",
                     x),
     };
 
-    // Read the transferable public keys from the given files.
-    let tpks: Vec<openpgp::TPK> = args[2..].iter().map(|f| {
-        openpgp::TPK::from_file(f)
+    // Read the certificates from the given files.
+    let certs: Vec<openpgp::Cert> = args[2..].iter().map(|f| {
+        openpgp::Cert::from_file(f)
             .expect("Failed to read key")
     }).collect();
 
     // Build a vector of recipients to hand to Encryptor.
-    let recipients =
-        tpks.iter()
-        .flat_map(|tpk| tpk.keys_valid().key_flags(mode.clone()))
-        .map(|(_, _, key)| key.into())
+    let mut recipients =
+        certs.iter()
+        .flat_map(|cert| {
+            cert.keys()
+                .with_policy(p, None).alive().revoked(false).key_flags(&mode)
+        })
+        .map(|ka| ka.key().into())
         .collect::<Vec<_>>();
 
     // Compose a writer stack corresponding to the output format and
@@ -52,13 +57,14 @@ fn main() {
     let message = Message::new(sink);
 
     // We want to encrypt a literal data packet.
-    let encryptor = Encryptor::new(message,
-                                   &[], // No symmetric encryption.
-                                   &recipients,
-                                   None, None)
-        .expect("Failed to create encryptor");
-    let mut literal_writer = LiteralWriter::new(encryptor, DataFormat::Binary,
-                                                None, None)
+    let mut encryptor = Encryptor::for_recipient(
+        message, recipients.pop().expect("No encryption key found"));
+    for r in recipients {
+        encryptor = encryptor.add_recipient(r)
+    }
+    let encryptor = encryptor.build().expect("Failed to create encryptor");
+
+    let mut literal_writer = LiteralWriter::new(encryptor).build()
         .expect("Failed to create literal writer");
 
     // Copy stdin to our writer stack to encrypt the data.
