@@ -4,64 +4,68 @@ use std::io::{self, Write};
 
 extern crate sequoia_openpgp as openpgp;
 use crate::openpgp::crypto::SessionKey;
-use crate::openpgp::constants::SymmetricAlgorithm;
+use crate::openpgp::types::SymmetricAlgorithm;
 use crate::openpgp::serialize::stream::*;
-use crate::openpgp::packet::KeyFlags;
 use crate::openpgp::parse::stream::*;
+use crate::openpgp::policy::Policy;
+use crate::openpgp::policy::StandardPolicy as P;
 
 const MESSAGE: &'static str = "дружба";
 
 fn main() {
+    let p = &P::new();
+
     // Generate a key.
     let key = generate().unwrap();
 
     // Encrypt the message.
     let mut ciphertext = Vec::new();
-    encrypt(&mut ciphertext, MESSAGE, &key).unwrap();
+    encrypt(p, &mut ciphertext, MESSAGE, &key).unwrap();
 
     // Decrypt the message.
     let mut plaintext = Vec::new();
-    decrypt(&mut plaintext, &ciphertext, &key).unwrap();
+    decrypt(p, &mut plaintext, &ciphertext, &key).unwrap();
 
     assert_eq!(MESSAGE.as_bytes(), &plaintext[..]);
 }
 
 /// Generates an encryption-capable key.
-fn generate() -> openpgp::Result<openpgp::TPK> {
-    let (tpk, _revocation) = openpgp::tpk::TPKBuilder::new()
+fn generate() -> openpgp::Result<openpgp::Cert> {
+    let (cert, _revocation) = openpgp::cert::CertBuilder::new()
         .add_userid("someone@example.org")
-        .add_encryption_subkey()
+        .add_transport_encryption_subkey()
         .generate()?;
 
     // Save the revocation certificate somewhere.
 
-    Ok(tpk)
+    Ok(cert)
 }
 
 /// Encrypts the given message.
-fn encrypt(sink: &mut Write, plaintext: &str, recipient: &openpgp::TPK)
-           -> openpgp::Result<()> {
+fn encrypt(p: &dyn Policy, sink: &mut dyn Write, plaintext: &str,
+           recipient: &openpgp::Cert)
+    -> openpgp::Result<()>
+{
     // Build a vector of recipients to hand to Encryptor.
-    let recipients =
-        recipient.keys_valid()
-        .key_flags(KeyFlags::default()
-                   .set_encrypt_at_rest(true)
-                   .set_encrypt_for_transport(true))
-        .map(|(_, _, key)| key.into())
+    let mut recipients =
+        recipient.keys().with_policy(p, None).alive().revoked(false)
+        .for_transport_encryption()
+        .map(|ka| ka.key().into())
         .collect::<Vec<_>>();
 
     // Start streaming an OpenPGP message.
     let message = Message::new(sink);
 
     // We want to encrypt a literal data packet.
-    let encryptor = Encryptor::new(message,
-                                   &[], // No symmetric encryption.
-                                   &recipients,
-                                   None, None)?;
+    let mut encryptor = Encryptor::for_recipient(
+        message, recipients.pop().expect("No encryption key found"));
+    for r in recipients {
+        encryptor = encryptor.add_recipient(r)
+    }
+    let encryptor = encryptor.build().expect("Failed to create encryptor");
 
     // Emit a literal data packet.
-    let mut literal_writer = LiteralWriter::new(
-        encryptor, openpgp::constants::DataFormat::Binary, None, None)?;
+    let mut literal_writer = LiteralWriter::new(encryptor).build()?;
 
     // Encrypt the data.
     literal_writer.write_all(plaintext.as_bytes())?;
@@ -74,16 +78,18 @@ fn encrypt(sink: &mut Write, plaintext: &str, recipient: &openpgp::TPK)
 }
 
 /// Decrypts the given message.
-fn decrypt(sink: &mut Write, ciphertext: &[u8], recipient: &openpgp::TPK)
+fn decrypt(p: &dyn Policy,
+           sink: &mut dyn Write, ciphertext: &[u8], recipient: &openpgp::Cert)
            -> openpgp::Result<()> {
     // Make a helper that that feeds the recipient's secret key to the
     // decryptor.
     let helper = Helper {
         secret: recipient,
+        policy: p,
     };
 
-    // Now, create a decryptor with a helper using the given TPKs.
-    let mut decryptor = Decryptor::from_bytes(ciphertext, helper, None)?;
+    // Now, create a decryptor with a helper using the given Certs.
+    let mut decryptor = Decryptor::from_bytes(p, ciphertext, helper, None)?;
 
     // Decrypt the data.
     io::copy(&mut decryptor, sink)?;
@@ -92,17 +98,18 @@ fn decrypt(sink: &mut Write, ciphertext: &[u8], recipient: &openpgp::TPK)
 }
 
 struct Helper<'a> {
-    secret: &'a openpgp::TPK,
+    secret: &'a openpgp::Cert,
+    policy: &'a dyn Policy,
 }
 
 impl<'a> VerificationHelper for Helper<'a> {
-    fn get_public_keys(&mut self, _ids: &[openpgp::KeyID])
-                       -> openpgp::Result<Vec<openpgp::TPK>> {
+    fn get_public_keys(&mut self, _ids: &[openpgp::KeyHandle])
+                       -> openpgp::Result<Vec<openpgp::Cert>> {
         // Return public keys for signature verification here.
         Ok(Vec::new())
     }
 
-    fn check(&mut self, _structure: &MessageStructure)
+    fn check(&mut self, _structure: MessageStructure)
              -> openpgp::Result<()> {
         // Implement your signature verification policy here.
         Ok(())
@@ -117,18 +124,16 @@ impl<'a> DecryptionHelper for Helper<'a> {
                   -> openpgp::Result<Option<openpgp::Fingerprint>>
         where D: FnMut(SymmetricAlgorithm, &SessionKey) -> openpgp::Result<()>
     {
-        // The encryption key is the first and only subkey.
-        let key = self.secret.subkeys().nth(0)
-            .map(|binding| binding.key().clone())
-            .unwrap();
+        let key = self.secret.keys().with_policy(self.policy, None)
+            .for_transport_encryption().nth(0).unwrap().key().clone();
 
         // The secret key is not encrypted.
-        let mut pair = key.mark_parts_secret().into_keypair().unwrap();
+        let mut pair = key.mark_parts_secret().unwrap().into_keypair().unwrap();
 
         pkesks[0].decrypt(&mut pair)
             .and_then(|(algo, session_key)| decrypt(algo, &session_key))
             .map(|_| None)
         // XXX: In production code, return the Fingerprint of the
-        // recipient's TPK here
+        // recipient's Cert here
     }
 }
