@@ -6,52 +6,57 @@ extern crate failure;
 extern crate sequoia_openpgp as openpgp;
 use crate::openpgp::serialize::stream::*;
 use crate::openpgp::parse::stream::*;
+use crate::openpgp::policy::Policy;
+use crate::openpgp::policy::StandardPolicy as P;
 
 const MESSAGE: &'static str = "дружба";
 
 fn main() {
+    let p = &P::new();
+
     // Generate a key.
     let key = generate().unwrap();
 
     // Sign the message.
     let mut signed_message = Vec::new();
-    sign(&mut signed_message, MESSAGE, &key).unwrap();
+    sign(p, &mut signed_message, MESSAGE, &key).unwrap();
 
     // Verify the message.
     let mut plaintext = Vec::new();
-    verify(&mut plaintext, &signed_message, &key).unwrap();
+    verify(p, &mut plaintext, &signed_message, &key).unwrap();
 
     assert_eq!(MESSAGE.as_bytes(), &plaintext[..]);
 }
 
 /// Generates an signing-capable key.
-fn generate() -> openpgp::Result<openpgp::TPK> {
-    let (tpk, _revocation) = openpgp::tpk::TPKBuilder::new()
+fn generate() -> openpgp::Result<openpgp::Cert> {
+    let (cert, _revocation) = openpgp::cert::CertBuilder::new()
         .add_userid("someone@example.org")
         .add_signing_subkey()
         .generate()?;
 
     // Save the revocation certificate somewhere.
 
-    Ok(tpk)
+    Ok(cert)
 }
 
 /// Signs the given message.
-fn sign(sink: &mut Write, plaintext: &str, tsk: &openpgp::TPK)
+fn sign(p: &dyn Policy, sink: &mut dyn Write, plaintext: &str, tsk: &openpgp::Cert)
            -> openpgp::Result<()> {
-    // Get the keypair to do the signing from the TPK.
-    let mut keypair = tsk.keys_valid().signing_capable().nth(0).unwrap().2
-        .clone().mark_parts_secret().into_keypair()?;
+    // Get the keypair to do the signing from the Cert.
+    let keypair = tsk
+        .keys().with_policy(p, None).alive().revoked(false).for_signing()
+        .nth(0).unwrap()
+        .key().clone().mark_parts_secret().unwrap().into_keypair()?;
 
     // Start streaming an OpenPGP message.
     let message = Message::new(sink);
 
     // We want to sign a literal data packet.
-    let signer = Signer::new(message, vec![&mut keypair], None)?;
+    let signer = Signer::new(message, keypair).build()?;
 
     // Emit a literal data packet.
-    let mut literal_writer = LiteralWriter::new(
-        signer, openpgp::constants::DataFormat::Binary, None, None)?;
+    let mut literal_writer = LiteralWriter::new(signer).build()?;
 
     // Sign the data.
     literal_writer.write_all(plaintext.as_bytes())?;
@@ -64,16 +69,17 @@ fn sign(sink: &mut Write, plaintext: &str, tsk: &openpgp::TPK)
 }
 
 /// Verifies the given message.
-fn verify(sink: &mut Write, signed_message: &[u8], sender: &openpgp::TPK)
+fn verify(p: &dyn Policy, sink: &mut dyn Write,
+          signed_message: &[u8], sender: &openpgp::Cert)
           -> openpgp::Result<()> {
     // Make a helper that that feeds the sender's public key to the
     // verifier.
     let helper = Helper {
-        tpk: sender,
+        cert: sender,
     };
 
-    // Now, create a verifier with a helper using the given TPKs.
-    let mut verifier = Verifier::from_bytes(signed_message, helper, None)?;
+    // Now, create a verifier with a helper using the given Certs.
+    let mut verifier = Verifier::from_bytes(p, signed_message, helper, None)?;
 
     // Verify the data.
     io::copy(&mut verifier, sink)?;
@@ -82,38 +88,41 @@ fn verify(sink: &mut Write, signed_message: &[u8], sender: &openpgp::TPK)
 }
 
 struct Helper<'a> {
-    tpk: &'a openpgp::TPK,
+    cert: &'a openpgp::Cert,
 }
 
 impl<'a> VerificationHelper for Helper<'a> {
-    fn get_public_keys(&mut self, _ids: &[openpgp::KeyID])
-                       -> openpgp::Result<Vec<openpgp::TPK>> {
+    fn get_public_keys(&mut self, _ids: &[openpgp::KeyHandle])
+                       -> openpgp::Result<Vec<openpgp::Cert>> {
         // Return public keys for signature verification here.
-        Ok(vec![self.tpk.clone()])
+        Ok(vec![self.cert.clone()])
     }
 
-    fn check(&mut self, structure: &MessageStructure)
+    fn check(&mut self, structure: MessageStructure)
              -> openpgp::Result<()> {
         // In this function, we implement our signature verification
         // policy.
 
         let mut good = false;
-        for (i, layer) in structure.iter().enumerate() {
+        for (i, layer) in structure.into_iter().enumerate() {
             match (i, layer) {
                 // First, we are interested in signatures over the
                 // data, i.e. level 0 signatures.
-                (0, MessageLayer::SignatureGroup { ref results }) => {
+                (0, MessageLayer::SignatureGroup { results }) => {
                     // Finally, given a VerificationResult, which only says
                     // whether the signature checks out mathematically, we apply
                     // our policy.
-                    match results.get(0) {
-                        Some(VerificationResult::GoodChecksum(..)) =>
+                    match results.into_iter().next() {
+                        Some(VerificationResult::GoodChecksum { .. }) =>
                             good = true,
-                        Some(VerificationResult::MissingKey(_)) =>
+                        Some(VerificationResult::NotAlive { .. }) =>
+                            return Err(failure::err_msg(
+                                "Signature good, but not alive")),
+                        Some(VerificationResult::MissingKey { .. }) =>
                             return Err(failure::err_msg(
                                 "Missing key to verify signature")),
-                        Some(VerificationResult::BadChecksum(_)) =>
-                            return Err(failure::err_msg("Bad signature")),
+                        Some(VerificationResult::Error { error, .. }) =>
+                            return Err(error),
                         None =>
                             return Err(failure::err_msg("No signature")),
                     }
