@@ -35,9 +35,9 @@ use std::str;
 use std::borrow::Cow;
 use quickcheck::{Arbitrary, Gen};
 
+use crate::vec_truncate;
 use crate::packet::prelude::*;
-use crate::packet::BodyLength;
-use crate::packet::ctb::{CTBNew, CTBOld};
+use crate::packet::header::{BodyLength, CTBNew, CTBOld};
 use crate::serialize::SerializeInto;
 
 /// The encoded output stream must be represented in lines of no more
@@ -57,7 +57,7 @@ pub enum Kind {
     /// validated, in this crate's terminology, this is just a
     /// `PacketPile`.)
     Message,
-    /// A transferable public key.
+    /// A certificate.
     PublicKey,
     /// A transferable secret key.
     SecretKey,
@@ -140,37 +140,39 @@ impl Kind {
 
 /// A filter that applies ASCII Armor to the data written to it.
 pub struct Writer<W: Write> {
-    sink: W,
+    sink: Option<W>,
     kind: Kind,
     stash: Vec<u8>,
     column: usize,
     crc: CRC,
-    epilogue: Vec<u8>,
+    header: Vec<u8>,
     dirty: bool,
-    finalized: bool,
 }
 
 impl<W: Write> Writer<W> {
     /// Constructs a new filter for the given type of data.
     ///
+    /// Note: To ensure that we can handle errors during writing of
+    /// the armor footer, this object must be consumed by calling
+    /// [`Writer::finalize()`].  If the object is dropped without
+    /// being finalized, we will panic in debug builds.
+    ///
+    ///   [`Writer::finalize()`]: #method.finalize
+    ///
     /// # Example
     ///
     /// ```
-    /// # use std::io::Write;
-    /// # extern crate sequoia_openpgp as openpgp;
-    /// # use openpgp::armor::{Writer, Kind};
-    /// # use std::io::{self, Result};
+    /// use std::io::{Read, Write, Cursor};
+    /// use sequoia_openpgp as openpgp;
+    /// use openpgp::armor::{Writer, Kind};
     /// # fn main() { f().unwrap(); }
-    /// # fn f() -> Result<()> {
-    /// let mut buffer = io::Cursor::new(vec![]);
-    /// {
-    ///     let mut writer = Writer::new(&mut buffer, Kind::File,
-    ///         &[ ("Key", "Value") ][..])?;
-    ///     writer.write_all(b"Hello world!")?;
-    ///     // writer is drop()ed here.
-    /// }
+    /// # fn f() -> std::io::Result<()> {
+    /// let mut writer = Writer::new(Vec::new(), Kind::File,
+    ///     &[ ("Key", "Value") ][..])?;
+    /// writer.write_all(b"Hello world!")?;
+    /// let buffer = writer.finalize()?;
     /// assert_eq!(
-    ///     String::from_utf8_lossy(buffer.get_ref()),
+    ///     String::from_utf8_lossy(&buffer),
     ///     "-----BEGIN PGP ARMORED FILE-----
     /// Key: Value
     ///
@@ -183,18 +185,17 @@ impl<W: Write> Writer<W> {
     /// ```
     pub fn new(inner: W, kind: Kind, headers: &[(&str, &str)]) -> Result<Self> {
         let mut w = Writer {
-            sink: inner,
+            sink: Some(inner),
             kind: kind,
             stash: Vec::<u8>::with_capacity(2),
             column: 0,
             crc: CRC::new(),
-            epilogue: Vec::with_capacity(128),
+            header: Vec::with_capacity(128),
             dirty: false,
-            finalized: false,
         };
 
         {
-            let mut cur = Cursor::new(&mut w.epilogue);
+            let mut cur = Cursor::new(&mut w.header);
             write!(&mut cur, "{}{}", kind.begin(), LINE_ENDING)?;
 
             for h in headers {
@@ -208,13 +209,18 @@ impl<W: Write> Writer<W> {
         Ok(w)
     }
 
-    fn write_epilogue(&mut self) -> Result<()> {
+    fn e_finalized() -> Error {
+        Error::new(ErrorKind::BrokenPipe, "Writer is finalized.")
+    }
+
+    fn finalize_headers(&mut self) -> Result<()> {
         if ! self.dirty {
             self.dirty = true;
-            self.sink.write_all(&self.epilogue)?;
+            self.sink.as_mut().ok_or_else(Self::e_finalized)?
+                .write_all(&self.header)?;
             // Release memory.
-            self.epilogue.clear();
-            self.epilogue.shrink_to_fit();
+            crate::vec_truncate(&mut self.header, 0);
+            self.header.shrink_to_fit();
         }
         Ok(())
     }
@@ -222,51 +228,83 @@ impl<W: Write> Writer<W> {
     /// Writes the footer.
     ///
     /// No more data can be written after this call.  If this is not
-    /// called explicitly, the header is written once the writer is
+    /// called explicitly, the footer is written once the writer is
     /// dropped.
-    pub fn finalize(&mut self) -> Result<()> {
-        if self.finalized {
-            return Err(Error::new(ErrorKind::BrokenPipe, "Writer is finalized."));
+    pub fn finalize(mut self) -> Result<W> {
+        if ! self.dirty {
+            // No data was written to us, don't emit anything.
+            return Ok(self.sink.take().ok_or_else(Self::e_finalized)?);
         }
 
+        // We need to clear self.sink if this succeeds or not, so be
+        // careful with the result here.  Otherwise, self.sink may
+        // still be present when the object is really dropped,
+        // invoking finalize_armor again.
+        let r = self.finalize_armor();
+        let sink = self.sink.take();
+        if let Err(e) = r {
+            Err(e)
+        } else {
+            sink.ok_or(Self::e_finalized())
+        }
+    }
+
+    /// Writes the footer.
+    fn finalize_armor(&mut self) -> Result<()> {
         if ! self.dirty {
             // No data was written to us, don't emit anything.
             return Ok(());
         }
-        self.write_epilogue()?;
+        self.finalize_headers()?;
+        if let Some(sink) = self.sink.as_mut() {
+            // Write any stashed bytes and pad.
+            if self.stash.len() > 0 {
+                sink.write_all(base64::encode_config(
+                    &self.stash, base64::STANDARD).as_bytes())?;
+                self.column += 4;
+            }
 
-        // Write any stashed bytes and pad.
-        if self.stash.len() > 0 {
-            self.sink.write_all(base64::encode_config(&self.stash,
-                                                      base64::STANDARD).as_bytes())?;
-            self.column += 4;
+            // Inserts a line break if necessary.
+            //
+            // Unfortunately, we cannot use
+            //self.linebreak()?;
+            //
+            // Therefore, we inline it here.  This is a bit sad.
+            assert!(self.column <= LINE_LENGTH);
+            if self.column == LINE_LENGTH {
+                write!(sink, "{}", LINE_ENDING)?;
+                self.column = 0;
+            }
+
+            if self.column > 0 {
+                write!(sink, "{}", LINE_ENDING)?;
+            }
+
+            let crc = self.crc.finalize();
+            let bytes: [u8; 3] = [
+                (crc >> 16) as u8,
+                (crc >>  8) as u8,
+                (crc >>  0) as u8,
+            ];
+
+            // CRC and footer.
+            write!(sink, "={}{}{}{}",
+                   base64::encode_config(&bytes, base64::STANDARD_NO_PAD),
+                   LINE_ENDING, self.kind.end(), LINE_ENDING)?;
+
+            self.dirty = false;
+            Ok(())
+        } else {
+            Err(Self::e_finalized())
         }
-        self.linebreak()?;
-        if self.column > 0 {
-            write!(self.sink, "{}", LINE_ENDING)?;
-        }
-
-        let crc = self.crc.finalize();
-        let bytes: [u8; 3] = [
-            (crc >> 16) as u8,
-            (crc >>  8) as u8,
-            (crc >>  0) as u8,
-        ];
-
-        // CRC and footer.
-        write!(self.sink, "={}{}{}{}",
-               base64::encode_config(&bytes, base64::STANDARD_NO_PAD),
-               LINE_ENDING, self.kind.end(), LINE_ENDING)?;
-
-        self.finalized = true;
-        Ok(())
     }
 
     /// Inserts a line break if necessary.
     fn linebreak(&mut self) -> Result<()> {
         assert!(self.column <= LINE_LENGTH);
         if self.column == LINE_LENGTH {
-            write!(self.sink, "{}", LINE_ENDING)?;
+            write!(self.sink.as_mut().ok_or_else(Self::e_finalized)?,
+                   "{}", LINE_ENDING)?;
             self.column = 0;
         }
         Ok(())
@@ -275,11 +313,7 @@ impl<W: Write> Writer<W> {
 
 impl<W: Write> Write for Writer<W> {
     fn write(&mut self, buf: &[u8]) -> Result<usize> {
-        if self.finalized {
-            return Err(Error::new(ErrorKind::BrokenPipe, "Writer is finalized."));
-        }
-
-        self.write_epilogue()?;
+        self.finalize_headers()?;
 
         // Update CRC on the unencoded data.
         self.crc.update(buf);
@@ -307,11 +341,12 @@ impl<W: Write> Write for Writer<W> {
 
             // If this fails for some reason, and the caller retries
             // the write, we might end up with a stash of size 3.
-            self.sink.write_all(base64::encode_config(&self.stash,
-                                                      base64::STANDARD_NO_PAD).as_bytes())?;
+            self.sink.as_mut().ok_or_else(Self::e_finalized)?
+                .write_all(base64::encode_config(
+                    &self.stash, base64::STANDARD_NO_PAD).as_bytes())?;
             self.column += 4;
             self.linebreak()?;
-            self.stash.clear();
+            crate::vec_truncate(&mut self.stash, 0);
         }
 
         // Ensure that a multiple of 3 bytes are encoded, stash the
@@ -332,7 +367,8 @@ impl<W: Write> Write for Writer<W> {
         let mut enc = encoded.as_bytes();
         while enc.len() > 0 {
             let n = cmp::min(LINE_LENGTH - self.column, enc.len());
-            self.sink.write_all(&enc[..n])?;
+            self.sink.as_mut().ok_or_else(Self::e_finalized)?
+                .write_all(&enc[..n])?;
             enc = &enc[n..];
             self.column += n;
             self.linebreak()?;
@@ -343,13 +379,16 @@ impl<W: Write> Write for Writer<W> {
     }
 
     fn flush(&mut self) -> Result<()> {
-        self.sink.flush()
+        self.sink.as_mut().ok_or_else(Self::e_finalized)?.flush()
     }
 }
 
 impl<W: Write> Drop for Writer<W> {
     fn drop(&mut self) {
-        let _ = self.finalize();
+        debug_assert!(self.sink.is_none(),
+                      "armor writer dropped without being finalized, \
+                       use armor::Writer::finalize()");
+        let _ = self.finalize_armor();
     }
 }
 
@@ -386,7 +425,7 @@ pub enum ReaderMode {
 
 /// A filter that strips ASCII Armor from a stream of data.
 pub struct Reader<'a> {
-    source: Box<'a + BufferedReader<()>>,
+    source: Box<dyn BufferedReader<()> + 'a>,
     kind: Option<Kind>,
     mode: ReaderMode,
     buffer: Vec<u8>,
@@ -395,6 +434,8 @@ pub struct Reader<'a> {
     initialized: bool,
     headers: Vec<(String, String)>,
     finalized: bool,
+    prefix_len: usize,
+    prefix_remaining: usize,
 }
 
 impl Default for ReaderMode {
@@ -442,7 +483,7 @@ impl<'a> Reader<'a> {
     /// reader.read_to_end(&mut buf)?;
     ///
     /// let message = Message::from_bytes(&buf)?;
-    /// assert_eq!(message.body().unwrap().body().unwrap(),
+    /// assert_eq!(message.body().unwrap().body(),
     ///            b"Hello world!");
     /// # Ok(())
     /// # }
@@ -513,7 +554,7 @@ impl<'a> Reader<'a> {
     }
 
     pub(crate) fn from_buffered_reader<C: 'a, M>(
-        inner: Box<'a + BufferedReader<C>>, mode: M) -> Self
+        inner: Box<dyn BufferedReader<C> + 'a>, mode: M) -> Self
         where M: Into<Option<ReaderMode>>
     {
         let mode = mode.into().unwrap_or(Default::default());
@@ -528,6 +569,8 @@ impl<'a> Reader<'a> {
             headers: Vec::new(),
             initialized: false,
             finalized: false,
+            prefix_len: 0,
+            prefix_remaining: 0,
         }
     }
 
@@ -562,10 +605,10 @@ impl<'a> Reader<'a> {
         lazy_static!{
             static ref START_CHARS : Vec<u8> = {
                 let mut valid_start = Vec::new();
-                for &tag in [ Tag::PKESK, Tag::SKESK,
+                for &tag in &[ Tag::PKESK, Tag::SKESK,
                               Tag::OnePassSig, Tag::Signature,
                               Tag::PublicKey, Tag::SecretKey,
-                              Tag::CompressedData, Tag::Literal ].into_iter() {
+                              Tag::CompressedData, Tag::Literal ] {
                     let mut ctb = [ 0u8; 1 ];
                     let mut o = [ 0u8; 4 ];
 
@@ -601,15 +644,31 @@ impl<'a> Reader<'a> {
         };
 
         let mut lines = 0;
+        let mut prefix = Vec::new();
         let n = 'search: loop {
             if lines > 0 {
                 // Find the start of the next line.
-                self.source.drop_through(&[b'\n'])?;
+                self.source.drop_through(&[b'\n'], true)?;
+                crate::vec_truncate(&mut prefix, 0);
             }
             lines += 1;
 
             // Ignore leading whitespace, etc.
-            while self.source.data_hard(1)?[0].is_ascii_whitespace() {
+            while match self.source.data_hard(1)?[0] {
+                // Skip some whitespace (previously .is_ascii_whitespace())
+                b' ' | b'\t' | b'\r' | b'\n' => true,
+                // Also skip common quote characters
+                b'>' | b'|' | b']' | b'}' => true,
+                // Do not skip anything else
+                _ => false,
+            } {
+                let c = self.source.data(1)?[0];
+                if c == b'\n' {
+                    // We found a newline while walking whitespace, reset prefix
+                    crate::vec_truncate(&mut prefix, 0);
+                } else {
+                    prefix.push(self.source.data_hard(1)?[0]);
+                }
                 self.source.consume(1);
             }
 
@@ -670,6 +729,8 @@ impl<'a> Reader<'a> {
         if found_blob {
             // Skip the rest of the initialization.
             self.initialized = true;
+            self.prefix_len = prefix.len();
+            self.prefix_remaining = prefix.len();
             return Ok(());
         }
 
@@ -686,10 +747,29 @@ impl<'a> Reader<'a> {
         };
         self.source.consume(n);
 
+        let next_prefix = &self.source.data_hard(prefix.len())?[..prefix.len()];
+        if prefix != next_prefix {
+            // If the next line doesn't start with the same prefix, we assume
+            // it was garbage on the front and drop the prefix so long as it
+            // was purely whitespace.  Any non-whitespace remains an error
+            // while searching for the armor header if it's not repeated.
+            if prefix.iter().all(|b| (*b as char).is_ascii_whitespace()) {
+                crate::vec_truncate(&mut prefix, 0);
+            } else {
+                // Nope, we have actually failed to read this properly
+                return Err(
+                    Error::new(ErrorKind::InvalidInput,
+                               "Reached EOF looking for Armor Header Line"));
+            }
+        }
+
         // Read the key-value headers.
         let mut n = 0;
         let mut lines = 0;
         loop {
+            // Skip any known prefix on lines
+            self.source.consume(prefix.len());
+
             self.source.consume(n);
 
             let line = self.source.read_to('\n' as u8)?;
@@ -743,6 +823,8 @@ impl<'a> Reader<'a> {
         self.source.consume(n);
 
         self.initialized = true;
+        self.prefix_len = prefix.len();
+        self.prefix_remaining = prefix.len();
         Ok(())
     }
 }
@@ -768,8 +850,9 @@ impl<'a> Reader<'a> {
 //
 // This function will stop after it sees base64 padding, and if it
 // sees invalid base64 data.
-fn base64_filter(mut bytes: Cow<[u8]>, base64_data_max: usize)
-    -> (Cow<[u8]>, usize)
+fn base64_filter(mut bytes: Cow<[u8]>, base64_data_max: usize,
+                 mut prefix_remaining: usize, prefix_len: usize)
+    -> (Cow<[u8]>, usize, usize)
 {
     let mut leading_whitespace = 0;
 
@@ -795,9 +878,31 @@ fn base64_filter(mut bytes: Cow<[u8]>, base64_data_max: usize)
         // A valid base64 chunk never starts with padding.
         && ! (padding > 0 && base64_len % 4 == 0)
     {
+        // If we have some prefix to skip, skip it.
+        if prefix_remaining > 0 {
+            prefix_remaining -= 1;
+            if unfiltered_offset == 0 {
+                match bytes {
+                    Cow::Borrowed(s) => {
+                        // We're at the beginning.  Avoid moving
+                        // data by cutting off the start of the
+                        // slice.
+                        bytes = Cow::Borrowed(&s[1..]);
+                        leading_whitespace += 1;
+                        continue;
+                    }
+                    Cow::Owned(_) => (),
+                }
+            }
+            unfiltered_offset += 1;
+            continue;
+        }
         match bytes[unfiltered_offset] {
             // White space.
             c if c.is_ascii_whitespace() => {
+                if c == b'\n' {
+                    prefix_remaining = prefix_len;
+                }
                 if unfiltered_offset == 0 {
                     match bytes {
                         Cow::Borrowed(s) => {
@@ -862,10 +967,11 @@ fn base64_filter(mut bytes: Cow<[u8]>, base64_data_max: usize)
     unfiltered_complete_len += leading_whitespace;
     match bytes {
         Cow::Borrowed(s) =>
-            (Cow::Borrowed(&s[..base64_len]), unfiltered_complete_len),
+            (Cow::Borrowed(&s[..base64_len]), unfiltered_complete_len,
+             prefix_remaining),
         Cow::Owned(mut v) => {
-            v.truncate(base64_len);
-            (Cow::Owned(v), unfiltered_complete_len)
+            vec_truncate(&mut v, base64_len);
+            (Cow::Owned(v), unfiltered_complete_len, prefix_remaining)
         }
     }
 }
@@ -875,7 +981,7 @@ fn is_armored_pgp_blob(bytes: &[u8]) -> bool {
     // Get up to 32 bytes of base64 data.  That's 24 bytes of data
     // (ignoring padding), which is more than enough to get the first
     // packet's header.
-    let (bytes, _) = base64_filter(Cow::Borrowed(bytes), 32);
+    let (bytes, _, _) = base64_filter(Cow::Borrowed(bytes), 32, 0, 0);
 
     match base64::decode_config(&bytes, base64::STANDARD) {
         Ok(d) => {
@@ -885,7 +991,7 @@ fn is_armored_pgp_blob(bytes: &[u8]) -> bool {
             } else {
                 let mut br = buffered_reader::Memory::new(&d);
                 if let Ok(header) = Header::parse(&mut br) {
-                    header.ctb.tag.valid_start_of_message()
+                    header.ctb().tag().valid_start_of_message()
                         && header.valid(false).is_ok()
                 } else {
                     false
@@ -925,6 +1031,14 @@ impl<'a> Read for Reader<'a> {
             self.initialize()?;
         }
 
+        if buf.len() == 0 {
+            // Short-circuit here.  Otherwise, we copy 0 bytes into
+            // the buffer, which means we decoded 0 bytes, and we
+            // wrongfully assume that we reached the end of the
+            // armored block.
+            return Ok(0);
+        }
+
         if self.finalized {
             assert_eq!(self.buffer.len(), 0);
             return Ok(0);
@@ -935,7 +1049,7 @@ impl<'a> Read for Reader<'a> {
 
             let amount = cmp::min(buf.len(), self.buffer.len());
             buf[..amount].copy_from_slice(&self.buffer[..amount]);
-            self.buffer.drain(..amount);
+            crate::vec_drain_prefix(&mut self.buffer, amount);
 
             (0, amount)
         } else {
@@ -985,12 +1099,14 @@ impl<'a> Read for Reader<'a> {
                 base64data
             };
 
-            let (base64data, consumed)
+            let (base64data, consumed, prefix_remaining)
                 = base64_filter(Cow::Borrowed(base64data),
                                 // base64_size rounds up, but we want
                                 // to round down as we have to double
                                 // buffer partial chunks.
-                                cmp::max(THRESHOLD, buf.len() / 3 * 4));
+                                cmp::max(THRESHOLD, buf.len() / 3 * 4),
+                                self.prefix_remaining,
+                                self.prefix_len);
 
             // We shouldn't have any partial chunks.
             assert_eq!(base64data.len() % 4, 0);
@@ -1008,7 +1124,7 @@ impl<'a> Read for Reader<'a> {
 
                 let copied = cmp::min(buf.len(), self.buffer.len());
                 buf[..copied].copy_from_slice(&self.buffer[..copied]);
-                self.buffer.drain(..copied);
+                crate::vec_drain_prefix(&mut self.buffer, copied);
 
                 copied
             } else {
@@ -1023,11 +1139,12 @@ impl<'a> Read for Reader<'a> {
                 decoded
             };
 
+            self.prefix_remaining = prefix_remaining;
+
             (consumed, decoded)
         };
 
         self.source.consume(consumed);
-
         if decoded == 0 {
             self.finalized = true;
 
@@ -1073,6 +1190,8 @@ impl<'a> Read for Reader<'a> {
             };
             self.source.consume(consumed);
 
+            // Skip any expected prefix
+            self.source.consume(self.prefix_len);
             // Look for a footer.
             let consumed = {
                 // Skip whitespace.
@@ -1211,12 +1330,11 @@ mod test {
     #[test]
     fn enarmor() {
         for (bin, asc) in TEST_BIN.iter().zip(TEST_ASC.iter()) {
-            let mut buf = Vec::new();
-            {
-                let mut w = Writer::new(&mut buf, Kind::File, &[]).unwrap();
-                w.write(&[]).unwrap();  // Avoid zero-length optimization.
-                w.write_all(bin).unwrap();
-            }
+            let mut w =
+                Writer::new(Vec::new(), Kind::File, &[]).unwrap();
+            w.write(&[]).unwrap();  // Avoid zero-length optimization.
+            w.write_all(bin).unwrap();
+            let buf = w.finalize().unwrap();
             assert_eq!(String::from_utf8_lossy(&buf),
                        String::from_utf8_lossy(asc));
         }
@@ -1225,14 +1343,12 @@ mod test {
     #[test]
     fn enarmor_bytewise() {
         for (bin, asc) in TEST_BIN.iter().zip(TEST_ASC.iter()) {
-            let mut buf = Vec::new();
-            {
-                let mut w = Writer::new(&mut buf, Kind::File, &[]).unwrap();
-                w.write(&[]).unwrap();  // Avoid zero-length optimization.
-                for b in bin.iter() {
-                    w.write(&[*b]).unwrap();
-                }
+            let mut w = Writer::new(Vec::new(), Kind::File, &[]).unwrap();
+            w.write(&[]).unwrap();  // Avoid zero-length optimization.
+            for b in bin.iter() {
+                w.write(&[*b]).unwrap();
             }
+            let buf = w.finalize().unwrap();
             assert_eq!(String::from_utf8_lossy(&buf),
                        String::from_utf8_lossy(asc));
         }
@@ -1242,19 +1358,14 @@ mod test {
     fn drop_writer() {
         // No ASCII frame shall be emitted if the writer is dropped
         // unused.
-        let mut buf = Vec::new();
-        {
-            drop(Writer::new(&mut buf, Kind::File, &[]).unwrap());
-        }
-        assert!(buf.is_empty());
+        assert!(Writer::new(Vec::new(), Kind::File, &[]).unwrap()
+                .finalize().unwrap().is_empty());
 
         // However, if the user insists, we will encode a zero-byte
         // string.
-        let mut buf = Vec::new();
-        {
-            let mut w = Writer::new(&mut buf, Kind::File, &[]).unwrap();
-            w.write(&[]).unwrap();
-        }
+        let mut w = Writer::new(Vec::new(), Kind::File, &[]).unwrap();
+        w.write(&[]).unwrap();
+        let buf = w.finalize().unwrap();
         assert_eq!(
             &buf[..],
             &b"-----BEGIN PGP ARMORED FILE-----\n\
@@ -1439,6 +1550,51 @@ mod test {
         }
     }
 
+    #[test]
+    fn dearmor_quoted() {
+        let mut r = Reader::new(
+            Cursor::new(
+                &include_bytes!("../tests/data/armor/test-3.with-headers-quoted.asc")[..]
+            ),
+            ReaderMode::VeryTolerant);
+        let mut buf = [0; 5];
+        let e = r.read(&mut buf);
+        assert!(r.kind() == Some(Kind::File));
+        assert!(e.is_ok());
+    }
+
+    #[test]
+    fn dearmor_quoted_a_lot() {
+        let mut r = Reader::new(
+            Cursor::new(
+                &include_bytes!("../tests/data/armor/test-3.with-headers-quoted-a-lot.asc")[..]
+            ),
+            ReaderMode::VeryTolerant);
+        let mut buf = [0; 5];
+        // Loop over the input to ensure we read and verify all the way to the
+        // end of the input in order to check the checksum and footer validation
+        loop {
+            let e = r.read(&mut buf);
+            assert!(r.kind() == Some(Kind::File));
+            assert!(e.is_ok());
+            if e.unwrap() == 0 {
+                break;
+            }
+        }
+    }
+
+    #[test]
+    fn dearmor_quoted_badly() {
+        let mut r = Reader::new(
+            Cursor::new(
+                &include_bytes!("../tests/data/armor/test-3.with-headers-quoted-badly.asc")[..]
+            ),
+            ReaderMode::VeryTolerant);
+        let mut buf = [0; 5];
+        let e = r.read(&mut buf);
+        assert!(e.is_err());
+    }
+
     quickcheck! {
         fn roundtrip(kind: Kind, payload: Vec<u8>) -> bool {
             if payload.is_empty() {
@@ -1448,10 +1604,9 @@ mod test {
                 return true;
             }
 
-            let mut encoded = Vec::new();
-            Writer::new(&mut encoded, kind, &[]).unwrap()
-                .write_all(&payload)
-                .unwrap();
+            let mut w = Writer::new(Vec::new(), kind, &[]).unwrap();
+            w.write_all(&payload).unwrap();
+            let encoded = w.finalize().unwrap();
 
             let mut recovered = Vec::new();
             Reader::new(Cursor::new(&encoded),
@@ -1467,4 +1622,14 @@ mod test {
             payload == recovered && payload == recovered_any
         }
     }
+
+    /// Tests issue #404, zero-sized reads break reader.
+    #[test]
+    fn zero_sized_read() {
+        let mut r = Reader::from_bytes(crate::tests::file("armor/test-1.asc"),
+                                       None);
+        let mut buf = Vec::new();
+        r.read(&mut buf).unwrap();
+        r.read(&mut buf).unwrap();
+    }
 }
