@@ -1,21 +1,28 @@
 use std::cmp;
 use std::fmt;
-use std::io::{self, Read};
+use std::io;
 
 use nettle::{aead, cipher};
 use buffered_reader::BufferedReader;
 
-use crate::constants::{
+use crate::types::{
     AEADAlgorithm,
     SymmetricAlgorithm,
 };
-use crate::conversions::{
+use crate::utils::{
     write_be_u64,
 };
 use crate::Error;
 use crate::Result;
 use crate::crypto::SessionKey;
 use crate::crypto::mem::secure_cmp;
+use crate::parse::Cookie;
+
+/// Disables authentication checks.
+///
+/// This is DANGEROUS, and is only useful for debugging problems with
+/// malformed AEAD-encrypted messages.
+const DANGER_DISABLE_AUTHENTICATION: bool = false;
 
 impl AEADAlgorithm {
     /// Returns the digest size of the AEAD algorithm.
@@ -25,6 +32,9 @@ impl AEADAlgorithm {
             &EAX =>
             // Digest size is independent of the cipher.
                 Ok(aead::Eax::<cipher::Aes128>::DIGEST_SIZE),
+            &OCB =>
+            // According to RFC4880bis, Section 5.16.2.
+                Ok(16),
             _ => Err(Error::UnsupportedAEADAlgorithm(self.clone()).into()),
         }
     }
@@ -35,13 +45,18 @@ impl AEADAlgorithm {
         match self {
             &EAX =>
                 Ok(16), // According to RFC4880bis, Section 5.16.1.
+            &OCB =>
+            // According to RFC4880bis, Section 5.16.2, the IV is "at
+            // least 15 octets long".  GnuPG hardcodes 15 in
+            // openpgp_aead_algo_info.
+                Ok(15),
             _ => Err(Error::UnsupportedAEADAlgorithm(self.clone()).into()),
         }
     }
 
     /// Creates a nettle context.
     pub fn context(&self, sym_algo: SymmetricAlgorithm, key: &[u8], nonce: &[u8])
-                   -> Result<Box<aead::Aead>> {
+                   -> Result<Box<dyn aead::Aead>> {
         match self {
             AEADAlgorithm::EAX => match sym_algo {
                 SymmetricAlgorithm::AES128 =>
@@ -77,9 +92,9 @@ impl AEADAlgorithm {
 const AD_PREFIX_LEN: usize = 5;
 
 /// A `Read`er for decrypting AEAD-encrypted data.
-pub struct Decryptor<R: io::Read> {
+pub struct Decryptor<'a> {
     // The encrypted data.
-    source: R,
+    source: Box<dyn BufferedReader<Cookie> + 'a>,
 
     sym_algo: SymmetricAlgorithm,
     aead: AEADAlgorithm,
@@ -95,13 +110,28 @@ pub struct Decryptor<R: io::Read> {
     buffer: Vec<u8>,
 }
 
-impl<R: io::Read> Decryptor<R> {
+impl<'a> Decryptor<'a> {
     /// Instantiate a new AEAD decryptor.
     ///
     /// `source` is the source to wrap.
-    pub fn new(version: u8, sym_algo: SymmetricAlgorithm, aead: AEADAlgorithm,
-               chunk_size: usize, iv: &[u8], key: &SessionKey, source: R)
-               -> Result<Self> {
+    pub fn new<R: io::Read>(version: u8, sym_algo: SymmetricAlgorithm,
+                            aead: AEADAlgorithm, chunk_size: usize,
+                            iv: &[u8], key: &SessionKey, source: R)
+        -> Result<Self>
+        where R: 'a
+    {
+        Self::from_buffered_reader(
+            version, sym_algo, aead, chunk_size, iv, key,
+            Box::new(buffered_reader::Generic::with_cookie(
+                source, None, Default::default())))
+    }
+
+    fn from_buffered_reader(version: u8, sym_algo: SymmetricAlgorithm,
+                            aead: AEADAlgorithm, chunk_size: usize,
+                            iv: &[u8], key: &SessionKey,
+                            source: Box<dyn 'a + BufferedReader<Cookie>>)
+        -> Result<Self>
+    {
         Ok(Decryptor {
             source: source,
             sym_algo: sym_algo,
@@ -125,7 +155,7 @@ impl<R: io::Read> Decryptor<R> {
         })
     }
 
-    fn hash_associated_data(&mut self, aead: &mut Box<aead::Aead>,
+    fn hash_associated_data(&mut self, aead: &mut Box<dyn aead::Aead>,
                             final_digest: bool) {
         // Prepare the associated data.
         write_be_u64(&mut self.ad[AD_PREFIX_LEN..AD_PREFIX_LEN + 8],
@@ -140,7 +170,7 @@ impl<R: io::Read> Decryptor<R> {
         }
     }
 
-    fn make_aead(&mut self) -> Result<Box<aead::Aead>> {
+    fn make_aead(&mut self) -> Result<Box<dyn aead::Aead>> {
         // The chunk index is XORed into the IV.
         let mut chunk_index_be64 = vec![0u8; 8];
         write_be_u64(&mut chunk_index_be64, self.chunk_index);
@@ -177,191 +207,177 @@ impl<R: io::Read> Decryptor<R> {
         }
     }
 
+    // Note: this implementation tries *very* hard to make sure we don't
+    // gratuitiously do a short read.  Specifically, if the return value
+    // is less than `plaintext.len()`, then it is either because we
+    // reached the end of the input or an error occurred.
     fn read_helper(&mut self, plaintext: &mut [u8]) -> Result<usize> {
         use std::cmp::Ordering;
 
         let mut pos = 0;
 
+        // Buffer to hold a digest.
+        let mut digest = vec![0u8; self.digest_size];
+
         // 1. Copy any buffered data.
         if self.buffer.len() > 0 {
             let to_copy = cmp::min(self.buffer.len(), plaintext.len());
             &plaintext[..to_copy].copy_from_slice(&self.buffer[..to_copy]);
-            self.buffer.drain(..to_copy);
+            crate::vec_drain_prefix(&mut self.buffer, to_copy);
+
             pos = to_copy;
+            if pos == plaintext.len() {
+                return Ok(pos);
+            }
         }
 
-        if pos == plaintext.len() {
-            return Ok(pos);
-        }
+        // 2. Decrypt the data a chunk at a time until we've filled
+        // `plaintext`.
+        //
+        // Unfortunately, framing is hard.
+        //
+        // Recall: AEAD data is of the form:
+        //
+        //   [ chunk1 ][ tag1 ] ... [ chunkN ][ tagN ][ tagF ]
+        //
+        // And, all chunks are the same size except for the last
+        // chunk, which may be shorter.
+        //
+        // The naive approach to decryption is to read a chunk and a
+        // tag at a time.  Unfortunately, this may not work if the
+        // last chunk is a partial chunk.
+        //
+        // Assume that the chunk size is 32 bytes and the digest size
+        // is 16 bytes, and consider a message with 17 bytes of data.
+        // That message will be encrypted as follows:
+        //
+        //   [ chunk1 ][ tag1 ][ tagF ]
+        //       17B     16B     16B
+        //
+        // If we read a chunk and a digest, we'll successfully read 48
+        // bytes of data.  Unfortunately, we'll have over read: the
+        // last 15 bytes are from the final tag.
+        //
+        // To correctly handle this case, we have to make sure that
+        // there are at least a tag worth of bytes left over when we
+        // read a chunk and a tag.
 
-        // 2. Decrypt as many whole chunks as `plaintext` can hold.
-        let n_chunks = (plaintext.len() - pos) / self.chunk_size;
+        let n_chunks
+            = (plaintext.len() - pos + self.chunk_size - 1) / self.chunk_size;
         let chunk_digest_size = self.chunk_size + self.digest_size;
-        let mut to_copy = n_chunks * self.chunk_size;
-        let to_read =     n_chunks * chunk_digest_size;
+        let final_digest_size = self.digest_size;
 
-        let mut ciphertext = Vec::new();
-        let result = (&mut self.source).take(to_read as u64)
-            .read_to_end(&mut ciphertext);
-        let short_read;
-        match result {
-            Ok(amount) => {
-                if to_read != 0 && amount == 0 {
-                    // Exhausted source.
-                    return Ok(pos);
-                }
-
-                short_read = amount < to_copy;
-                to_copy = amount;
-                ciphertext.truncate(to_copy);
-            },
-            // We encountered an error, but we did read some.
-            Err(_) if pos > 0 => return Ok(pos),
-            Err(e) => return Err(e.into()),
-        }
-
-        // Buffer to hold digests.
-        let mut digest = vec![0u8; self.digest_size];
-
-        // At the end of the stream, there is an additional tag.  Be
-        // careful not to consume this tag.
-        let ciphertext_end = if short_read {
-            ciphertext.len() - self.digest_size
-        } else {
-            ciphertext.len()
-        };
-
-        for chunk in (&ciphertext[..ciphertext_end]).chunks(chunk_digest_size) {
+        for _ in 0..n_chunks {
             let mut aead = self.make_aead()?;
-
             // Digest the associated data.
             self.hash_associated_data(&mut aead, false);
 
-            // Decrypt the chunk.
-            aead.decrypt(
-                &mut plaintext[pos..pos + chunk.len() - self.digest_size],
-                &chunk[..chunk.len() - self.digest_size]);
-            self.bytes_decrypted += (chunk.len() - self.digest_size) as u64;
+            // Do a little dance to avoid exclusively locking
+            // `self.source`.
+            let to_read = chunk_digest_size + final_digest_size;
+            let result = {
+                match self.source.data(to_read) {
+                    Ok(_) => Ok(self.source.buffer()),
+                    Err(err) => Err(err),
+                }
+            };
 
-            // Check digest.
-            aead.digest(&mut digest);
-            let dig_ord = secure_cmp(&digest[..],
-                                     &chunk[chunk.len() - self.digest_size..]);
-            if dig_ord != Ordering::Equal {
+            let check_final_tag;
+            let chunk = match result {
+                Ok(chunk) => {
+                    if chunk.len() == 0 {
+                        // Exhausted source.
+                        return Ok(pos);
+                    }
+
+                    if chunk.len() < final_digest_size {
+                        return Err(Error::ManipulatedMessage.into());
+                    }
+
+                    check_final_tag = chunk.len() < to_read;
+
+                    // Return the chunk.
+                    &chunk[..cmp::min(chunk.len(), to_read) - final_digest_size]
+                },
+                Err(e) => return Err(e.into()),
+            };
+
+            assert!(chunk.len() <= chunk_digest_size);
+
+            if chunk.len() == 0 {
+                // There is nothing to decrypt: all that is left is
+                // the final tag.
+            } else if chunk.len() <= self.digest_size {
+                // A chunk has to include at least one byte and a tag.
                 return Err(Error::ManipulatedMessage.into());
-            }
+            } else {
+                // Decrypt the chunk and check the tag.
+                let to_decrypt = chunk.len() - self.digest_size;
 
-            // Increase index, update position in plaintext.
-            self.chunk_index += 1;
-            pos += chunk.len() - self.digest_size;
-        }
+                // If plaintext doesn't have enough room for the whole
+                // chunk, then we have to double buffer.
+                let double_buffer = to_decrypt > plaintext.len() - pos;
+                let buffer = if double_buffer {
+                    self.buffer.resize(to_decrypt, 0);
+                    &mut self.buffer[..]
+                } else {
+                    &mut plaintext[pos..pos + to_decrypt]
+                };
 
-        if short_read {
-            // We read the whole ciphertext, now check the final digest.
-            let mut aead = self.make_aead()?;
-            self.hash_associated_data(&mut aead, true);
+                aead.decrypt(buffer, &chunk[..to_decrypt]);
 
-            let mut nada = [0; 0];
-            aead.decrypt(&mut nada, b"");
-            aead.digest(&mut digest);
-
-            let dig_ord = secure_cmp(&digest[..], &ciphertext[ciphertext_end..]);
-            if dig_ord != Ordering::Equal {
-                return Err(Error::ManipulatedMessage.into());
-            }
-        }
-
-        if short_read || pos == plaintext.len() {
-            return Ok(pos);
-        }
-
-        // 3. The last bit is a partial chunk.  Buffer it.
-        let mut to_copy = plaintext.len() - pos;
-        assert!(0 < to_copy);
-        assert!(to_copy < self.chunk_size);
-
-        let mut ciphertext = Vec::new();
-        let result = (&mut self.source).take(chunk_digest_size as u64)
-            .read_to_end(&mut ciphertext);
-        let short_read;
-        match result {
-            Ok(amount) => {
-                if amount == 0 {
-                    return Ok(pos);
+                // Check digest.
+                aead.digest(&mut digest);
+                if secure_cmp(&digest[..], &chunk[to_decrypt..])
+                    != Ordering::Equal && ! DANGER_DISABLE_AUTHENTICATION
+                {
+                    return Err(Error::ManipulatedMessage.into());
                 }
 
-                short_read = amount < chunk_digest_size;
+                if double_buffer {
+                    let to_copy = plaintext.len() - pos;
+                    assert!(0 < to_copy);
+                    assert!(to_copy < self.chunk_size);
 
-                // Make sure `ciphertext` is not larger than the
-                // amount of data that was actually read.
-                ciphertext.truncate(amount);
+                    &plaintext[pos..pos + to_copy]
+                        .copy_from_slice(&self.buffer[..to_copy]);
+                    crate::vec_drain_prefix(&mut self.buffer, to_copy);
+                    pos += to_copy;
+                } else {
+                    pos += to_decrypt;
+                }
 
-                // Make sure we don't read more than is available.
-                to_copy = cmp::min(to_copy,
-                                   ciphertext.len() - self.digest_size
-                                   - if short_read {
-                                       self.digest_size
-                                   } else {
-                                       0
-                                   });
-            },
-            // We encountered an error, but we did read some.
-            Err(_) if pos > 0 => return Ok(pos),
-            Err(e) => return Err(e.into()),
-        }
-        assert!(ciphertext.len() <= self.chunk_size + self.digest_size);
+                // Increase index, update position in plaintext.
+                self.chunk_index += 1;
+                self.bytes_decrypted += to_decrypt as u64;
 
-        let mut aead = self.make_aead()?;
+                // Consume the data only on success so that we keep
+                // returning the error.
+                let chunk_len = chunk.len();
+                self.source.consume(chunk_len);
+            }
 
-        // Digest the associated data.
-        self.hash_associated_data(&mut aead, false);
+            if check_final_tag {
+                // We read the whole ciphertext, now check the final digest.
+                let mut aead = self.make_aead()?;
+                self.hash_associated_data(&mut aead, true);
 
-        // At the end of the stream, there is an additional tag.  Be
-        // careful not to consume this tag.
-        let ciphertext_end = if short_read {
-            ciphertext.len() - self.digest_size
-        } else {
-            ciphertext.len()
-        };
+                let mut nada = [0; 0];
+                aead.decrypt(&mut nada, b"");
+                aead.digest(&mut digest);
 
-        while self.buffer.len() < ciphertext_end - self.digest_size {
-            self.buffer.push(0u8);
-        }
-        self.buffer.truncate(ciphertext_end - self.digest_size);
+                let final_digest = self.source.data(final_digest_size)?;
+                if final_digest.len() != final_digest_size
+                    || secure_cmp(&digest[..], final_digest) != Ordering::Equal
+                    && ! DANGER_DISABLE_AUTHENTICATION
+                {
+                    return Err(Error::ManipulatedMessage.into());
+                }
 
-        // Decrypt the chunk.
-        aead.decrypt(&mut self.buffer,
-                     &ciphertext[..ciphertext_end - self.digest_size]);
-        self.bytes_decrypted += (ciphertext_end - self.digest_size) as u64;
-
-        // Check digest.
-        aead.digest(&mut digest);
-        let mac_ord = secure_cmp(
-            &digest[..],
-            &ciphertext[ciphertext_end - self.digest_size..ciphertext_end]);
-        if mac_ord != Ordering::Equal {
-            return Err(Error::ManipulatedMessage.into());
-        }
-
-        // Increase index.
-        self.chunk_index += 1;
-
-        &plaintext[pos..pos + to_copy].copy_from_slice(&self.buffer[..to_copy]);
-        self.buffer.drain(..to_copy);
-        pos += to_copy;
-
-        if short_read {
-            // We read the whole ciphertext, now check the final digest.
-            let mut aead = self.make_aead()?;
-            self.hash_associated_data(&mut aead, true);
-
-            let mut nada = [0; 0];
-            aead.decrypt(&mut nada, b"");
-            aead.digest(&mut digest);
-
-            let dig_ord = secure_cmp(&digest[..], &ciphertext[ciphertext_end..]);
-            if dig_ord != Ordering::Equal {
-                return Err(Error::ManipulatedMessage.into());
+                // Consume the data only on success so that we keep
+                // returning the error.
+                self.source.consume(final_digest_size);
+                break;
             }
         }
 
@@ -372,8 +388,8 @@ impl<R: io::Read> Decryptor<R> {
 // Note: this implementation tries *very* hard to make sure we don't
 // gratuitiously do a short read.  Specifically, if the return value
 // is less than `plaintext.len()`, then it is either because we
-// reached the end of the input or an error occured.
-impl<R: io::Read> io::Read for Decryptor<R> {
+// reached the end of the input or an error occurred.
+impl<'a> io::Read for Decryptor<'a> {
     fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
         match self.read_helper(buf) {
             Ok(n) => Ok(n),
@@ -390,17 +406,18 @@ impl<R: io::Read> io::Read for Decryptor<R> {
 
 /// A `BufferedReader` that decrypts AEAD-encrypted data as it is
 /// read.
-pub(crate) struct BufferedReaderDecryptor<R: BufferedReader<C>, C> {
-    reader: buffered_reader::Generic<Decryptor<R>, C>,
+pub(crate) struct BufferedReaderDecryptor<'a> {
+    reader: buffered_reader::Generic<Decryptor<'a>, Cookie>,
 }
 
-impl <R: BufferedReader<C>, C> BufferedReaderDecryptor<R, C> {
+impl<'a> BufferedReaderDecryptor<'a> {
     /// Like `new()`, but sets a cookie, which can be retrieved using
     /// the `cookie_ref` and `cookie_mut` methods, and set using
     /// the `cookie_set` method.
     pub fn with_cookie(version: u8, sym_algo: SymmetricAlgorithm,
                        aead: AEADAlgorithm, chunk_size: usize, iv: &[u8],
-                       key: &SessionKey, source: R, cookie: C)
+                       key: &SessionKey, source: Box<dyn BufferedReader<Cookie> + 'a>,
+                       cookie: Cookie)
         -> Result<Self>
     {
         Ok(BufferedReaderDecryptor {
@@ -412,19 +429,19 @@ impl <R: BufferedReader<C>, C> BufferedReaderDecryptor<R, C> {
     }
 }
 
-impl<R: BufferedReader<C>, C> io::Read for BufferedReaderDecryptor<R, C> {
+impl<'a> io::Read for BufferedReaderDecryptor<'a> {
     fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
         self.reader.read(buf)
     }
 }
 
-impl<R: BufferedReader<C>, C> fmt::Display for BufferedReaderDecryptor<R, C> {
+impl<'a> fmt::Display for BufferedReaderDecryptor<'a> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(f, "BufferedReaderDecryptor")
     }
 }
 
-impl<R: BufferedReader<C>, C> fmt::Debug for BufferedReaderDecryptor<R, C> {
+impl<'a> fmt::Debug for BufferedReaderDecryptor<'a> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.debug_struct("BufferedReaderDecryptor")
             .field("reader", &self.get_ref().unwrap())
@@ -432,8 +449,7 @@ impl<R: BufferedReader<C>, C> fmt::Debug for BufferedReaderDecryptor<R, C> {
     }
 }
 
-impl<R: BufferedReader<C>, C> BufferedReader<C>
-        for BufferedReaderDecryptor<R, C> {
+impl<'a> BufferedReader<Cookie> for BufferedReaderDecryptor<'a> {
     fn buffer(&self) -> &[u8] {
         return self.reader.buffer();
     }
@@ -479,28 +495,28 @@ impl<R: BufferedReader<C>, C> BufferedReader<C>
         return self.reader.steal_eof();
     }
 
-    fn get_mut(&mut self) -> Option<&mut BufferedReader<C>> {
+    fn get_mut(&mut self) -> Option<&mut dyn BufferedReader<Cookie>> {
         Some(&mut self.reader.reader.source)
     }
 
-    fn get_ref(&self) -> Option<&BufferedReader<C>> {
+    fn get_ref(&self) -> Option<&dyn BufferedReader<Cookie>> {
         Some(&self.reader.reader.source)
     }
 
     fn into_inner<'b>(self: Box<Self>)
-            -> Option<Box<BufferedReader<C> + 'b>> where Self: 'b {
-        Some(Box::new(self.reader.reader.source))
+            -> Option<Box<dyn BufferedReader<Cookie> + 'b>> where Self: 'b {
+        Some(self.reader.reader.source.as_boxed())
     }
 
-    fn cookie_set(&mut self, cookie: C) -> C {
+    fn cookie_set(&mut self, cookie: Cookie) -> Cookie {
         self.reader.cookie_set(cookie)
     }
 
-    fn cookie_ref(&self) -> &C {
+    fn cookie_ref(&self) -> &Cookie {
         self.reader.cookie_ref()
     }
 
-    fn cookie_mut(&mut self) -> &mut C {
+    fn cookie_mut(&mut self) -> &mut Cookie {
         self.reader.cookie_mut()
     }
 }
@@ -558,7 +574,7 @@ impl<W: io::Write> Encryptor<W> {
         })
     }
 
-    fn hash_associated_data(&mut self, aead: &mut Box<aead::Aead>,
+    fn hash_associated_data(&mut self, aead: &mut Box<dyn aead::Aead>,
                             final_digest: bool) {
         // Prepare the associated data.
         write_be_u64(&mut self.ad[AD_PREFIX_LEN..AD_PREFIX_LEN + 8],
@@ -573,7 +589,7 @@ impl<W: io::Write> Encryptor<W> {
         }
     }
 
-    fn make_aead(&mut self) -> Result<Box<aead::Aead>> {
+    fn make_aead(&mut self) -> Result<Box<dyn aead::Aead>> {
         // The chunk index is XORed into the IV.
         let mut chunk_index_be64 = vec![0u8; 8];
         write_be_u64(&mut chunk_index_be64, self.chunk_index);
@@ -636,7 +652,7 @@ impl<W: io::Write> Encryptor<W> {
                 aead.encrypt(&mut self.scratch, &self.buffer);
                 self.bytes_encrypted += self.scratch.len() as u64;
                 self.chunk_index += 1;
-                self.buffer.clear();
+                crate::vec_truncate(&mut self.buffer, 0);
                 inner.write_all(&self.scratch)?;
 
                 // Write digest.
@@ -685,22 +701,23 @@ impl<W: io::Write> Encryptor<W> {
                 aead.encrypt(&mut self.scratch, &self.buffer);
                 self.bytes_encrypted += self.scratch.len() as u64;
                 self.chunk_index += 1;
-                self.buffer.clear();
+                crate::vec_truncate(&mut self.buffer, 0);
                 inner.write_all(&self.scratch)?;
 
                 // Write digest.
                 unsafe { self.scratch.set_len(self.digest_size) }
                 aead.digest(&mut self.scratch[..self.digest_size]);
                 inner.write_all(&self.scratch[..self.digest_size])?;
-
-                // Write final digest.
-                let mut aead = self.make_aead()?;
-                self.hash_associated_data(&mut aead, true);
-                let mut nada = [0; 0];
-                aead.encrypt(&mut nada, b"");
-                aead.digest(&mut self.scratch[..self.digest_size]);
-                inner.write_all(&self.scratch[..self.digest_size])?;
             }
+
+            // Write final digest.
+            let mut aead = self.make_aead()?;
+            self.hash_associated_data(&mut aead, true);
+            let mut nada = [0; 0];
+            aead.encrypt(&mut nada, b"");
+            aead.digest(&mut self.scratch[..self.digest_size]);
+            inner.write_all(&self.scratch[..self.digest_size])?;
+
             Ok(inner)
         } else {
             Err(io::Error::new(io::ErrorKind::BrokenPipe,
