@@ -1,18 +1,19 @@
 //! Elliptic Curve Diffie-Hellman.
 
+use crate::vec_truncate;
 use crate::Error;
 use crate::packet::{
     Key,
     key,
 };
 use crate::Result;
-use crate::constants::{
+use crate::types::{
     Curve,
     HashAlgorithm,
     SymmetricAlgorithm,
     PublicKeyAlgorithm,
 };
-use crate::conversions::{
+use crate::utils::{
     write_be_u64,
     read_be_u64,
 };
@@ -426,7 +427,7 @@ pub fn pkcs5_unpad(sk: Protected, target_len: usize) -> Result<Protected> {
     }
 
     if good {
-        buf.truncate(target_len);
+        vec_truncate(&mut buf, target_len);
         Ok(buf.into())
     } else {
         let sk: Protected = buf.into();
@@ -458,7 +459,7 @@ pub fn aes_key_wrap(algo: SymmetricAlgorithm, key: &Protected,
     // nettle::Mode:ECB, and we need nettle::Mode for polymorphism (we
     // cannot have Box<nettle::Cipher>).  To work around this, we use
     // CBC, and always use an all-zero IV.
-    let mut cipher: Box<Mode> = match algo {
+    let mut cipher: Box<dyn Mode> = match algo {
         AES128 => Box::new(
             mode::Cbc::<cipher::Aes128>::with_encrypt_key(key)?),
         AES192 => Box::new(
@@ -544,7 +545,7 @@ pub fn aes_key_unwrap(algo: SymmetricAlgorithm, key: &Protected,
     // nettle::Mode:ECB, and we need nettle::Mode for polymorphism (we
     // cannot have Box<nettle::Cipher>).  To work around this, we use
     // CBC, and always use an all-zero IV.
-    let mut cipher: Box<Mode> = match algo {
+    let mut cipher: Box<dyn Mode> = match algo {
         AES128 => Box::new(
             mode::Cbc::<cipher::Aes128>::with_decrypt_key(key)?),
         AES192 => Box::new(
