@@ -1,6 +1,9 @@
 //! Functionality to hash packets, and generate hashes.
 
+use std::convert::TryFrom;
+
 use crate::HashAlgorithm;
+use crate::packet::Key;
 use crate::packet::UserID;
 use crate::packet::UserAttribute;
 use crate::packet::key;
@@ -9,10 +12,7 @@ use crate::packet::Signature;
 use crate::packet::signature::{self, Signature4};
 use crate::Error;
 use crate::Result;
-use crate::conversions::Time;
-
-use nettle;
-use nettle::Hash as NettleHash;
+use crate::types::Timestamp;
 
 use std::fs::{File, OpenOptions};
 use std::io::{self, Write};
@@ -23,17 +23,25 @@ const DUMP_HASHED_VALUES: Option<&str> = None;
 
 /// State of a hash function.
 #[derive(Clone)]
-pub struct Context(Box<nettle::Hash>);
+pub struct Context {
+    algo: HashAlgorithm,
+    ctx: Box<dyn nettle::Hash>,
+}
 
 impl Context {
+    /// Returns the algorithm.
+    pub fn algo(&self) -> HashAlgorithm {
+        self.algo
+    }
+
     /// Size of the digest in bytes
     pub fn digest_size(&self) -> usize {
-        self.0.digest_size()
+        self.ctx.digest_size()
     }
 
     /// Writes data into the hash function.
     pub fn update<D: AsRef<[u8]>>(&mut self, data: D) {
-        self.0.update(data.as_ref());
+        self.ctx.update(data.as_ref());
     }
 
     /// Finalizes the hash function and writes the digest into the
@@ -44,7 +52,7 @@ impl Context {
     /// `digest` must be at least `self.digest_size()` bytes large,
     /// otherwise the digest will be truncated.
     pub fn digest<D: AsMut<[u8]>>(&mut self, mut digest: D) {
-        self.0.digest(digest.as_mut());
+        self.ctx.digest(digest.as_mut());
     }
 }
 
@@ -75,7 +83,7 @@ impl HashAlgorithm {
         }
     }
 
-    /// Creates a new Nettle hash context for this algorith.
+    /// Creates a new Nettle hash context for this algorithm.
     ///
     /// # Errors
     ///
@@ -85,10 +93,10 @@ impl HashAlgorithm {
     ///
     ///   [`HashAlgorithm::is_supported`]: #method.is_supported
     pub fn context(self) -> Result<Context> {
-        use nettle::hash::*;
+        use nettle::hash::{Sha224, Sha256, Sha384, Sha512};
         use nettle::hash::insecure_do_not_use::Sha1;
 
-        let c: Result<Box<nettle::Hash>> = match self {
+        let c: Result<Box<dyn nettle::Hash>> = match self {
             HashAlgorithm::SHA1 => Ok(Box::new(Sha1::default())),
             HashAlgorithm::SHA224 => Ok(Box::new(Sha224::default())),
             HashAlgorithm::SHA256 => Ok(Box::new(Sha256::default())),
@@ -101,11 +109,14 @@ impl HashAlgorithm {
         };
 
         if let Some(prefix) = DUMP_HASHED_VALUES {
-            c.map(|c: Box<nettle::Hash>| {
-                Context(Box::new(HashDumper::new(c, prefix)))
+            c.map(|c: Box<dyn nettle::Hash>| {
+                Context {
+                    algo: self,
+                    ctx: Box::new(HashDumper::new(c, prefix)),
+                }
             })
         } else {
-            c.map(|c| Context(c))
+            c.map(|c| Context { algo: self, ctx: c })
         }
     }
 
@@ -128,14 +139,14 @@ impl HashAlgorithm {
 }
 
 struct HashDumper {
-    h: Box<nettle::Hash>,
+    h: Box<dyn nettle::Hash>,
     sink: File,
     filename: String,
     written: usize,
 }
 
 impl HashDumper {
-    fn new(h: Box<nettle::Hash>, prefix: &str) -> Self {
+    fn new(h: Box<dyn nettle::Hash>, prefix: &str) -> Self {
         let mut n = 0;
         let mut filename;
         let sink = loop {
@@ -176,7 +187,7 @@ impl nettle::Hash for HashDumper {
     fn digest(&mut self, digest: &mut [u8]) {
         self.h.digest(digest);
     }
-    fn box_clone(&self) -> Box<nettle::Hash> {
+    fn box_clone(&self) -> Box<dyn nettle::Hash> {
         Box::new(Self::new(self.h.box_clone(), &DUMP_HASHED_VALUES.unwrap()))
     }
 }
@@ -244,8 +255,10 @@ impl<P, R> Hash for Key4<P, R>
         header.push(4);
 
         // Creation time.
-        let creation_time = self.creation_time().to_pgp()
-            .unwrap_or(0);
+        let creation_time: u32 =
+            Timestamp::try_from(self.creation_time())
+            .unwrap_or_else(|_| Timestamp::try_from(0).unwrap())
+            .into();
         header.push((creation_time >> 24) as u8);
         header.push((creation_time >> 16) as u8);
         header.push((creation_time >> 8) as u8);
@@ -266,6 +279,7 @@ impl Hash for Signature {
     fn hash(&self, hash: &mut Context) {
         match self {
             Signature::V4(sig) => sig.hash(hash),
+            Signature::__Nonexhaustive => unreachable!(),
         }
     }
 }
@@ -280,6 +294,12 @@ impl Hash for Signature4 {
 impl Hash for signature::Builder {
     /// Adds the `Signature` to the provided hash context.
     fn hash(&self, hash: &mut Context) {
+        use crate::serialize::SerializeInto;
+        // XXX: Annoyingly, we have no proper way of handling errors
+        // here.
+        let hashed_area = self.hashed_area().to_vec()
+            .unwrap_or_else(|_| Vec::new());
+
         // A version 4 signature packet is laid out as follows:
         //
         //   version - 1 byte                    \
@@ -299,13 +319,12 @@ impl Hash for signature::Builder {
         header[3] = self.hash_algo().into();
 
         // The length of the hashed area, as a 16-bit endian number.
-        let len = self.hashed_area().data.len();
+        let len = hashed_area.len();
         header[4] = (len >> 8) as u8;
         header[5] = len as u8;
 
         hash.update(&header[..]);
-
-        hash.update(&self.hashed_area().data[..]);
+        hash.update(&hashed_area);
 
         // A version 4 signature trailer is:
         //
@@ -324,7 +343,7 @@ impl Hash for signature::Builder {
         trailer[1] = 0xff;
         // The signature packet's length, not including the previous
         // two bytes and the length.
-        let len = header.len() + self.hashed_area().data.len();
+        let len = header.len() + hashed_area.len();
         trailer[2] = (len >> 24) as u8;
         trailer[3] = (len >> 16) as u8;
         trailer[4] = (len >> 8) as u8;
@@ -337,7 +356,7 @@ impl Hash for signature::Builder {
 /// Hashing-related functionality.
 impl Signature {
     /// Computes the message digest of standalone signatures.
-    pub fn standalone_hash<'a, S>(sig: S) -> Result<Vec<u8>>
+    pub fn hash_standalone<'a, S>(sig: S) -> Result<Vec<u8>>
         where S: Into<&'a signature::Builder>
     {
         let sig = sig.into();
@@ -351,17 +370,18 @@ impl Signature {
     }
 
     /// Computes the message digest of timestamp signatures.
-    pub fn timestamp_hash<'a, S>(sig: S) -> Result<Vec<u8>>
+    pub fn hash_timestamp<'a, S>(sig: S) -> Result<Vec<u8>>
         where S: Into<&'a signature::Builder>
     {
-        Self::standalone_hash(sig)
+        Self::hash_standalone(sig)
     }
 
-    /// Returns the message digest of the primary key binding over the
-    /// specified primary key.
-    pub fn primary_key_binding_hash<'a, S>(sig: S, key: &key::PublicKey)
+    /// Returns the message digest of the direct key signature over
+    /// the specified primary key.
+    pub fn hash_direct_key<'a, P, S>(sig: S, key: &Key<P, key::PrimaryRole>)
         -> Result<Vec<u8>>
-        where S: Into<&'a signature::Builder>
+        where P: key::KeyParts,
+              S: Into<&'a signature::Builder>,
     {
 
         let sig = sig.into();
@@ -377,11 +397,14 @@ impl Signature {
 
     /// Returns the message digest of the subkey binding over the
     /// specified primary key and subkey.
-    pub fn subkey_binding_hash<'a, S>(sig: S,
-                                      key: &key::PublicKey,
-                                      subkey: &key::PublicSubkey)
+    pub fn hash_subkey_binding<'a, P, Q, S>(
+        sig: S,
+        key: &Key<P, key::PrimaryRole>,
+        subkey: &Key<Q, key::SubordinateRole>)
         -> Result<Vec<u8>>
-        where S: Into<&'a signature::Builder>
+        where P: key::KeyParts,
+              Q: key::KeyParts,
+              S: Into<&'a signature::Builder>
     {
 
         let sig = sig.into();
@@ -396,13 +419,28 @@ impl Signature {
         Ok(digest)
     }
 
+    /// Returns the message digest of the primary key binding over the
+    /// specified primary key and subkey.
+    pub fn hash_primary_key_binding<'a, P, Q, S>(
+        sig: S,
+        key: &Key<P, key::PrimaryRole>,
+        subkey: &Key<Q, key::SubordinateRole>)
+        -> Result<Vec<u8>>
+        where P: key::KeyParts,
+              Q: key::KeyParts,
+              S: Into<&'a signature::Builder>
+    {
+        Self::hash_subkey_binding(sig.into(), key, subkey)
+    }
+
     /// Returns the message digest of the user ID binding over the
     /// specified primary key, user ID, and signature.
-    pub fn userid_binding_hash<'a, S>(sig: S,
-                                      key: &key::PublicKey,
-                                      userid: &UserID)
+    pub fn hash_userid_binding<'a, P, S>(sig: S,
+                                         key: &Key<P, key::PrimaryRole>,
+                                         userid: &UserID)
         -> Result<Vec<u8>>
-        where S: Into<&'a signature::Builder>
+        where P: key::KeyParts,
+              S: Into<&'a signature::Builder>
     {
         let sig = sig.into();
         let mut h = sig.hash_algo().context()?;
@@ -418,11 +456,13 @@ impl Signature {
 
     /// Returns the message digest of the user attribute binding over
     /// the specified primary key, user attribute, and signature.
-    pub fn user_attribute_binding_hash<'a, S>(sig: S,
-                                              key: &key::PublicKey,
-                                              ua: &UserAttribute)
+    pub fn hash_user_attribute_binding<'a, P, S>(
+        sig: S,
+        key: &Key<P, key::PrimaryRole>,
+        ua: &UserAttribute)
         -> Result<Vec<u8>>
-        where S: Into<&'a signature::Builder>
+        where P: key::KeyParts,
+              S: Into<&'a signature::Builder>,
     {
 
         let sig = sig.into();
@@ -441,57 +481,58 @@ impl Signature {
 #[cfg(test)]
 mod test {
     use super::*;
-    use crate::TPK;
+    use crate::Cert;
     use crate::parse::Parse;
 
     #[test]
     fn hash_verification() {
-        fn check(tpk: TPK) -> (usize, usize, usize) {
+        fn check(cert: Cert) -> (usize, usize, usize) {
             let mut userid_sigs = 0;
-            for (i, binding) in tpk.userids().enumerate() {
-                for selfsig in binding.selfsigs() {
-                    let h = Signature::userid_binding_hash(
+            for (i, binding) in cert.userids().bundles().enumerate() {
+                for selfsig in binding.self_signatures() {
+                    let h = Signature::hash_userid_binding(
                         selfsig,
-                        tpk.primary().key(),
+                        cert.primary_key().key(),
                         binding.userid()).unwrap();
-                    if &h[..2] != selfsig.hash_prefix() {
+                    if &h[..2] != selfsig.digest_prefix() {
                         eprintln!("{:?}: {:?} / {:?}",
                                   i, binding.userid(), selfsig);
                         eprintln!("  Hash: {:?}", h);
                     }
-                    assert_eq!(&h[..2], selfsig.hash_prefix());
+                    assert_eq!(&h[..2], selfsig.digest_prefix());
                     userid_sigs += 1;
                 }
             }
             let mut ua_sigs = 0;
-            for (i, binding) in tpk.user_attributes().enumerate() {
-                for selfsig in binding.selfsigs() {
-                    let h = Signature::user_attribute_binding_hash(
+            for (i, binding) in cert.user_attributes().bundles().enumerate()
+            {
+                for selfsig in binding.self_signatures() {
+                    let h = Signature::hash_user_attribute_binding(
                         selfsig,
-                        tpk.primary().key(),
+                        cert.primary_key().key(),
                         binding.user_attribute()).unwrap();
-                    if &h[..2] != selfsig.hash_prefix() {
+                    if &h[..2] != selfsig.digest_prefix() {
                         eprintln!("{:?}: {:?} / {:?}",
                                   i, binding.user_attribute(), selfsig);
                         eprintln!("  Hash: {:?}", h);
                     }
-                    assert_eq!(&h[..2], selfsig.hash_prefix());
+                    assert_eq!(&h[..2], selfsig.digest_prefix());
                     ua_sigs += 1;
                 }
             }
             let mut subkey_sigs = 0;
-            for (i, binding) in tpk.subkeys().enumerate() {
-                for selfsig in binding.selfsigs() {
-                    let h = Signature::subkey_binding_hash(
+            for (i, binding) in cert.subkeys().enumerate() {
+                for selfsig in binding.self_signatures() {
+                    let h = Signature::hash_subkey_binding(
                         selfsig,
-                        tpk.primary().key(),
+                        cert.primary_key().key(),
                         binding.key()).unwrap();
-                    if &h[..2] != selfsig.hash_prefix() {
+                    if &h[..2] != selfsig.digest_prefix() {
                         eprintln!("{:?}: {:?}", i, binding);
                         eprintln!("  Hash: {:?}", h);
                     }
-                    assert_eq!(h[0], selfsig.hash_prefix()[0]);
-                    assert_eq!(h[1], selfsig.hash_prefix()[1]);
+                    assert_eq!(h[0], selfsig.digest_prefix()[0]);
+                    assert_eq!(h[1], selfsig.digest_prefix()[1]);
                     subkey_sigs += 1;
                 }
             }
@@ -499,13 +540,13 @@ mod test {
             (userid_sigs, ua_sigs, subkey_sigs)
         }
 
-        check(TPK::from_bytes(crate::tests::key("hash-algos/SHA224.gpg")).unwrap());
-        check(TPK::from_bytes(crate::tests::key("hash-algos/SHA256.gpg")).unwrap());
-        check(TPK::from_bytes(crate::tests::key("hash-algos/SHA384.gpg")).unwrap());
-        check(TPK::from_bytes(crate::tests::key("hash-algos/SHA512.gpg")).unwrap());
-        check(TPK::from_bytes(crate::tests::key("bannon-all-uids-subkeys.gpg")).unwrap());
+        check(Cert::from_bytes(crate::tests::key("hash-algos/SHA224.gpg")).unwrap());
+        check(Cert::from_bytes(crate::tests::key("hash-algos/SHA256.gpg")).unwrap());
+        check(Cert::from_bytes(crate::tests::key("hash-algos/SHA384.gpg")).unwrap());
+        check(Cert::from_bytes(crate::tests::key("hash-algos/SHA512.gpg")).unwrap());
+        check(Cert::from_bytes(crate::tests::key("bannon-all-uids-subkeys.gpg")).unwrap());
         let (_userid_sigs, ua_sigs, _subkey_sigs)
-            = check(TPK::from_bytes(crate::tests::key("dkg.gpg")).unwrap());
+            = check(Cert::from_bytes(crate::tests::key("dkg.gpg")).unwrap());
         assert!(ua_sigs > 0);
     }
 }
