@@ -2,7 +2,7 @@ use std::fmt;
 
 use crate::Error;
 use crate::Result;
-use crate::constants::{Curve, HashAlgorithm};
+use crate::types::{Curve, HashAlgorithm};
 use crate::crypto::mpis::{MPI, PublicKey};
 
 /// A proprietary, protocol agnostic identifier for public keys.
@@ -40,7 +40,7 @@ impl std::str::FromStr for Keygrip {
 impl Keygrip {
     /// Parses a keygrip.
     pub fn from_hex(hex: &str) -> Result<Self> {
-        let bytes = crate::conversions::from_hex(hex, true)?;
+        let bytes = crate::fmt::from_hex(hex, true)?;
         if bytes.len() != 20 {
             return Err(Error::InvalidArgument(
                 format!("Expected 20 bytes, got {}", bytes.len())).into());
@@ -212,13 +212,13 @@ fn ecc_param(curve: &Curve, i: usize) -> MPI {
         (_, _) => unreachable!(),
     };
 
-    crate::conversions::from_hex(hex, true).unwrap().into()
+    crate::fmt::from_hex(hex, true).unwrap().into()
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::conversions::from_hex;
+    use crate::fmt::from_hex;
 
     /// Test vectors from libgcrypt/tests/basic.c.
     #[test]
@@ -327,7 +327,7 @@ mod tests {
              KG::from_hex("8F669049015534649776D0F1F439D37EE3F3D948").unwrap()),
         ].iter().cloned().collect();
 
-        for (name, tpk) in [
+        for (name, cert) in [
             "testy.pgp",
             "neal.pgp",
             "dennis-simon-anton.pgp",
@@ -337,13 +337,13 @@ mod tests {
             "erika-corinna-daniela-simone-antonia-nistp384.pgp",
             "erika-corinna-daniela-simone-antonia-nistp521.pgp",
         ]
-            .iter().map(|n| (n, crate::TPK::from_bytes(crate::tests::key(n)).unwrap()))
+            .iter().map(|n| (n, crate::Cert::from_bytes(crate::tests::key(n)).unwrap()))
         {
             eprintln!("{}", name);
-            for key in tpk.keys_all() {
-                let fp = key.2.fingerprint();
+            for key in cert.keys().map(|a| a.key()) {
+                let fp = key.fingerprint();
                 eprintln!("(sub)key: {}", fp);
-                assert_eq!(&key.2.mpis().keygrip().unwrap(),
+                assert_eq!(&key.mpis().keygrip().unwrap(),
                            keygrips.get(&fp).unwrap());
             }
         }
