@@ -6,7 +6,7 @@ use std::cmp::Ordering;
 use quickcheck::{Arbitrary, Gen};
 use rand::Rng;
 
-use crate::constants::{
+use crate::types::{
     Curve,
     HashAlgorithm,
     PublicKeyAlgorithm,
@@ -20,7 +20,7 @@ use crate::Error;
 use crate::Result;
 
 /// Holds a single MPI.
-#[derive(Clone, Hash)]
+#[derive(Clone)]
 pub struct MPI {
     /// Integer value as big-endian.
     value: Box<[u8]>,
@@ -170,7 +170,7 @@ impl fmt::Debug for MPI {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.write_fmt(format_args!(
             "{} bits: {}", self.bits(),
-            crate::conversions::to_hex(&*self.value, true)))
+            crate::fmt::to_hex(&*self.value, true)))
     }
 }
 
@@ -216,10 +216,16 @@ impl PartialEq for MPI {
 
 impl Eq for MPI {}
 
+impl std::hash::Hash for MPI {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        self.value.hash(state);
+    }
+}
+
 /// Holds a single MPI containing secrets.
 ///
 /// The memory will be cleared when the object is dropped.
-#[derive(Clone, Hash)]
+#[derive(Clone)]
 pub struct ProtectedMPI {
     /// Integer value as big-endian.
     value: Protected,
@@ -245,6 +251,12 @@ impl From<MPI> for ProtectedMPI {
     }
 }
 
+impl std::hash::Hash for ProtectedMPI {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        self.value.hash(state);
+    }
+}
+
 impl ProtectedMPI {
     /// Returns the length of the MPI in bits.
     pub fn bits(&self) -> usize {
@@ -264,7 +276,7 @@ impl fmt::Debug for ProtectedMPI {
         if cfg!(debug_assertions) {
             f.write_fmt(format_args!(
                 "{} bits: {}", self.bits(),
-                crate::conversions::to_hex(&*self.value, true)))
+                crate::fmt::to_hex(&*self.value, true)))
         } else {
             f.write_str("<Redacted>")
         }
@@ -481,6 +493,8 @@ impl Arbitrary for PublicKey {
 ///
 /// Provides a typed and structured way of storing multiple MPIs in
 /// packets.
+// Deriving Hash here is okay: PartialEq is manually implemented to
+// ensure that secrets are compared in constant-time.
 #[derive(Clone, Hash)]
 pub enum SecretKeyMaterial {
     /// RSA secret key.
@@ -1046,10 +1060,10 @@ mod tests {
             ("erika-corinna-daniela-simone-antonia-nistp384.pgp", 0, 384),
             ("erika-corinna-daniela-simone-antonia-nistp521.pgp", 0, 521),
         ] {
-            let tpk = crate::TPK::from_bytes(crate::tests::key(name)).unwrap();
-            let key = tpk.keys_all().nth(*key_no).unwrap().2;
-            assert_eq!(key.mpis().bits().unwrap(), *bits,
-                       "TPK {}, key no {}", name, *key_no);
+            let cert = crate::Cert::from_bytes(crate::tests::key(name)).unwrap();
+            let ka = cert.keys().nth(*key_no).unwrap();
+            assert_eq!(ka.key().mpis().bits().unwrap(), *bits,
+                       "Cert {}, key no {}", name, *key_no);
         }
     }
 
