@@ -74,12 +74,12 @@ impl Default for S2K {
 }
 
 impl S2K {
-    /// Convert the string to a key using the S2K's paramters.
-    pub fn derive_key(&self, string: &Password, key_size: usize)
+    /// Convert the string to a key using the S2K's parameters.
+    pub fn derive_key(&self, password: &Password, key_size: usize)
     -> Result<SessionKey> {
         match self {
             &S2K::Simple { hash } | &S2K::Salted { hash, .. }
-            | &S2K::Iterated { hash, .. } => {
+            | &S2K::Iterated { hash, .. } => password.map(|string| {
                 let mut hash = hash.context()?;
 
                 // If the digest length is shorter than the key length,
@@ -143,7 +143,7 @@ impl S2K {
                 }
 
                 Ok(ret.into())
-            }
+            }),
             &S2K::Unknown(u) | &S2K::Private(u) =>
                 Err(Error::MalformedPacket(
                         format!("Unknown S2K type {:#x}", u)).into()),
@@ -159,9 +159,9 @@ impl S2K {
         use std::usize;
 
         match hash_bytes {
-            0...1024 => 1024,
-            1025...2048 => hash_bytes as u32,
-            0x3e00001...usize::MAX => 0x3e00000,
+            0..=1024 => 1024,
+            1025..=2048 => hash_bytes as u32,
+            0x3e00001..=usize::MAX => 0x3e00000,
             hash_bytes => {
                 let hash_bytes = hash_bytes as u32;
                 let msb = 32 - hash_bytes.leading_zeros();
@@ -206,12 +206,12 @@ impl S2K {
 
         let msb = 32 - hash_bytes.leading_zeros();
         let (mantissa_mask, tail_mask) = match msb {
-            0...10 => {
+            0..=10 => {
                 return Err(Error::InvalidArgument(
                     format!("S2K: cannot encode iteration count of {}",
                             hash_bytes)).into());
             }
-            11...32 => {
+            11..=32 => {
                 let m = 0b1111_000000 << (msb - 11);
                 let t = 1 << (msb - 11);
 
@@ -286,7 +286,7 @@ impl Arbitrary for S2K {
 mod tests {
     use super::*;
 
-    use crate::conversions::to_hex;
+    use crate::fmt::to_hex;
     use crate::SymmetricAlgorithm;
     use crate::Packet;
     use crate::parse::{Parse, PacketParser};
