@@ -7,21 +7,22 @@ use std::fmt;
 use crate::Result;
 use crate::Error;
 use crate::SymmetricAlgorithm;
+use crate::vec_truncate;
 
 use buffered_reader::BufferedReader;
 
-use nettle::Cipher;
-use nettle::Mode;
+use nettle::cipher::{self, Cipher};
+use nettle::mode::{self, Mode};
 
 impl SymmetricAlgorithm {
     /// Length of a key for this algorithm in bytes.  Fails if Sequoia
     /// does not support this algorithm.
     pub fn key_size(self) -> Result<usize> {
-        use nettle::cipher;
         match self {
             SymmetricAlgorithm::TripleDES => Ok(cipher::Des3::KEY_SIZE),
             SymmetricAlgorithm::CAST5 => Ok(cipher::Cast128::KEY_SIZE),
-            SymmetricAlgorithm::Blowfish => Ok(cipher::Blowfish::KEY_SIZE),
+            // RFC4880, Section 9.2: Blowfish (128 bit key, 16 rounds)
+            SymmetricAlgorithm::Blowfish => Ok(16),
             SymmetricAlgorithm::AES128 => Ok(cipher::Aes128::KEY_SIZE),
             SymmetricAlgorithm::AES192 => Ok(cipher::Aes192::KEY_SIZE),
             SymmetricAlgorithm::AES256 => Ok(cipher::Aes256::KEY_SIZE),
@@ -36,7 +37,6 @@ impl SymmetricAlgorithm {
     /// Length of a block for this algorithm in bytes.  Fails if
     /// Sequoia does not support this algorithm.
     pub fn block_size(self) -> Result<usize> {
-        use nettle::cipher;
         match self {
             SymmetricAlgorithm::TripleDES => Ok(cipher::Des3::BLOCK_SIZE),
             SymmetricAlgorithm::CAST5 => Ok(cipher::Cast128::BLOCK_SIZE),
@@ -53,8 +53,7 @@ impl SymmetricAlgorithm {
     }
 
     /// Creates a Nettle context for encrypting in CFB mode.
-    pub fn make_encrypt_cfb(self, key: &[u8]) -> Result<Box<Mode>> {
-        use nettle::{mode, cipher};
+    pub fn make_encrypt_cfb(self, key: &[u8]) -> Result<Box<dyn Mode>> {
         match self {
             SymmetricAlgorithm::TripleDES =>
                 Ok(Box::new(
@@ -91,8 +90,7 @@ impl SymmetricAlgorithm {
     }
 
     /// Creates a Nettle context for decrypting in CFB mode.
-    pub fn make_decrypt_cfb(self, key: &[u8]) -> Result<Box<Mode>> {
-        use nettle::{mode, cipher};
+    pub fn make_decrypt_cfb(self, key: &[u8]) -> Result<Box<dyn Mode>> {
         match self {
             SymmetricAlgorithm::TripleDES =>
                 Ok(Box::new(
@@ -134,7 +132,7 @@ pub struct Decryptor<R: io::Read> {
     // The encrypted data.
     source: R,
 
-    dec: Box<Mode>,
+    dec: Box<dyn Mode>,
     block_size: usize,
     iv: Vec<u8>,
     // Up to a block of unread data.
@@ -194,7 +192,7 @@ fn read_exact<R: io::Read>(reader: &mut R, mut buffer: &mut [u8])
 // Note: this implementation tries *very* hard to make sure we don't
 // gratuitiously do a short read.  Specifically, if the return value
 // is less than `plaintext.len()`, then it is either because we
-// reached the end of the input or an error occured.
+// reached the end of the input or an error occurred.
 impl<R: io::Read> io::Read for Decryptor<R> {
     fn read(&mut self, plaintext: &mut [u8]) -> io::Result<usize> {
         let mut pos = 0;
@@ -203,7 +201,7 @@ impl<R: io::Read> io::Read for Decryptor<R> {
         if self.buffer.len() > 0 {
             let to_copy = cmp::min(self.buffer.len(), plaintext.len());
             &plaintext[..to_copy].copy_from_slice(&self.buffer[..to_copy]);
-            self.buffer.drain(..to_copy);
+            crate::vec_drain_prefix(&mut self.buffer, to_copy);
             pos = to_copy;
         }
 
@@ -221,7 +219,7 @@ impl<R: io::Read> io::Read for Decryptor<R> {
             Ok(amount) => {
                 short_read = amount < to_copy;
                 to_copy = amount;
-                ciphertext.truncate(to_copy);
+                vec_truncate(&mut ciphertext, to_copy);
             },
             // We encountered an error, but we did read some.
             Err(_) if pos > 0 => return Ok(pos),
@@ -251,7 +249,7 @@ impl<R: io::Read> io::Read for Decryptor<R> {
             Ok(amount) => {
                 // Make sure `ciphertext` is not larger than the
                 // amount of data that was actually read.
-                ciphertext.truncate(amount);
+                vec_truncate(&mut ciphertext, amount);
 
                 // Make sure we don't read more than is available.
                 to_copy = cmp::min(to_copy, ciphertext.len());
@@ -265,14 +263,14 @@ impl<R: io::Read> io::Read for Decryptor<R> {
         while self.buffer.len() < ciphertext.len() {
             self.buffer.push(0u8);
         }
-        self.buffer.truncate(ciphertext.len());
+        vec_truncate(&mut self.buffer, ciphertext.len());
 
         self.dec.decrypt(&mut self.iv, &mut self.buffer, &ciphertext[..])
             .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput,
                                         format!("{}", e)))?;
 
         &plaintext[pos..pos + to_copy].copy_from_slice(&self.buffer[..to_copy]);
-        self.buffer.drain(..to_copy);
+        crate::vec_drain_prefix(&mut self.buffer, to_copy);
 
         pos += to_copy;
 
@@ -368,17 +366,17 @@ impl<R: BufferedReader<C>, C> BufferedReader<C>
         return self.reader.steal_eof();
     }
 
-    fn get_mut(&mut self) -> Option<&mut BufferedReader<C>> {
+    fn get_mut(&mut self) -> Option<&mut dyn BufferedReader<C>> {
         Some(&mut self.reader.reader.source)
     }
 
-    fn get_ref(&self) -> Option<&BufferedReader<C>> {
+    fn get_ref(&self) -> Option<&dyn BufferedReader<C>> {
         Some(&self.reader.reader.source)
     }
 
     fn into_inner<'b>(self: Box<Self>)
-            -> Option<Box<BufferedReader<C> + 'b>> where Self: 'b {
-        Some(Box::new(self.reader.reader.source))
+            -> Option<Box<dyn BufferedReader<C> + 'b>> where Self: 'b {
+        Some(self.reader.reader.source.as_boxed())
     }
 
     fn cookie_set(&mut self, cookie: C) -> C {
@@ -398,7 +396,7 @@ impl<R: BufferedReader<C>, C> BufferedReader<C>
 pub struct Encryptor<W: io::Write> {
     inner: Option<W>,
 
-    cipher: Box<Mode>,
+    cipher: Box<dyn Mode>,
     block_size: usize,
     iv: Vec<u8>,
     // Up to a block of unencrypted data.
@@ -431,7 +429,7 @@ impl<W: io::Write> Encryptor<W> {
             if self.buffer.len() > 0 {
                 unsafe { self.scratch.set_len(self.buffer.len()) }
                 self.cipher.encrypt(&mut self.iv, &mut self.scratch, &self.buffer)?;
-                self.buffer.clear();
+                crate::vec_truncate(&mut self.buffer, 0);
                 inner.write_all(&self.scratch)?;
             }
             Ok(inner)
@@ -474,7 +472,7 @@ impl<W: io::Write> io::Write for Encryptor<W> {
                 self.cipher.encrypt(&mut self.iv, &mut self.scratch, &self.buffer)
                     .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput,
                                                 format!("{}", e)))?;
-                self.buffer.clear();
+                crate::vec_truncate(&mut self.buffer, 0);
                 inner.write_all(&self.scratch)?;
             }
         }
