@@ -1,12 +1,11 @@
 use std::fmt;
 
 use crate::Fingerprint;
-use crate::KeyID;
 use crate::Result;
 
 impl fmt::Display for Fingerprint {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "{}", self.to_string())
+        write!(f, "{}", self.convert_to_string(true))
     }
 }
 
@@ -53,7 +52,7 @@ impl Fingerprint {
     /// assert_eq!(fp.unwrap().to_hex(), hex);
     /// ```
     pub fn from_hex(hex: &str) -> Result<Fingerprint> {
-        Ok(Fingerprint::from_bytes(&crate::conversions::from_hex(hex, true)?[..]))
+        Ok(Fingerprint::from_bytes(&crate::fmt::from_hex(hex, true)?[..]))
     }
 
     /// Returns a reference to the raw Fingerprint.
@@ -64,13 +63,6 @@ impl Fingerprint {
         }
     }
 
-    /// Converts the fingerprint to its standard representation.
-    ///
-    /// Returns the fingerprint suitable for human consumption.
-    pub fn to_string(&self) -> String {
-        self.convert_to_string(true)
-    }
-
     /// Converts the fingerprint to a hexadecimal number.
     pub fn to_hex(&self) -> String {
         self.convert_to_string(false)
@@ -129,17 +121,6 @@ impl Fingerprint {
         String::from_utf8(output).unwrap()
     }
 
-    /// Converts the fingerprint to a key ID.
-    pub fn to_keyid(&self) -> KeyID {
-        match self {
-            &Fingerprint::V4(ref fp) =>
-                KeyID::from_bytes(&fp[fp.len() - 8..]),
-            &Fingerprint::Invalid(ref fp) => {
-                KeyID::Invalid(fp.clone())
-            }
-        }
-    }
-
     /// Converts the hex representation of the fingerprint to a phrase in the
     /// ICAO alphabet.
     pub fn to_icao(&self) -> String {
