@@ -64,10 +64,6 @@ extern crate quickcheck;
 
 extern crate rand;
 
-extern crate time;
-
-extern crate sequoia_rfc2822 as rfc2822;
-
 #[macro_use] extern crate lazy_static;
 
 extern crate idna;
@@ -75,6 +71,44 @@ extern crate idna;
 #[macro_use]
 mod macros;
 
+// On debug builds, Vec<u8>::truncate is very, very slow.  For
+// instance, running the decrypt_test_stream test takes 51 seconds on
+// my (Neal's) computer using Vec<u8>::truncate and <0.1 seconds using
+// `unsafe { v.set_len(len); }`.
+//
+// The issue is that the compiler calls drop on every element that is
+// dropped, even though a u8 doesn't have a drop implementation.  The
+// compiler optimizes this away at high optimization levels, but those
+// levels make debugging harder.
+fn vec_truncate(v: &mut Vec<u8>, len: usize) {
+    if cfg!(debug_assertions) {
+        if len < v.len() {
+            unsafe { v.set_len(len); }
+        }
+    } else {
+        v.truncate(len);
+    }
+}
+
+/// Like `drop(Vec<u8>::drain(..prefix_len))`, but fast in debug
+/// builds.
+fn vec_drain_prefix(v: &mut Vec<u8>, prefix_len: usize) {
+    if cfg!(debug_assertions) {
+        // Panic like v.drain(..prefix_len).
+        assert!(prefix_len <= v.len(), "prefix len {} > vector len {}",
+                prefix_len, v.len());
+        let new_len = v.len() - prefix_len;
+        unsafe {
+            std::ptr::copy(v[prefix_len..].as_ptr(),
+                           v[..].as_mut_ptr(),
+                           new_len);
+        }
+        vec_truncate(v, new_len);
+    } else {
+        v.drain(..prefix_len);
+    }
+}
+
 // Like assert!, but checks a pattern.
 //
 //   assert_match!(Some(_) = x);
@@ -104,25 +138,23 @@ macro_rules! assert_match {
 
 #[macro_use]
 pub mod armor;
-pub mod autocrypt;
-pub mod conversions;
+pub mod fmt;
 pub mod crypto;
 
 pub mod packet;
-use crate::packet::{BodyLength, Header, Container, key};
-use crate::packet::ctb::{CTB, CTBOld, CTBNew};
+use crate::packet::{Container, key};
 
 pub mod parse;
 
-pub mod tpk;
-pub use tpk::TPK;
+pub mod cert;
+pub use cert::Cert;
 pub mod serialize;
 
 mod packet_pile;
 pub mod message;
 
-pub mod constants;
-use crate::constants::{
+pub mod types;
+use crate::types::{
     PublicKeyAlgorithm,
     SymmetricAlgorithm,
     HashAlgorithm,
@@ -131,6 +163,11 @@ use crate::constants::{
 
 mod fingerprint;
 mod keyid;
+mod keyhandle;
+pub use keyhandle::KeyHandle;
+pub mod policy;
+
+pub(crate) mod utils;
 
 #[cfg(test)]
 mod tests;
@@ -140,9 +177,8 @@ mod tests;
 /// The time is chosen to that the subkeys in
 /// openpgp/tests/data/keys/neal.pgp are not expired.
 #[cfg(test)]
-fn frozen_time() -> time::Tm {
-    use crate::conversions::Time;
-    time::Tm::from_pgp(1554542220 - 1)
+fn frozen_time() -> std::time::SystemTime {
+    crate::types::Timestamp::from(1554542220 - 1).into()
 }
 
 /// Crate result specialization.
@@ -150,6 +186,9 @@ pub type Result<T> = ::std::result::Result<T, failure::Error>;
 
 #[derive(Fail, Debug, Clone)]
 /// Errors returned by this module.
+///
+/// Note: This enum cannot be exhaustively matched to allow future
+/// extensions.
 pub enum Error {
     /// Invalid argument.
     #[fail(display = "Invalid argument: {}", _0)]
@@ -182,7 +221,7 @@ pub enum Error {
 
     /// Unsupported elliptic curve ASN.1 OID.
     #[fail(display = "Unsupported elliptic curve: {}", _0)]
-    UnsupportedEllipticCurve(constants::Curve),
+    UnsupportedEllipticCurve(types::Curve),
 
     /// Unsupported symmetric key algorithm.
     #[fail(display = "Unsupported symmetric algorithm: {}", _0)]
@@ -190,11 +229,11 @@ pub enum Error {
 
     /// Unsupported AEAD algorithm.
     #[fail(display = "Unsupported AEAD algorithm: {}", _0)]
-    UnsupportedAEADAlgorithm(constants::AEADAlgorithm),
+    UnsupportedAEADAlgorithm(types::AEADAlgorithm),
 
     /// Unsupported Compression algorithm.
     #[fail(display = "Unsupported Compression algorithm: {}", _0)]
-    UnsupportedCompressionAlgorithm(constants::CompressionAlgorithm),
+    UnsupportedCompressionAlgorithm(types::CompressionAlgorithm),
 
     /// Unsupported signature type.
     #[fail(display = "Unsupported signature type: {}", _0)]
@@ -228,21 +267,41 @@ pub enum Error {
     #[fail(display = "Malformed Message: {}", _0)]
     MalformedMessage(String),
 
-    /// Malformed tranferable public key.
-    #[fail(display = "Malformed TPK: {}", _0)]
-    MalformedTPK(String),
+    /// Malformed certificate.
+    #[fail(display = "Malformed Cert: {}", _0)]
+    MalformedCert(String),
 
-    /// Unsupported TPK.
+    /// Unsupported Cert.
     ///
     /// This usually occurs, because the primary key is in an
     /// unsupported format.  In particular, Sequoia does not support
     /// version 3 keys.
-    #[fail(display = "Unsupported TPK: {}", _0)]
-    UnsupportedTPK(String),
+    #[fail(display = "Unsupported Cert: {}", _0)]
+    UnsupportedCert(String),
 
     /// Index out of range.
     #[fail(display = "Index out of range")]
     IndexOutOfRange,
+
+    /// Expired.
+    #[fail(display = "Expired on {:?}", _0)]
+    Expired(std::time::SystemTime),
+
+    /// Not yet live.
+    #[fail(display = "Not live until {:?}", _0)]
+    NotYetLive(std::time::SystemTime),
+
+    /// No binding signature.
+    #[fail(display = "No binding signature at time {:?}", _0)]
+    NoBindingSignature(std::time::SystemTime),
+
+    /// Invalid key.
+    #[fail(display = "Invalid key: {:?}", _0)]
+    InvalidKey(String),
+
+    /// This marks this enum as non-exhaustive.  Do not use this
+    /// variant.
+    #[doc(hidden)] #[fail(display = "__Nonexhaustive")] __Nonexhaustive,
 }
 
 /// The OpenPGP packets that Sequoia understands.
@@ -260,6 +319,9 @@ pub enum Error {
 /// than a `CompressedData` packet.
 ///
 ///   [Section 5 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-5
+///
+/// Note: This enum cannot be exhaustively matched to allow future
+/// extensions.
 #[derive(Debug)]
 #[derive(PartialEq, Eq, Hash, Clone)]
 pub enum Packet {
@@ -299,6 +361,10 @@ pub enum Packet {
     MDC(packet::MDC),
     /// AEAD Encrypted Data Packet.
     AED(packet::AED),
+
+    /// This marks this enum as non-exhaustive.  Do not use this
+    /// variant.
+    #[doc(hidden)] __Nonexhaustive,
 }
 
 impl Packet {
@@ -328,6 +394,7 @@ impl Packet {
             &Packet::SEIP(_) => Tag::SEIP,
             &Packet::MDC(_) => Tag::MDC,
             &Packet::AED(_) => Tag::AED,
+            Packet::__Nonexhaustive => unreachable!(),
         }
     }
 
@@ -359,6 +426,7 @@ impl Packet {
             &Packet::SEIP(_) => Some(Tag::SEIP),
             &Packet::MDC(_) => Some(Tag::MDC),
             &Packet::AED(_) => Some(Tag::AED),
+            Packet::__Nonexhaustive => unreachable!(),
         }
     }
 }
@@ -369,7 +437,7 @@ impl Packet {
 /// [`PacketPileParser`], or [`PacketPile::from_file`] (or related
 /// routines).
 ///
-/// Normally, you'll want to convert the `PacketPile` to a TPK or a
+/// Normally, you'll want to convert the `PacketPile` to a Cert or a
 /// `Message`.
 ///
 ///   [`PacketParser`]: parse/struct.PacketParser.html
@@ -403,7 +471,7 @@ pub struct Message {
 /// 4880].
 ///
 ///   [Section 12.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-12.2
-#[derive(PartialEq, Eq, Clone, Hash)]
+#[derive(PartialEq, Eq, PartialOrd, Ord, Clone, Hash)]
 pub enum Fingerprint {
     /// 20 byte SHA-1 hash.
     V4([u8;20]),
@@ -435,18 +503,14 @@ pub enum KeyID {
 pub enum RevocationStatus<'a> {
     /// The key is definitely revoked.
     ///
-    /// All self-revocations are returned, the most recent revocation
-    /// first.
-    Revoked(&'a [packet::Signature]),
-    /// We have a third-party revocation certificate that is allegedly
-    /// from a designated revoker, but we don't have the designated
-    /// revoker's key to check its validity.
+    /// The relevant self-revocations are returned.
+    Revoked(Vec<&'a packet::Signature>),
+    /// There is a revocation certificate from a possible designated
+    /// revoker.
+    CouldBe(Vec<&'a packet::Signature>),
+    /// The key does not appear to be revoked.
     ///
-    /// All such certificates are returned.  The caller must check
-    /// them manually.
-    CouldBe(&'a [packet::Signature]),
-    /// The key does not appear to be revoked, but perhaps an attacker
-    /// has performed a DoS, which prevents us from seeing the
-    /// revocation certificate.
+    /// An attacker could still have performed a DoS, which prevents
+    /// us from seeing the revocation certificate.
     NotAsFarAsWeKnow,
 }
