@@ -36,12 +36,19 @@ use lalrpop_util::ParseError;
 use self::grammar::MessageParser;
 
 /// Errors that MessageValidator::check may return.
+///
+/// Note: This enum cannot be exhaustively matched to allow future
+/// extensions.
 #[derive(Debug, Clone)]
 pub enum MessageParserError {
     /// A parser error.
     Parser(ParseError<usize, Token, LexicalError>),
     /// An OpenPGP error.
     OpenPGP(Error),
+
+    /// This marks this enum as non-exhaustive.  Do not use this
+    /// variant.
+    #[doc(hidden)] __Nonexhaustive,
 }
 
 impl From<MessageParserError> for failure::Error {
@@ -49,6 +56,7 @@ impl From<MessageParserError> for failure::Error {
         match err {
             MessageParserError::Parser(p) => p.into(),
             MessageParserError::OpenPGP(p) => p.into(),
+            MessageParserError::__Nonexhaustive => unreachable!(),
         }
     }
 }
@@ -333,8 +341,8 @@ impl<'a> Parse<'a, Message> for Message {
     /// See [`Message::from_packet_pile`] for more details.
     ///
     ///   [`Message::from_packet_pile`]: #method.from_packet_pile
-    fn from_bytes(buf: &'a [u8]) -> Result<Self> {
-        Self::from_packet_pile(PacketPile::from_bytes(buf)?)
+    fn from_bytes<D: AsRef<[u8]> + ?Sized>(data: &'a D) -> Result<Self> {
+        Self::from_packet_pile(PacketPile::from_bytes(data)?)
     }
 }
 
@@ -380,7 +388,9 @@ impl Message {
                     // we treat the content as an opaque message.
 
                     path.push(0);
-                    if packet.children.is_none() && packet.body.is_some() {
+                    if packet.children().next().is_none()
+                        && packet.body().is_some()
+                    {
                         v.push_token(Token::OpaqueContent, &path);
                     }
                 }
@@ -442,17 +452,16 @@ impl ::std::ops::Deref for Message {
 mod tests {
     use super::*;
 
-    use crate::constants::DataFormat::Text;
+    use crate::types::DataFormat::Text;
     use crate::HashAlgorithm;
-    use crate::constants::CompressionAlgorithm;
+    use crate::types::CompressionAlgorithm;
     use crate::SymmetricAlgorithm;
     use crate::PublicKeyAlgorithm;
     use crate::SignatureType;
-    use crate::crypto::s2k::S2K;
+    use crate::crypto::S2K;
     use crate::crypto::mpis::{Ciphertext, MPI};
     use crate::packet::prelude::*;
     use crate::KeyID;
-    use crate::Container;
 
     #[test]
     fn tokens() {
@@ -579,7 +588,7 @@ mod tests {
             },
         ];
 
-        for v in test_vectors.into_iter() {
+        for v in &test_vectors {
             if v.result {
                 let mut l = MessageValidator::new();
                 for token in v.s.iter() {
@@ -698,7 +707,7 @@ mod tests {
             },
         ];
 
-        for v in test_vectors.into_iter() {
+        for v in &test_vectors {
             let mut l = MessageValidator::new();
             for (token, depth) in v.s.iter() {
                 l.push(*token,
@@ -805,13 +814,13 @@ mod tests {
         let mut lit = Literal::new(Text);
         lit.set_body(b"data".to_vec());
 
-        let hash = crate::constants::HashAlgorithm::SHA512;
+        let hash = crate::types::HashAlgorithm::SHA512;
         let key: key::SecretKey =
-            crate::packet::key::Key4::generate_ecc(true, crate::constants::Curve::Ed25519)
+            crate::packet::key::Key4::generate_ecc(true, crate::types::Curve::Ed25519)
             .unwrap().into();
         let mut pair = key.clone().into_keypair().unwrap();
         let sig = crate::packet::signature::Builder::new(SignatureType::Binary)
-            .sign_hash(&mut pair, hash, hash.context().unwrap()).unwrap();
+            .sign_hash(&mut pair, hash.context().unwrap()).unwrap();
 
         // 0: OnePassSig
         // => bad.
@@ -917,13 +926,13 @@ mod tests {
         let mut lit = Literal::new(Text);
         lit.set_body(b"data".to_vec());
 
-        let hash = crate::constants::HashAlgorithm::SHA512;
+        let hash = crate::types::HashAlgorithm::SHA512;
         let key: key::SecretKey =
-            crate::packet::key::Key4::generate_ecc(true, crate::constants::Curve::Ed25519)
+            crate::packet::key::Key4::generate_ecc(true, crate::types::Curve::Ed25519)
             .unwrap().into();
         let mut pair = key.clone().into_keypair().unwrap();
         let sig = crate::packet::signature::Builder::new(SignatureType::Binary)
-            .sign_hash(&mut pair, hash, hash.context().unwrap()).unwrap();
+            .sign_hash(&mut pair, hash.context().unwrap()).unwrap();
 
         // 0: Signature
         // => bad.
@@ -961,7 +970,7 @@ mod tests {
         // There are no simple constructors for SEIP packets: they are
         // interleaved with SK-ESK and PK-ESK packets.  And, the
         // session key needs to be managed.  Instead, we use some
-        // internal iterfaces to progressively build up more
+        // internal interfaces to progressively build up more
         // complicated messages.
 
         let mut lit = Literal::new(Text);
@@ -996,10 +1005,9 @@ mod tests {
         //  1: MDC
         // => good.
         let mut seip = SEIP1::new();
-        seip.common.children = Some(Container::new());
-        seip.common.children.as_mut().unwrap().push(
+        seip.children_mut().push(
             lit.clone().into());
-        seip.common.children.as_mut().unwrap().push(
+        seip.children_mut().push(
             MDC::from([0u8; 20]).into());
         packets[1] = seip.into();
 
@@ -1079,7 +1087,8 @@ mod tests {
         //  2: Literal
         // => bad.
         packets.remove(3);
-        packets[2].children.as_mut().unwrap().push(lit.clone().into());
+        packets[2].container_mut().unwrap()
+            .children_mut().push(lit.clone().into());
 
         assert!(packets.iter().map(|p| p.tag()).collect::<Vec<Tag>>()
                 == [ Tag::SKESK, Tag::SKESK, Tag::SEIP ]);
@@ -1093,7 +1102,7 @@ mod tests {
         // 2: SEIP
         //  0: Literal
         // => good.
-        packets[2].children.as_mut().unwrap().packets.pop().unwrap();
+        packets[2].container_mut().unwrap().packets.pop().unwrap();
 
         #[allow(deprecated)]
         packets.insert(
