@@ -1,10 +1,8 @@
 use std::fmt;
-use std::ops::{Deref, DerefMut};
 
-use crate::packet::{self, Common};
+use crate::packet;
 use crate::Packet;
-use crate::Container;
-use crate::constants::CompressionAlgorithm;
+use crate::types::CompressionAlgorithm;
 
 /// Holds a compressed data packet.
 ///
@@ -16,23 +14,41 @@ use crate::constants::CompressionAlgorithm;
 /// of a `CompressedData` packet.
 ///
 /// [Section 5.6 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-5.6
-#[derive(PartialEq, Eq, Hash, Clone)]
+#[derive(Clone)]
 pub struct CompressedData {
     /// CTB packet header fields.
     pub(crate) common: packet::Common,
     /// Algorithm used to compress the payload.
     algo: CompressionAlgorithm,
+
+    /// This is a container packet.
+    container: packet::Container,
+}
+
+impl PartialEq for CompressedData {
+    fn eq(&self, other: &CompressedData) -> bool {
+        self.algo == other.algo
+            && self.container == other.container
+    }
+}
+
+impl Eq for CompressedData {}
+
+impl std::hash::Hash for CompressedData {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        std::hash::Hash::hash(&self.algo, state);
+        std::hash::Hash::hash(&self.container, state);
+    }
 }
 
 impl fmt::Debug for CompressedData {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.debug_struct("CompressedData")
             .field("algo", &self.algo)
-            .field("children",
-                   &self.common.children.as_ref()
-                       .map(|c| &c.packets).unwrap_or(&Vec::new()))
+            .field("children", &self.container.children_ref())
             .field("body (bytes)",
-                   &self.common.body.as_ref().unwrap_or(&b"".to_vec()).len())
+                   &self.container.body().len())
+            .field("body_digest", &self.container.body_digest())
             .finish()
     }
 }
@@ -43,25 +59,24 @@ impl CompressedData {
         CompressedData {
             common: Default::default(),
             algo: algo,
+            container: Default::default(),
         }
     }
 
     /// Gets the compression algorithm.
-    pub fn algorithm(&self) -> CompressionAlgorithm {
+    pub fn algo(&self) -> CompressionAlgorithm {
         self.algo
     }
 
     /// Sets the compression algorithm.
-    pub fn set_algorithm(&mut self, algo: CompressionAlgorithm) -> CompressionAlgorithm {
+    pub fn set_algo(&mut self, algo: CompressionAlgorithm) -> CompressionAlgorithm {
         ::std::mem::replace(&mut self.algo, algo)
     }
 
     /// Adds a new packet to the container.
+    #[cfg(test)]
     pub fn push(mut self, packet: Packet) -> Self {
-        if self.common.children.is_none() {
-            self.common.children = Some(Container::new());
-        }
-        self.common.children.as_mut().unwrap().push(packet);
+        self.container.children_mut().push(packet);
         self
     }
 
@@ -69,33 +84,17 @@ impl CompressedData {
     /// If `i` is 0, the new packet is insert at the front of the
     /// container.  If `i` is one, it is inserted after the first
     /// packet, etc.
+    #[cfg(test)]
     pub fn insert(mut self, i: usize, packet: Packet) -> Self {
-        if self.common.children.is_none() {
-            self.common.children = Some(Container::new());
-        }
-        self.common.children.as_mut().unwrap().insert(i, packet);
+        self.container.children_mut().insert(i, packet);
         self
     }
 }
 
+impl_container_forwards!(CompressedData);
+
 impl From<CompressedData> for Packet {
     fn from(s: CompressedData) -> Self {
         Packet::CompressedData(s)
     }
 }
-
-// Allow transparent access of common fields.
-impl<'a> Deref for CompressedData {
-    type Target = Common;
-
-    fn deref(&self) -> &Self::Target {
-        &self.common
-    }
-}
-
-// Allow transparent access of common fields.
-impl<'a> DerefMut for CompressedData {
-    fn deref_mut(&mut self) -> &mut Self::Target {
-        &mut self.common
-    }
-}
