@@ -9,12 +9,26 @@ use crate::Packet;
 /// See [Section 5.10 of RFC 4880] for details.
 ///
 ///   [Section 5.10 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-5.10
-#[derive(PartialEq, Eq, Hash, Clone)]
+#[derive(Clone)]
 pub struct Trust {
     pub(crate) common: packet::Common,
     value: Vec<u8>,
 }
 
+impl PartialEq for Trust {
+    fn eq(&self, other: &Trust) -> bool {
+        self.value == other.value
+    }
+}
+
+impl Eq for Trust {}
+
+impl std::hash::Hash for Trust {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        std::hash::Hash::hash(&self.value, state);
+    }
+}
+
 impl From<Vec<u8>> for Trust {
     fn from(u: Vec<u8>) -> Self {
         Trust {
@@ -34,7 +48,7 @@ impl fmt::Display for Trust {
 impl fmt::Debug for Trust {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.debug_struct("Trust")
-            .field("value", &crate::conversions::hex::encode(&self.value))
+            .field("value", &crate::fmt::hex::encode(&self.value))
             .finish()
     }
 }
