@@ -9,7 +9,7 @@ use buffered_reader::BufferedReader;
 use crate::Result;
 use crate::Error;
 use crate::Packet;
-use crate::packet::{Container, PacketIter};
+use crate::packet::{self, Container};
 use crate::PacketPile;
 use crate::parse::PacketParserResult;
 use crate::parse::PacketParserBuilder;
@@ -57,9 +57,9 @@ impl<'a> Parse<'a, PacketPile> for PacketPile {
     /// Deserializes the OpenPGP message stored in the provided buffer.
     ///
     /// See `from_reader` for more details and caveats.
-    fn from_bytes(data: &'a [u8]) -> Result<PacketPile> {
+    fn from_bytes<D: AsRef<[u8]> + ?Sized>(data: &'a D) -> Result<PacketPile> {
         let bio = buffered_reader::Memory::with_cookie(
-            data, Cookie::default());
+            data.as_ref(), Cookie::default());
         PacketPile::from_buffered_reader(Box::new(bio))
     }
 }
@@ -74,7 +74,7 @@ impl std::str::FromStr for PacketPile {
 
 impl From<Vec<Packet>> for PacketPile {
     fn from(p: Vec<Packet>) -> Self {
-        PacketPile { top_level: Container { packets: p } }
+        PacketPile { top_level: Container::from(p) }
     }
 }
 
@@ -126,7 +126,7 @@ impl PacketPile {
                 if *i < c.packets.len() {
                     let p = &c.packets[*i];
                     packet = Some(p);
-                    cont = p.children.as_ref();
+                    cont = p.container_ref();
                     continue;
                 }
             }
@@ -156,7 +156,7 @@ impl PacketPile {
                 return Some(p)
             }
 
-            container = p.children.as_mut().unwrap();
+            container = p.container_mut().unwrap();
         }
 
         None
@@ -180,7 +180,7 @@ impl PacketPile {
     ///
     /// ```rust
     /// # extern crate sequoia_openpgp as openpgp;
-    /// # use openpgp::{Result, constants::{CompressionAlgorithm, DataFormat},
+    /// # use openpgp::{Result, types::{CompressionAlgorithm, DataFormat},
     /// #     Packet, PacketPile, packet::Literal, packet::CompressedData};
     ///
     /// # fn main() { f().unwrap(); }
@@ -188,9 +188,10 @@ impl PacketPile {
     /// // A compressed data packet that contains a literal data packet.
     /// let mut literal = Literal::new(DataFormat::Text);
     /// literal.set_body(b"old".to_vec());
-    /// let mut pile = PacketPile::from(Packet::from(
-    ///     CompressedData::new(CompressionAlgorithm::Uncompressed)
-    ///         .push(literal.into())));
+    /// let mut compressed =
+    ///     CompressedData::new(CompressionAlgorithm::Uncompressed);
+    /// compressed.children_mut().push(literal.into());
+    /// let mut pile = PacketPile::from(Packet::from(compressed));
     ///
     /// // Replace the literal data packet.
     /// let mut literal = Literal::new(DataFormat::Text);
@@ -200,7 +201,7 @@ impl PacketPile {
     ///     [ literal.into() ].to_vec())
     ///     .unwrap();
     /// # if let Some(Packet::Literal(lit)) = pile.path_ref(&[0, 0]) {
-    /// #     assert_eq!(lit.body(), Some(&b"new"[..]), "{:#?}", lit);
+    /// #     assert_eq!(lit.body(), &b"new"[..], "{:#?}", lit);
     /// # } else {
     /// #     panic!("Unexpected packet!");
     /// # }
@@ -243,21 +244,16 @@ impl PacketPile {
                 return Err(Error::IndexOutOfRange.into());
             }
 
-            let p = &mut tmp.packets[i];
-            if p.children.is_none() {
-                match p {
-                    Packet::CompressedData(_) | Packet::SEIP(_) => {
-                        // We have a container with no children.
-                        // That's okay.  We can create the container.
-                        p.children = Some(Container::new());
-                    },
-                    _ => {
-                        return Err(Error::IndexOutOfRange.into());
-                    }
-                }
+            match tmp.packets[i] {
+                // The structured container types.
+                Packet::CompressedData(_)
+                    | Packet::SEIP(_)
+                    | Packet::AED(_)
+                    => (), // Ok.
+                _ => return Err(Error::IndexOutOfRange.into()),
             }
-
-            container = p.children.as_mut().unwrap();
+            container = tmp.packets[i].container_mut()
+                .expect("The above packets are structured containers");
         }
 
         return Err(Error::IndexOutOfRange.into());
@@ -265,7 +261,7 @@ impl PacketPile {
 
     /// Returns an iterator over all of the packet's descendants, in
     /// depth-first order.
-    pub fn descendants(&self) -> PacketIter {
+    pub fn descendants(&self) -> packet::Iter {
         self.top_level.descendants()
     }
 
@@ -280,7 +276,7 @@ impl PacketPile {
     }
 
 
-    pub(crate) fn from_buffered_reader<'a>(bio: Box<'a + BufferedReader<Cookie>>)
+    pub(crate) fn from_buffered_reader<'a>(bio: Box<dyn BufferedReader<Cookie> + 'a>)
             -> Result<PacketPile> {
         PacketParserBuilder::from_buffered_reader(bio)?
             .buffer_unread_content()
@@ -304,7 +300,7 @@ impl PacketPile {
         }
 
         // Create a top-level container.
-        let mut top_level = Container::new();
+        let mut top_level = Container::default();
 
         let mut last_position = 0;
 
@@ -332,7 +328,7 @@ impl PacketPile {
                 let packets_len = tmp.packets.len();
                 let p = &mut tmp.packets[packets_len - 1];
 
-                container = p.children.as_mut().unwrap();
+                container = p.container_mut().unwrap();
             }
 
             if relative_position < 0 {
@@ -347,9 +343,7 @@ impl PacketPile {
                     // Create a new container.
                     let tmp = container;
                     let i = tmp.packets.len() - 1;
-                    assert!(tmp.packets[i].children.is_none());
-                    tmp.packets[i].children = Some(Container::new());
-                    container = tmp.packets[i].children.as_mut().unwrap();
+                    container = tmp.packets[i].container_mut().unwrap();
                 }
 
                 container.packets.push(packet);
@@ -418,8 +412,8 @@ impl<'a> PacketParserBuilder<'a> {
 mod test {
     use super::*;
 
-    use crate::constants::CompressionAlgorithm;
-    use crate::constants::DataFormat::Text;
+    use crate::types::CompressionAlgorithm;
+    use crate::types::DataFormat::Text;
     use crate::packet::Literal;
     use crate::packet::CompressedData;
     use crate::packet::seip::SEIP1;
@@ -496,9 +490,9 @@ mod test {
             .buffer_unread_content()
             .into_packet_pile_parser().unwrap();
 
-        while ppp.recurse() {
-            //let pp = ppp.ppo.as_mut().unwrap();
-            //eprintln!("{:?}", pp);
+        let mut ppr = ppp.recurse().unwrap();
+        while ppr.is_some() {
+            ppr = ppp.recurse().unwrap();
         }
         let pile = ppp.finish();
         //pile.pretty_print();
@@ -510,14 +504,10 @@ mod test {
             .buffer_unread_content()
             .into_packet_pile_parser().unwrap();
 
-        while ppp.recurse() {
-            if let PacketParserResult::Some(ref pp) = ppp.ppr {
-                eprintln!("{:?}", pp);
-            } else {
-                // If PacketPileParser::recurse returns true, then
-                // ppp.ppr is not EOF.
-                unreachable!();
-            }
+        let mut ppr = ppp.recurse().unwrap();
+        while let Some(pp) = ppr.as_mut() {
+            eprintln!("{:?}", pp);
+            ppr = ppp.recurse().unwrap();
         }
         let pile = ppp.finish();
         pile.pretty_print();
@@ -602,12 +592,12 @@ mod test {
 
         // recurse should now not recurse.  Since there is nothing
         // following the compressed packet, ppr should be EOF.
-        let (mut packet, ppr) = pp.next().unwrap();
+        let (packet, ppr) = pp.next().unwrap();
         assert!(ppr.is_none());
 
         // Get the rest of the content and put the initial byte that
         // we stole back.
-        let mut content = packet.body.take().unwrap();
+        let mut content = packet.body().unwrap().to_vec();
         content.insert(0, data[0]);
 
         let content = &content.into_boxed_slice()[..];
@@ -644,8 +634,7 @@ mod test {
         }
 
         let mut seip = SEIP1::new();
-        seip.common.children = Some(Container::new());
-        seip.common.children.as_mut().unwrap().push(cd.into());
+        seip.children_mut().push(cd.into());
         packets.push(seip.into());
 
         eprintln!("{:#?}", packets);
@@ -659,16 +648,24 @@ mod test {
         assert_eq!(pile.path_ref_mut(&[ 0, 0 ]).unwrap().tag(),
                    Tag::CompressedData);
 
-        for (i, t) in text.iter().enumerate() {
+        for (i, t) in text.into_iter().enumerate() {
             assert_eq!(pile.path_ref(&[ 0, 0, i ]).unwrap().tag(),
                        Tag::Literal);
             assert_eq!(pile.path_ref_mut(&[ 0, 0, i ]).unwrap().tag(),
                        Tag::Literal);
 
-            assert_eq!(pile.path_ref(&[ 0, 0, i ]).unwrap().body,
-                       Some(t.to_vec()));
-            assert_eq!(pile.path_ref_mut(&[ 0, 0, i ]).unwrap().body,
-                       Some(t.to_vec()));
+            let packet = pile.path_ref(&[ 0, 0, i ]).unwrap();
+            if let Packet::Literal(l) = packet {
+                assert_eq!(l.body(), t);
+            } else {
+                panic!("Expected literal, got: {:?}", packet);
+            }
+            let packet = pile.path_ref_mut(&[ 0, 0, i ]).unwrap();
+            if let Packet::Literal(l) = packet {
+                assert_eq!(l.body(), t);
+            } else {
+                panic!("Expected literal, got: {:?}", packet);
+            }
         }
 
         // Try a few out of bounds accesses.
@@ -721,8 +718,7 @@ mod test {
         let children = pile.into_children().collect::<Vec<Packet>>();
         assert_eq!(children.len(), 1, "{:#?}", children);
         if let Packet::Literal(ref literal) = children[0] {
-            assert_eq!(literal.common.body, Some(b"two".to_vec()),
-                       "{:#?}", literal);
+            assert_eq!(literal.body(), &b"two"[..], "{:#?}", literal);
         } else {
             panic!("WTF");
         }
@@ -759,7 +755,7 @@ mod test {
                         .children()
                         .map(|p| {
                             if let Packet::Literal(ref literal) = p {
-                                &literal.common.body.as_ref().unwrap()[..]
+                                literal.body()
                             } else {
                                 panic!("Expected a literal packet, got: {:?}", p);
                             }
@@ -813,10 +809,10 @@ mod test {
                     assert_eq!(top_level.len(), 1);
 
                     let values = top_level[0]
-                        .children.as_ref().unwrap().children()
+                        .children()
                         .map(|p| {
                             if let Packet::Literal(ref literal) = p {
-                                &literal.common.body.as_ref().unwrap()[..]
+                                literal.body()
                             } else {
                                 panic!("Expected a literal packet, got: {:?}", p);
                             }
