@@ -39,7 +39,8 @@ impl<R: BufferedReader<Cookie>> HashedReader<R> {
             -> Self {
         let mut cookie = Cookie::default();
         for &algo in &algos {
-            cookie.sig_group_mut().hashes.insert(algo, algo.context().unwrap());
+            cookie.sig_group_mut().hashes
+                .push(algo.context().unwrap());
         }
         cookie.hashes_for = hashes_for;
 
@@ -68,11 +69,10 @@ impl Cookie {
             // We fix that here by hashing the stashed data into the
             // former topmost signature-group's hash.
             assert!(ngroups > 1);
-            for (algo, ref mut h) in
-                self.sig_groups[ngroups-2].hashes.iter_mut()
+            for h in self.sig_groups[ngroups-2].hashes.iter_mut()
             {
                 t!("({:?}): group {} {:?} hashing {} stashed bytes.",
-                   hashes_for, ngroups-2, algo, data.len());
+                   hashes_for, ngroups-2, h.algo(), data.len());
 
                 h.update(&stashed_data);
             }
@@ -87,7 +87,7 @@ impl Cookie {
 
         if self.hashing == Hashing::Disabled {
             t!("    hash_update: NOT hashing {} bytes: {}.",
-               data.len(), crate::conversions::to_hex(data, true));
+               data.len(), crate::fmt::to_hex(data, true));
             return;
         }
 
@@ -95,14 +95,14 @@ impl Cookie {
         for (i, sig_group) in self.sig_groups.iter_mut().enumerate() {
             if topmost_group(i) && self.hashing != Hashing::Enabled {
                 t!("topmost group {} NOT hashing {} bytes: {}.",
-                   i, data.len(), crate::conversions::to_hex(data, true));
+                   i, data.len(), crate::fmt::to_hex(data, true));
 
                 return;
             }
 
-            for (algo, ref mut h) in sig_group.hashes.iter_mut() {
+            for h in sig_group.hashes.iter_mut() {
                 t!("{:?}): group {} {:?} hashing {} bytes.",
-                   hashes_for, i, algo, data.len());
+                   hashes_for, i, h.algo(), data.len());
                 h.update(data);
             }
         }
@@ -194,18 +194,18 @@ impl<R: BufferedReader<Cookie>>
         result
     }
 
-    fn get_mut(&mut self) -> Option<&mut BufferedReader<Cookie>> {
+    fn get_mut(&mut self) -> Option<&mut dyn BufferedReader<Cookie>> {
         Some(&mut self.reader)
     }
 
-    fn get_ref(&self) -> Option<&BufferedReader<Cookie>> {
+    fn get_ref(&self) -> Option<&dyn BufferedReader<Cookie>> {
         Some(&self.reader)
     }
 
     fn into_inner<'b>(self: Box<Self>)
-            -> Option<Box<BufferedReader<Cookie> + 'b>>
+            -> Option<Box<dyn BufferedReader<Cookie> + 'b>>
             where Self: 'b {
-        Some(Box::new(self.reader))
+        Some(self.reader.as_boxed())
     }
 
     fn cookie_set(&mut self, cookie: Cookie) -> Cookie {
@@ -274,12 +274,13 @@ mod test {
 
             let mut hashes = mem::replace(&mut cookie.sig_group_mut().hashes,
                                           Default::default());
-            for (algo, ref mut hash) in hashes.iter_mut() {
+            for hash in hashes.iter_mut() {
+                let algo = hash.algo();
                 let mut digest = vec![0u8; hash.digest_size()];
                 hash.digest(&mut digest);
 
                 assert_eq!(digest,
-                           &crate::conversions::from_hex(test.expected.get(algo)
+                           &crate::fmt::from_hex(test.expected.get(&algo)
                                                     .unwrap(), true)
                            .unwrap()[..],
                            "Algo: {:?}", algo);
