@@ -1,6 +1,7 @@
 //! Functions for parsing MPIs.
 
 use std::io::Read;
+use buffered_reader::BufferedReader;
 use crate::{
     Result,
     Error,
@@ -8,7 +9,7 @@ use crate::{
     SymmetricAlgorithm,
     HashAlgorithm,
 };
-use crate::constants::Curve;
+use crate::types::Curve;
 use crate::crypto::mpis::{self, MPI};
 use crate::parse::{
     PacketHeaderParser,
@@ -30,7 +31,7 @@ impl mpis::PublicKey {
         let cur = Cursor::new(buf);
         let bio = buffered_reader::Generic::with_cookie(
             cur, None, Cookie::default());
-        let mut php = PacketHeaderParser::new_naked(Box::new(bio));
+        let mut php = PacketHeaderParser::new_naked(bio);
         Self::_parse(algo, &mut php)
     }
 
@@ -39,8 +40,9 @@ impl mpis::PublicKey {
     /// See [Section 3.2 of RFC 4880] for details.
     ///
     ///   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
-    pub(crate) fn _parse<'a>(algo: PublicKeyAlgorithm,
-                            php: &mut PacketHeaderParser<'a>)
+    pub(crate) fn _parse<'a, T: 'a + BufferedReader<Cookie>>(
+        algo: PublicKeyAlgorithm,
+        php: &mut PacketHeaderParser<T>)
         -> Result<Self>
     {
         use crate::PublicKeyAlgorithm::*;
@@ -130,8 +132,8 @@ impl mpis::PublicKey {
 
             Unknown(_) | Private(_) => {
                 let mut mpis = Vec::new();
-                while let Ok(mpi) = MPI::parse("unknown_parameter_len",
-                                               "unknown_parameter", php) {
+                while let Ok(mpi) = MPI::parse("unknown_len",
+                                               "unknown", php) {
                     mpis.push(mpi);
                 }
                 let rest = php.parse_bytes_eof("rest")?;
@@ -156,7 +158,7 @@ impl mpis::SecretKeyMaterial {
         // read mpis
         let bio = buffered_reader::Generic::with_cookie(
             cur, None, Cookie::default());
-        let mut php = PacketHeaderParser::new_naked(Box::new(bio));
+        let mut php = PacketHeaderParser::new_naked(bio);
         let mpis = Self::_parse(algo, &mut php)?;
 
         // read expected sha1 hash of the mpis
@@ -192,7 +194,7 @@ impl mpis::SecretKeyMaterial {
         let cur = Cursor::new(buf);
         let bio = buffered_reader::Generic::with_cookie(
             cur, None, Cookie::default());
-        let mut php = PacketHeaderParser::new_naked(Box::new(bio));
+        let mut php = PacketHeaderParser::new_naked(bio);
         Self::_parse(algo, &mut php)
     }
 
@@ -201,9 +203,10 @@ impl mpis::SecretKeyMaterial {
     /// See [Section 3.2 of RFC 4880] for details.
     ///
     ///   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
-    pub(crate) fn _parse<'a>(algo: PublicKeyAlgorithm,
-                             php: &mut PacketHeaderParser<'a>)
-                             -> Result<Self>
+    pub(crate) fn _parse<'a, T: 'a + BufferedReader<Cookie>>(
+        algo: PublicKeyAlgorithm,
+        php: &mut PacketHeaderParser<T>)
+        -> Result<Self>
     {
         use crate::PublicKeyAlgorithm::*;
 
@@ -263,8 +266,8 @@ impl mpis::SecretKeyMaterial {
 
             Unknown(_) | Private(_) => {
                 let mut mpis = Vec::new();
-                while let Ok(mpi) = MPI::parse("unknown_parameter_len",
-                                               "unknown_parameter", php) {
+                while let Ok(mpi) = MPI::parse("unknown_len",
+                                               "unknown", php) {
                     mpis.push(mpi.into());
                 }
                 let rest = php.parse_bytes_eof("rest")?;
@@ -292,7 +295,7 @@ impl mpis::Ciphertext {
         let cur = Cursor::new(buf);
         let bio = buffered_reader::Generic::with_cookie(
             cur, None, Cookie::default());
-        let mut php = PacketHeaderParser::new_naked(Box::new(bio));
+        let mut php = PacketHeaderParser::new_naked(bio);
         Self::_parse(algo, &mut php)
     }
 
@@ -302,15 +305,16 @@ impl mpis::Ciphertext {
     /// See [Section 3.2 of RFC 4880] for details.
     ///
     ///   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
-    pub(crate) fn _parse<'a>(algo: PublicKeyAlgorithm,
-                             php: &mut PacketHeaderParser<'a>)
-                             -> Result<Self> {
+    pub(crate) fn _parse<'a, T: 'a + BufferedReader<Cookie>>(
+        algo: PublicKeyAlgorithm,
+        php: &mut PacketHeaderParser<T>)
+        -> Result<Self> {
         use crate::PublicKeyAlgorithm::*;
 
         #[allow(deprecated)]
         match algo {
             RSAEncryptSign | RSAEncrypt => {
-                let c = MPI::parse("rsa_ciphertext_len", "rsa_ciphertext",
+                let c = MPI::parse("rsa_ciphertxt_len", "rsa_ciphertxt",
                                    php)?;
 
                 Ok(mpis::Ciphertext::RSA {
@@ -341,8 +345,8 @@ impl mpis::Ciphertext {
 
             Unknown(_) | Private(_) => {
                 let mut mpis = Vec::new();
-                while let Ok(mpi) = MPI::parse("unknown_parameter_len",
-                                               "unknown_parameter", php) {
+                while let Ok(mpi) = MPI::parse("unknown_len",
+                                               "unknown", php) {
                     mpis.push(mpi);
                 }
                 let rest = php.parse_bytes_eof("rest")?;
@@ -373,7 +377,7 @@ impl mpis::Signature {
         let cur = Cursor::new(buf);
         let bio = buffered_reader::Generic::with_cookie(
             cur, None, Cookie::default());
-        let mut php = PacketHeaderParser::new_naked(Box::new(bio));
+        let mut php = PacketHeaderParser::new_naked(bio);
         Self::_parse(algo, &mut php)
     }
 
@@ -383,9 +387,10 @@ impl mpis::Signature {
     /// See [Section 3.2 of RFC 4880] for details.
     ///
     ///   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
-    pub(crate) fn _parse<'a>(algo: PublicKeyAlgorithm,
-                             php: &mut PacketHeaderParser<'a>)
-                             -> Result<Self> {
+    pub(crate) fn _parse<'a, T: 'a + BufferedReader<Cookie>>(
+        algo: PublicKeyAlgorithm,
+        php: &mut PacketHeaderParser<T>)
+        -> Result<Self> {
         use crate::PublicKeyAlgorithm::*;
 
         #[allow(deprecated)]
@@ -399,9 +404,9 @@ impl mpis::Signature {
             }
 
             DSA => {
-                let r = MPI::parse("dsa_signature_r_len", "dsa_signature_r",
+                let r = MPI::parse("dsa_sig_r_len", "dsa_sig_r",
                                    php)?;
-                let s = MPI::parse("dsa_signature_s_len", "dsa_signature_s",
+                let s = MPI::parse("dsa_sig_s_len", "dsa_sig_s",
                                    php)?;
 
                 Ok(mpis::Signature::DSA {
@@ -411,10 +416,10 @@ impl mpis::Signature {
             }
 
             ElgamalEncryptSign => {
-                let r = MPI::parse("elgamal_signature_r_len",
-                                   "elgamal_signature_r", php)?;
-                let s = MPI::parse("elgamal_signature_s_len",
-                                   "elgamal_signature_s", php)?;
+                let r = MPI::parse("elgamal_sig_r_len",
+                                   "elgamal_sig_r", php)?;
+                let s = MPI::parse("elgamal_sig_s_len",
+                                   "elgamal_sig_s", php)?;
 
                 Ok(mpis::Signature::Elgamal {
                     r: r,
@@ -423,9 +428,9 @@ impl mpis::Signature {
             }
 
             EdDSA => {
-                let r = MPI::parse("eddsa_signature_r_len", "eddsa_signature_r",
+                let r = MPI::parse("eddsa_sig_r_len", "eddsa_sig_r",
                                    php)?;
-                let s = MPI::parse("eddsa_signature_s_len", "eddsa_signature_s",
+                let s = MPI::parse("eddsa_sig_s_len", "eddsa_sig_s",
                                    php)?;
 
                 Ok(mpis::Signature::EdDSA {
@@ -435,9 +440,9 @@ impl mpis::Signature {
             }
 
             ECDSA => {
-                let r = MPI::parse("ecdsa_signature_r_len", "ecdsa_signature_r",
+                let r = MPI::parse("ecdsa_sig_r_len", "ecdsa_sig_r",
                                    php)?;
-                let s = MPI::parse("ecdsa_signature_s_len", "ecdsa_signature_s",
+                let s = MPI::parse("ecdsa_sig_s_len", "ecdsa_sig_s",
                                    php)?;
 
                 Ok(mpis::Signature::ECDSA {
@@ -448,8 +453,8 @@ impl mpis::Signature {
 
             Unknown(_) | Private(_) => {
                 let mut mpis = Vec::new();
-                while let Ok(mpi) = MPI::parse("unknown_parameter_len",
-                                               "unknown_parameter", php) {
+                while let Ok(mpi) = MPI::parse("unknown_len",
+                                               "unknown", php) {
                     mpis.push(mpi);
                 }
                 let rest = php.parse_bytes_eof("rest")?;
