@@ -36,7 +36,7 @@ pub enum Dearmor {
 /// for instance, `PacketParser::from_file` or
 /// `PacketParser::from_reader` to start parsing an OpenPGP message.
 pub struct PacketParserBuilder<'a> {
-    bio: Box<'a + BufferedReader<Cookie>>,
+    bio: Box<dyn BufferedReader<Cookie> + 'a>,
     dearmor: Dearmor,
     settings: PacketParserSettings,
 }
@@ -59,10 +59,10 @@ impl<'a> Parse<'a, PacketParserBuilder<'a>> for PacketParserBuilder<'a> {
 
     /// Creates a `PacketParserBuilder` for an OpenPGP message stored
     /// in the specified buffer.
-    fn from_bytes(bytes: &'a [u8]) -> Result<PacketParserBuilder> {
+    fn from_bytes<D: AsRef<[u8]> + ?Sized>(data: &'a D) -> Result<PacketParserBuilder<'a>> {
         PacketParserBuilder::from_buffered_reader(
             Box::new(buffered_reader::Memory::with_cookie(
-                bytes, Cookie::default())))
+                data.as_ref(), Cookie::default())))
     }
 }
 
@@ -72,7 +72,7 @@ impl<'a> PacketParserBuilder<'a> {
     //
     // Note: this clears the `level` field of the
     // `Cookie` cookie.
-    pub(crate) fn from_buffered_reader(mut bio: Box<'a + BufferedReader<Cookie>>)
+    pub(crate) fn from_buffered_reader(mut bio: Box<dyn BufferedReader<Cookie> + 'a>)
             -> Result<Self> {
         bio.cookie_mut().level = None;
         Ok(PacketParserBuilder {
@@ -170,22 +170,26 @@ impl<'a> PacketParserBuilder<'a> {
             Dearmor::Enabled(mode) => Some(mode),
             Dearmor::Disabled => None,
             Dearmor::Auto(mode) => {
-                let mut reader = buffered_reader::Dup::with_cookie(
-                    self.bio, Cookie::default());
-                let header = packet::Header::parse(&mut reader);
-                self.bio = Box::new(reader).into_inner().unwrap();
-                if let Ok(header) = header {
-                    if let Err(_) = header.valid(false) {
-                        // Invalid header: better try an ASCII armor
-                        // decoder.
-                        Some(mode)
-                    } else {
-                        None
-                    }
+                if self.bio.eof() {
+                    None
                 } else {
-                    // Failed to parse the header: better try an ASCII
-                    // armor decoder.
-                    Some(mode)
+                    let mut reader = buffered_reader::Dup::with_cookie(
+                        self.bio, Cookie::default());
+                    let header = packet::Header::parse(&mut reader);
+                    self.bio = Box::new(reader).into_inner().unwrap();
+                    if let Ok(header) = header {
+                        if let Err(_) = header.valid(false) {
+                            // Invalid header: better try an ASCII armor
+                            // decoder.
+                            Some(mode)
+                        } else {
+                            None
+                        }
+                    } else {
+                        // Failed to parse the header: better try an ASCII
+                        // armor decoder.
+                        Some(mode)
+                    }
                 }
             }
         };
@@ -203,7 +207,7 @@ impl<'a> PacketParserBuilder<'a> {
                 // We successfully parsed the first packet's header.
                 pp.state.message_validator.push(pp.packet.tag(), &[0]);
                 pp.state.keyring_validator.push(pp.packet.tag());
-                pp.state.tpk_validator.push(pp.packet.tag());
+                pp.state.cert_validator.push(pp.packet.tag());
                 Ok(PacketParserResult::Some(pp))
             },
             ParserResult::EOF((_reader, state, _path)) => {
@@ -237,13 +241,7 @@ mod tests {
         let ppr = PacketParserBuilder::from_bytes(msg).unwrap()
             .dearmor(Dearmor::Enabled(Default::default()))
             .finalize();
-        // XXX: If the dearmorer doesn't find a header and has no
-        // data, then it should return an error.  Fix this when
-        // https://gitlab.com/sequoia-pgp/sequoia/issues/174 is
-        // resolved.
-        //
-        // assert_match!(Err(_) = ppr);
-        assert_match!(Ok(PacketParserResult::EOF(ref _pp)) = ppr);
+        assert_match!(Err(_) = ppr);
 
         // ASCII armor encoded data.
         let msg = crate::tests::message("a-cypherpunks-manifesto.txt.ed25519.sig");
