@@ -4,7 +4,6 @@ use std::path::Path;
 use crate::{
     Result,
     Packet,
-    Container,
     PacketPile,
 };
 use crate::parse::{
@@ -34,19 +33,6 @@ use buffered_reader::BufferedReader;
 /// should be preferred.  If no per-packet processing needs to be
 /// done, then `PacketPile::from_file` will be slightly faster.
 ///
-/// Note: due to how lifetimes interact, it is not possible for the
-/// [`next()`] and [`recurse()`] methods to return a mutable reference
-/// to the packet (`&mut Packet`) that is currently being processed
-/// while continuing to support streaming operations.  It is also not
-/// possible to return a mutable reference to the `PacketParser`.
-/// Thus, we expose the `PacketParserResult` directly to the user.
-/// *However*, do *not* directly call `PacketParser::next()` or
-/// `PacketParser::recurse()`.  This will break the `PacketPileParser`
-/// implementation.
-///
-///   [`next()`]: #method.next
-///   [`recurse()`]: #method.recurse
-///
 /// # Examples
 ///
 /// ```rust
@@ -57,9 +43,10 @@ use buffered_reader::BufferedReader;
 /// #
 /// # fn f(message_data: &[u8]) -> Result<()> {
 /// let mut ppp = PacketPileParser::from_bytes(message_data)?;
-/// while ppp.recurse() {
-///     let pp = ppp.ppr.as_mut().unwrap();
+/// let mut ppr = ppp.recurse()?;
+/// while let Some(pp) = ppr.as_ref() {
 ///     eprintln!("{:?}", pp);
+///     ppr = ppp.recurse()?;
 /// }
 /// let message = ppp.finish();
 /// message.pretty_print();
@@ -69,7 +56,7 @@ use buffered_reader::BufferedReader;
 #[derive(Debug)]
 pub struct PacketPileParser<'a> {
     /// The current packet.
-    pub ppr: PacketParserResult<'a>,
+    ppr: PacketParserResult<'a>,
 
     /// Whether the first packet has been returned.
     returned_first: bool,
@@ -107,10 +94,10 @@ impl<'a> Parse<'a, PacketPileParser<'a>> for PacketPileParser<'a> {
 
     /// Creates a `PacketPileParser` to parse the OpenPGP message stored
     /// in the provided buffer.
-    fn from_bytes(data: &'a [u8])
+    fn from_bytes<D: AsRef<[u8]> + ?Sized>(data: &'a D)
             -> Result<PacketPileParser<'a>> {
         let bio = Box::new(buffered_reader::Memory::with_cookie(
-            data, Cookie::default()));
+            data.as_ref(), Cookie::default()));
         PacketPileParser::from_buffered_reader(bio)
     }
 }
@@ -121,7 +108,7 @@ impl<'a> PacketPileParser<'a> {
         -> Result<PacketPileParser<'a>>
     {
         Ok(PacketPileParser {
-            pile: PacketPile { top_level: Container::new() },
+            pile: PacketPile { top_level: Default::default() },
             ppr: ppr,
             returned_first: false,
         })
@@ -129,7 +116,7 @@ impl<'a> PacketPileParser<'a> {
 
     /// Creates a `PacketPileParser` to parse the OpenPGP message stored
     /// in the `BufferedReader` object.
-    pub(crate) fn from_buffered_reader(bio: Box<BufferedReader<Cookie> + 'a>)
+    pub(crate) fn from_buffered_reader(bio: Box<dyn BufferedReader<Cookie> + 'a>)
             -> Result<PacketPileParser<'a>> {
         Self::from_packet_parser(PacketParser::from_buffered_reader(bio)?)
     }
@@ -146,17 +133,12 @@ impl<'a> PacketPileParser<'a> {
             let tmp = container;
             let packets_len = tmp.packets.len();
             let p = &mut tmp.packets[packets_len - 1];
-            if p.children.is_none() {
-                if i == position - 1 {
-                    // This is the leaf.  Create a new container
-                    // here.
-                    p.children = Some(Container::new());
-                } else {
-                    panic!("Internal inconsistency while building message.");
-                }
+            if p.children().next().is_none() {
+                assert!(i == position - 1,
+                        "Internal inconsistency while building message.");
             }
 
-            container = p.children.as_mut().unwrap();
+            container = p.container_mut().unwrap();
         }
 
         container.packets.push(packet);
@@ -170,26 +152,15 @@ impl<'a> PacketPileParser<'a> {
     /// packet parser for the next packet.  If the current packet is a
     /// container, this function tries to recurse into it.  Otherwise,
     /// it returns the following packet.
-    ///
-    /// Due to lifetime issues, this function does not return a
-    /// reference to the `PacketParser`, but a boolean indicating
-    /// whether a new packet is available.  Instead, the
-    /// `PacketParser` can be accessed as `self.ppo`.
-    pub fn recurse(&mut self) -> bool {
+    pub fn recurse(&mut self) -> Result<&mut PacketParserResult<'a>> {
         if self.returned_first {
             match self.ppr.take() {
                 PacketParserResult::Some(pp) => {
-                    match pp.recurse() {
-                        Ok((packet, ppr)) => {
-                            self.insert_packet(
-                                packet,
-                                ppr.last_recursion_depth().unwrap() as isize);
-                            self.ppr = ppr;
-                        }
-                        Err(_) => {
-                            // XXX: What should we do with the error?
-                        }
-                    }
+                    let (packet, ppr) = pp.recurse()?;
+                    self.insert_packet(
+                        packet,
+                        ppr.last_recursion_depth().unwrap() as isize);
+                    self.ppr = ppr;
                 }
                 eof @ PacketParserResult::EOF(_) => {
                     self.ppr = eof;
@@ -199,7 +170,7 @@ impl<'a> PacketPileParser<'a> {
             self.returned_first = true;
         }
 
-        !self.is_done()
+        Ok(&mut self.ppr)
     }
 
     /// Finishes parsing the current packet and starts parsing the
@@ -210,26 +181,15 @@ impl<'a> PacketPileParser<'a> {
     /// packet parser for the following packet.  If the current packet
     /// is a container, this function does *not* recurse into the
     /// container; it skips any packets that it may contain.
-    ///
-    /// Due to lifetime issues, this function does not return a
-    /// reference to the `PacketParser`, but a boolean indicating
-    /// whether a new packet is available.  Instead, the
-    /// `PacketParser` can be accessed as `self.ppo`.
-    pub fn next(&mut self) -> bool {
+    pub fn next(&mut self) -> Result<&mut PacketParserResult<'a>> {
         if self.returned_first {
             match self.ppr.take() {
                 PacketParserResult::Some(pp) => {
-                    match pp.next() {
-                        Ok((packet, ppr)) => {
-                            self.insert_packet(
-                                packet,
-                                ppr.last_recursion_depth().unwrap() as isize);
-                            self.ppr = ppr;
-                        }
-                        Err(_) => {
-                            // XXX: What should we do with the error?
-                        }
-                    }
+                    let (packet, ppr) = pp.next()?;
+                    self.insert_packet(
+                        packet,
+                        ppr.last_recursion_depth().unwrap() as isize);
+                    self.ppr = ppr;
                 },
                 eof @ PacketParserResult::EOF(_) => {
                     self.ppr = eof
@@ -239,7 +199,7 @@ impl<'a> PacketPileParser<'a> {
             self.returned_first = true;
         }
 
-        !self.is_done()
+        Ok(&mut self.ppr)
     }
 
     /// Returns the current packet's recursion depth.
@@ -276,11 +236,13 @@ impl<'a> PacketPileParser<'a> {
 #[test]
 fn message_parser_test() {
     let mut count = 0;
-    let mut mp =
+    let mut ppp =
         PacketPileParser::from_bytes(crate::tests::key("public-key.gpg"))
         .unwrap();
-    while mp.recurse() {
+    let mut ppr = ppp.recurse().unwrap();
+    while ppr.is_some() {
         count += 1;
+        ppr = ppp.recurse().unwrap();
     }
     assert_eq!(count, 61);
 }
@@ -296,11 +258,11 @@ fn message_parser_reader_interface() {
 
     // A message containing a compressed packet that contains a
     // literal packet.
-    let mut mp = PacketPileParser::from_bytes(
+    let mut ppp = PacketPileParser::from_bytes(
         crate::tests::message("compressed-data-algo-1.gpg")).unwrap();
     let mut count = 0;
-    while mp.recurse() {
-        let pp = mp.ppr.as_mut().unwrap();
+    let mut ppr = ppp.recurse().unwrap();
+    while let Some(pp) = ppr.as_mut() {
         count += 1;
         if let Packet::Literal(_) = pp.packet {
             assert_eq!(count, 2);
@@ -318,6 +280,7 @@ fn message_parser_reader_interface() {
             let r = pp.read(&mut buf).unwrap();
             assert_eq!(r, 0);
         }
+        ppr = ppp.recurse().unwrap();
     }
     assert_eq!(count, 2);
 }
