@@ -5,35 +5,44 @@
 //! already assembled in memory, with all information already in place
 //! (e.g. because it was parsed).
 //!
-//! If it is, you can use the `Serialize` or `SerializeKey`.
+//! If it is, you can use the [`Serialize`] or [`SerializeInto`]
+//! trait.  Otherwise, please use our [streaming serialization
+//! interface].
 //!
-//! Otherwise, please use our streaming serialization interface.
+//!   [`Serialize`]: trait.Serialize.html
+//!   [`SerializeInto`]: trait.SerializeInto.html
+//!   [streaming serialization interface]: stream/index.html
 
 use std::io::{self, Write};
 use std::cmp;
+use std::convert::TryFrom;
 
-use crate::autocrypt;
 use super::*;
 
 mod partial_body;
 mod sexp;
-mod tpk;
-pub use self::tpk::TSK;
-mod tpk_armored;
+mod cert;
+pub use self::cert::TSK;
+mod cert_armored;
 use self::partial_body::PartialBodyFilter;
 pub mod writer;
 pub mod stream;
 #[cfg(feature = "compression-deflate")]
 pub mod padding;
-use crate::crypto::s2k::S2K;
-use crate::packet::signature::subpacket::{
-    Subpacket, SubpacketValue, SubpacketLengthTrait,
+use crate::crypto::S2K;
+use crate::packet::header::{
+    BodyLength,
+    CTB,
+    CTBNew,
+    CTBOld,
 };
-use crate::conversions::{
-    Time,
-    Duration,
+use crate::packet::signature::subpacket::{
+    SubpacketArea, Subpacket, SubpacketValue,
 };
 use crate::packet::prelude::*;
+use crate::types::{
+    Timestamp,
+};
 
 // Whether to trace the modules execution (on stderr).
 const TRACE : bool = false;
@@ -49,13 +58,13 @@ pub trait Serialize {
     ///
     ///   - It is an error to export a [`Signature`] if it is marked
     ///     as non-exportable.
-    ///   - When exporting a [`TPK`], non-exportable signatures are
+    ///   - When exporting a [`Cert`], non-exportable signatures are
     ///     not exported, and any component bound merely by
     ///     non-exportable signatures is not exported.
     ///
     ///   [`serialize(..)`]: #tymethod.serialize
     ///   [`Signature`]: ../packet/enum.Signature.html
-    ///   [`TPK`]: ../struct.TPK.html
+    ///   [`Cert`]: ../struct.Cert.html
     fn export(&self, o: &mut dyn std::io::Write) -> Result<()> {
         self.serialize(o)
     }
@@ -86,7 +95,7 @@ pub trait SerializeInto {
     fn to_vec(&self) -> Result<Vec<u8>> {
         let mut o = vec![0; self.serialized_len()];
         let len = self.serialize_into(&mut o[..])?;
-        o.truncate(len);
+        vec_truncate(&mut o, len);
         o.shrink_to_fit();
         Ok(o)
     }
@@ -98,13 +107,13 @@ pub trait SerializeInto {
     ///
     ///   - It is an error to export a [`Signature`] if it is marked
     ///     as non-exportable.
-    ///   - When exporting a [`TPK`], non-exportable signatures are
+    ///   - When exporting a [`Cert`], non-exportable signatures are
     ///     not exported, and any component bound merely by
     ///     non-exportable signatures is not exported.
     ///
     ///   [`serialize_into(..)`]: #tymethod.serialize_into
     ///   [`Signature`]: ../packet/enum.Signature.html
-    ///   [`TPK`]: ../struct.TPK.html
+    ///   [`Cert`]: ../struct.Cert.html
     ///
     /// Returns the length of the serialized representation.
     ///
@@ -123,17 +132,17 @@ pub trait SerializeInto {
     ///
     ///   - It is an error to export a [`Signature`] if it is marked
     ///     as non-exportable.
-    ///   - When exporting a [`TPK`], non-exportable signatures are
+    ///   - When exporting a [`Cert`], non-exportable signatures are
     ///     not exported, and any component bound merely by
     ///     non-exportable signatures is not exported.
     ///
     ///   [`to_vec()`]: #method.to_vec
     ///   [`Signature`]: ../packet/enum.Signature.html
-    ///   [`TPK`]: ../struct.TPK.html
+    ///   [`Cert`]: ../struct.Cert.html
     fn export_to_vec(&self) -> Result<Vec<u8>> {
         let mut o = vec![0; self.serialized_len()];
         let len = self.export_into(&mut o[..])?;
-        o.truncate(len);
+        vec_truncate(&mut o, len);
         o.shrink_to_fit();
         Ok(o)
     }
@@ -299,8 +308,8 @@ impl Serialize for BodyLength {
     /// [`BodyLength::Indeterminate`].  If you want to serialize an
     /// old-style length, use [`serialize_old(..)`].
     ///
-    /// [`Error::InvalidArgument`]: ../enum.Error.html#variant.InvalidArgument
-    /// [`BodyLength::Indeterminate`]: ../packet/enum.BodyLength.html#variant.Indeterminate
+    /// [`Error::InvalidArgument`]: ../../enum.Error.html#variant.InvalidArgument
+    /// [`BodyLength::Indeterminate`]: #variant.Indeterminate
     /// [`serialize_old(..)`]: #method.serialize_old
     fn serialize(&self, o: &mut dyn std::io::Write) -> Result<()> {
         match self {
@@ -377,8 +386,8 @@ impl BodyLength {
     /// [`BodyLength::Partial`].  If you want to serialize a
     /// new-style length, use [`serialize(..)`].
     ///
-    /// [`Error::InvalidArgument`]: ../enum.Error.html#variant.InvalidArgument
-    /// [`BodyLength::Partial`]: ../packet/enum.BodyLength.html#variant.Partial
+    /// [`Error::InvalidArgument`]: ../../enum.Error.html#variant.InvalidArgument
+    /// [`BodyLength::Partial`]: #variant.Partial
     /// [`serialize(..)`]: #impl-Serialize
     pub fn serialize_old<W: io::Write>(&self, o: &mut W) -> Result<()> {
         // Assume an optimal encoding is desired.
@@ -388,10 +397,10 @@ impl BodyLength {
                 match l {
                     // One octet length.
                     // write_byte can't fail for a Vec.
-                    0 ... 0xFF =>
+                    0 ..= 0xFF =>
                         write_byte(&mut buffer, l as u8).unwrap(),
                     // Two octet length.
-                    0x1_00 ... 0xFF_FF =>
+                    0x1_00 ..= 0xFF_FF =>
                         write_be_u16(&mut buffer, l as u16).unwrap(),
                     // Four octet length,
                     _ =>
@@ -412,7 +421,7 @@ impl BodyLength {
 
 impl Serialize for CTBNew {
     fn serialize(&self, o: &mut dyn std::io::Write) -> Result<()> {
-        let tag: u8 = self.common.tag.into();
+        let tag: u8 = self.tag().into();
         o.write_all(&[0b1100_0000u8 | tag])?;
         Ok(())
     }
@@ -428,7 +437,7 @@ impl SerializeInto for CTBNew {
 
 impl Serialize for CTBOld {
     fn serialize(&self, o: &mut dyn std::io::Write) -> Result<()> {
-        let tag: u8 = self.common.tag.into();
+        let tag: u8 = self.tag().into();
         let length_type: u8 = self.length_type.into();
         o.write_all(&[0b1000_0000u8 | (tag << 2) | length_type])?;
         Ok(())
@@ -463,8 +472,8 @@ impl SerializeInto for CTB {
 
 impl Serialize for Header {
     fn serialize(&self, o: &mut dyn std::io::Write) -> Result<()> {
-        self.ctb.serialize(o)?;
-        self.length.serialize(o)?;
+        self.ctb().serialize(o)?;
+        self.length().serialize(o)?;
         Ok(())
     }
 }
@@ -917,21 +926,14 @@ impl SerializeInto for S2K {
 
 impl Serialize for Unknown {
     fn serialize(&self, o: &mut dyn std::io::Write) -> Result<()> {
-        let body = if let Some(ref body) = self.common.body {
-            &body[..]
-        } else {
-            &b""[..]
-        };
-
-        o.write_all(&body[..])?;
-
+        o.write_all(self.body())?;
         Ok(())
     }
 }
 
 impl NetLength for Unknown {
     fn net_len(&self) -> usize {
-        self.body().unwrap_or(&b""[..]).len()
+        self.body().len()
     }
 }
 
@@ -945,21 +947,44 @@ impl SerializeInto for Unknown {
     }
 }
 
-impl<'a> Serialize for Subpacket<'a> {
+impl Serialize for SubpacketArea {
     fn serialize(&self, o: &mut dyn std::io::Write) -> Result<()> {
-        let tag = u8::from(self.tag)
-            | if self.critical { 1 << 7 } else { 0 };
-        let len = 1 + self.value.len();
-
-        len.serialize(o)?;
-        o.write_all(&[tag])?;
-        self.value.serialize(o)
+        for sb in self.iter() {
+            sb.serialize(o)?;
+        }
+        Ok(())
     }
 }
 
-impl<'a> SerializeInto for Subpacket<'a> {
+impl SerializeInto for SubpacketArea {
     fn serialized_len(&self) -> usize {
-        (1 + self.value.len()).len() + 1 + self.value.serialized_len()
+        self.iter().map(|sb| sb.serialized_len()).sum()
+    }
+
+    fn serialize_into(&self, buf: &mut [u8]) -> Result<usize> {
+        let mut written = 0;
+        for sb in self.iter() {
+            let n = sb.serialize_into(&mut buf[written..])?;
+            written += cmp::min(buf.len() - written, n);
+        }
+        Ok(written)
+    }
+}
+
+impl Serialize for Subpacket {
+    fn serialize(&self, o: &mut dyn std::io::Write) -> Result<()> {
+        let tag = u8::from(self.tag())
+            | if self.critical() { 1 << 7 } else { 0 };
+
+        self.length.serialize(o)?;
+        o.write_all(&[tag])?;
+        self.value().serialize(o)
+    }
+}
+
+impl SerializeInto for Subpacket {
+    fn serialized_len(&self) -> usize {
+        self.length.serialized_len() + 1 + self.value().serialized_len()
     }
 
     fn serialize_into(&self, buf: &mut [u8]) -> Result<usize> {
@@ -967,14 +992,14 @@ impl<'a> SerializeInto for Subpacket<'a> {
     }
 }
 
-impl<'a> Serialize for SubpacketValue<'a> {
+impl Serialize for SubpacketValue {
     fn serialize(&self, o: &mut dyn std::io::Write) -> Result<()> {
         use self::SubpacketValue::*;
         match self {
             SignatureCreationTime(t) =>
-                write_be_u32(o, t.to_pgp()?)?,
+                write_be_u32(o, t.clone().into())?,
             SignatureExpirationTime(t) =>
-                write_be_u32(o, t.to_pgp()?)?,
+                write_be_u32(o, t.clone().into())?,
             ExportableCertification(e) =>
                 o.write_all(&[if *e { 1 } else { 0 }])?,
             TrustSignature { ref level, ref trust } =>
@@ -986,7 +1011,7 @@ impl<'a> Serialize for SubpacketValue<'a> {
             Revocable(r) =>
                 o.write_all(&[if *r { 1 } else { 0 }])?,
             KeyExpirationTime(t) =>
-                write_be_u32(o, t.to_pgp()?)?,
+                write_be_u32(o, t.clone().into())?,
             PreferredSymmetricAlgorithms(ref p) =>
                 for a in p {
                     o.write_all(&[(*a).into()])?;
@@ -1001,7 +1026,7 @@ impl<'a> Serialize for SubpacketValue<'a> {
                 write_be_u32(o, nd.flags().raw())?;
                 write_be_u16(o, nd.name().len() as u16)?;
                 write_be_u16(o, nd.value().len() as u16)?;
-                o.write_all(nd.name())?;
+                o.write_all(nd.name().as_bytes())?;
                 o.write_all(nd.value())?;
             },
             PreferredHashAlgorithms(ref p) =>
@@ -1013,7 +1038,7 @@ impl<'a> Serialize for SubpacketValue<'a> {
                     o.write_all(&[(*a).into()])?;
                 },
             KeyServerPreferences(ref p) =>
-                o.write_all(&p.as_vec())?,
+                o.write_all(&p.to_vec())?,
             PreferredKeyServer(ref p) =>
                 o.write_all(p)?,
             PrimaryUserID(p) =>
@@ -1021,7 +1046,7 @@ impl<'a> Serialize for SubpacketValue<'a> {
             PolicyURI(ref p) =>
                 o.write_all(p)?,
             KeyFlags(ref f) =>
-                o.write_all(&f.as_vec())?,
+                o.write_all(&f.to_vec())?,
             SignersUserID(ref uid) =>
                 o.write_all(uid)?,
             ReasonForRevocation { ref code, ref reason } => {
@@ -1029,16 +1054,12 @@ impl<'a> Serialize for SubpacketValue<'a> {
                 o.write_all(reason)?;
             },
             Features(ref f) =>
-                o.write_all(&f.as_vec())?,
+                o.write_all(&f.to_vec())?,
             SignatureTarget { pk_algo, hash_algo, ref digest } => {
                 o.write_all(&[(*pk_algo).into(), (*hash_algo).into()])?;
                 o.write_all(digest)?;
             },
-            EmbeddedSignature(ref p) => match p {
-                &Packet::Signature(ref sig) => sig.serialize(o)?,
-                _ => return Err(Error::InvalidArgument(
-                    format!("Not a signature: {:?}", p)).into()),
-            },
+            EmbeddedSignature(sig) => sig.serialize(o)?,
             IssuerFingerprint(ref fp) => match fp {
                 Fingerprint::V4(_) => {
                     o.write_all(&[4])?;
@@ -1059,16 +1080,15 @@ impl<'a> Serialize for SubpacketValue<'a> {
                 _ => return Err(Error::InvalidArgument(
                     "Unknown kind of fingerprint".into()).into()),
             }
-            Unknown(ref raw) =>
-                o.write_all(raw)?,
-            Invalid(ref raw) =>
-                o.write_all(raw)?,
+            Unknown { body, .. } =>
+                o.write_all(body)?,
+            __Nonexhaustive => unreachable!(),
         }
         Ok(())
     }
 }
 
-impl<'a> SerializeInto for SubpacketValue<'a> {
+impl SerializeInto for SubpacketValue {
     fn serialized_len(&self) -> usize {
         use self::SubpacketValue::*;
         match self {
@@ -1085,19 +1105,16 @@ impl<'a> SerializeInto for SubpacketValue<'a> {
             NotationData(nd) => 4 + 2 + 2 + nd.name().len() + nd.value().len(),
             PreferredHashAlgorithms(ref p) => p.len(),
             PreferredCompressionAlgorithms(ref p) => p.len(),
-            KeyServerPreferences(ref p) => p.as_vec().len(),
+            KeyServerPreferences(ref p) => p.to_vec().len(),
             PreferredKeyServer(ref p) => p.len(),
             PrimaryUserID(_) => 1,
             PolicyURI(ref p) => p.len(),
-            KeyFlags(ref f) => f.as_vec().len(),
+            KeyFlags(ref f) => f.to_vec().len(),
             SignersUserID(ref uid) => uid.len(),
             ReasonForRevocation { ref reason, .. } => 1 + reason.len(),
-            Features(ref f) => f.as_vec().len(),
+            Features(ref f) => f.to_vec().len(),
             SignatureTarget { ref digest, .. } => 2 + digest.len(),
-            EmbeddedSignature(ref p) => match p {
-                &Packet::Signature(ref sig) => sig.serialized_len(),
-                _ => 0,
-            },
+            EmbeddedSignature(sig) => sig.serialized_len(),
             IssuerFingerprint(ref fp) => match fp {
                 Fingerprint::V4(_) => 1 + fp.serialized_len(),
                 _ => 0,
@@ -1107,8 +1124,8 @@ impl<'a> SerializeInto for SubpacketValue<'a> {
                 Fingerprint::V4(_) => 1 + fp.serialized_len(),
                 _ => 0,
             },
-            Unknown(ref raw) => raw.len(),
-            Invalid(ref raw) => raw.len(),
+            Unknown { body, .. } => body.len(),
+            __Nonexhaustive => unreachable!(),
         }
     }
 
@@ -1121,12 +1138,14 @@ impl Serialize for Signature {
     fn serialize(&self, o: &mut dyn std::io::Write) -> Result<()> {
         match self {
             &Signature::V4(ref s) => s.serialize(o),
+            Signature::__Nonexhaustive => unreachable!(),
         }
     }
 
     fn export(&self, o: &mut dyn std::io::Write) -> Result<()> {
         match self {
             &Signature::V4(ref s) => s.export(o),
+            Signature::__Nonexhaustive => unreachable!(),
         }
     }
 }
@@ -1135,24 +1154,28 @@ impl SerializeInto for Signature {
     fn serialized_len(&self) -> usize {
         match self {
             &Signature::V4(ref s) => s.serialized_len(),
+            Signature::__Nonexhaustive => unreachable!(),
         }
     }
 
     fn serialize_into(&self, buf: &mut [u8]) -> Result<usize> {
         match self {
             &Signature::V4(ref s) => s.serialize_into(buf),
+            Signature::__Nonexhaustive => unreachable!(),
         }
     }
 
     fn export_into(&self, buf: &mut [u8]) -> Result<usize> {
         match self {
             &Signature::V4(ref s) => s.export_into(buf),
+            Signature::__Nonexhaustive => unreachable!(),
         }
     }
 
     fn export_to_vec(&self) -> Result<Vec<u8>> {
         match self {
             &Signature::V4(ref s) => s.export_to_vec(),
+            Signature::__Nonexhaustive => unreachable!(),
         }
     }
 }
@@ -1174,22 +1197,24 @@ impl Serialize for Signature4 {
         write_byte(o, self.pk_algo().into())?;
         write_byte(o, self.hash_algo().into())?;
 
-        if self.hashed_area().data.len() > std::u16::MAX as usize {
+        let l = self.hashed_area().serialized_len();
+        if l > std::u16::MAX as usize {
             return Err(Error::InvalidArgument(
                 "Hashed area too large".into()).into());
         }
-        write_be_u16(o, self.hashed_area().data.len() as u16)?;
-        o.write_all(&self.hashed_area().data[..])?;
+        write_be_u16(o, l as u16)?;
+        self.hashed_area().serialize(o)?;
 
-        if self.unhashed_area().data.len() > std::u16::MAX as usize {
+        let l = self.unhashed_area().serialized_len();
+        if l > std::u16::MAX as usize {
             return Err(Error::InvalidArgument(
                 "Unhashed area too large".into()).into());
         }
-        write_be_u16(o, self.unhashed_area().data.len() as u16)?;
-        o.write_all(&self.unhashed_area().data[..])?;
+        write_be_u16(o, l as u16)?;
+        self.unhashed_area().serialize(o)?;
 
-        write_byte(o, self.hash_prefix()[0])?;
-        write_byte(o, self.hash_prefix()[1])?;
+        write_byte(o, self.digest_prefix()[0])?;
+        write_byte(o, self.digest_prefix()[1])?;
 
         self.mpis().serialize(o)?;
 
@@ -1213,9 +1238,9 @@ impl NetLength for Signature4 {
             + 1 // PK algorithm.
             + 1 // Hash algorithm.
             + 2 // Hashed area size.
-            + self.hashed_area().data.len()
+            + self.hashed_area().serialized_len()
             + 2 // Unhashed area size.
-            + self.unhashed_area().data.len()
+            + self.unhashed_area().serialized_len()
             + 2 // Hash prefix.
             + self.mpis().serialized_len()
     }
@@ -1253,6 +1278,7 @@ impl Serialize for OnePassSig {
     fn serialize(&self, o: &mut dyn std::io::Write) -> Result<()> {
         match self {
             &OnePassSig::V3(ref s) => s.serialize(o),
+            OnePassSig::__Nonexhaustive => unreachable!(),
         }
     }
 }
@@ -1261,12 +1287,14 @@ impl SerializeInto for OnePassSig {
     fn serialized_len(&self) -> usize {
         match self {
             &OnePassSig::V3(ref s) => s.serialized_len(),
+            OnePassSig::__Nonexhaustive => unreachable!(),
         }
     }
 
     fn serialize_into(&self, buf: &mut [u8]) -> Result<usize> {
         match self {
             &OnePassSig::V3(ref s) => s.serialize_into(buf),
+            OnePassSig::__Nonexhaustive => unreachable!(),
         }
     }
 }
@@ -1306,9 +1334,10 @@ impl SerializeInto for OnePassSig3 {
 }
 
 impl<P: key::KeyParts, R: key::KeyRole> Serialize for Key<P, R> {
-    fn serialize(&self, o: &mut io::Write) -> Result<()> {
+    fn serialize(&self, o: &mut dyn io::Write) -> Result<()> {
         match self {
             &Key::V4(ref p) => p.serialize(o),
+            Key::__Nonexhaustive => unreachable!(),
         }
     }
 }
@@ -1317,6 +1346,7 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
     fn net_len_key(&self, serialize_secrets: bool) -> usize {
         match self {
             &Key::V4(ref p) => p.net_len_key(serialize_secrets),
+            Key::__Nonexhaustive => unreachable!(),
         }
     }
 }
@@ -1325,12 +1355,14 @@ impl<P: key::KeyParts, R: key::KeyRole> SerializeInto for Key<P, R> {
     fn serialized_len(&self) -> usize {
         match self {
             &Key::V4(ref p) => p.serialized_len(),
+            Key::__Nonexhaustive => unreachable!(),
         }
     }
 
     fn serialize_into(&self, buf: &mut [u8]) -> Result<usize> {
         match self {
             &Key::V4(ref p) => p.serialize_into(buf),
+            Key::__Nonexhaustive => unreachable!(),
         }
     }
 }
@@ -1339,7 +1371,7 @@ impl<P, R> Serialize for Key4<P, R>
     where P: key::KeyParts,
           R: key::KeyRole,
 {
-    fn serialize(&self, o: &mut io::Write) -> Result<()> {
+    fn serialize(&self, o: &mut dyn io::Write) -> Result<()> {
         self.serialize_key(o, true)
     }
 }
@@ -1349,12 +1381,12 @@ impl<P, R> Key4<P, R>
           R: key::KeyRole,
 {
     pub(crate) // For tests in key.
-    fn serialize_key(&self, o: &mut io::Write, serialize_secrets: bool)
+    fn serialize_key(&self, o: &mut dyn io::Write, serialize_secrets: bool)
                      -> Result<()> {
         let have_secret_key = self.secret().is_some() && serialize_secrets;
 
         write_byte(o, 4)?; // Version.
-        write_be_u32(o, self.creation_time().to_pgp()?)?;
+        write_be_u32(o, Timestamp::try_from(self.creation_time())?.into())?;
         write_byte(o, self.pk_algo().into())?;
         self.mpis().serialize(o)?;
 
@@ -1600,14 +1632,14 @@ impl Literal {
         };
 
         let date = if let Some(d) = self.date() {
-            d.to_pgp()?
+            Timestamp::try_from(d)?.into()
         } else {
             0
         };
 
         if write_tag {
             let len = 1 + (1 + filename.len()) + 4
-                + self.common.body.as_ref().map(|b| b.len()).unwrap_or(0);
+                + self.body().len();
             CTB::new(Tag::Literal).serialize(o)?;
             BodyLength::Full(len as u32).serialize(o)?;
         }
@@ -1621,12 +1653,7 @@ impl Literal {
 
 impl Serialize for Literal {
     fn serialize(&self, o: &mut dyn std::io::Write) -> Result<()> {
-        let body = if let Some(ref body) = self.common.body {
-            &body[..]
-        } else {
-            &b""[..]
-        };
-
+        let body = self.body();
         if TRACE {
             let prefix = &body[..cmp::min(body.len(), 20)];
             eprintln!("Literal::serialize({}{}, {} bytes)",
@@ -1645,7 +1672,7 @@ impl Serialize for Literal {
 impl NetLength for Literal {
     fn net_len(&self) -> usize {
         1 + (1 + self.filename().map(|f| f.len()).unwrap_or(0)) + 4
-            + self.common.body.as_ref().map(|b| b.len()).unwrap_or(0)
+            + self.body().len()
     }
 }
 
@@ -1669,28 +1696,22 @@ impl Serialize for CompressedData {
         if TRACE {
             eprintln!("CompressedData::serialize(\
                        algo: {}, {:?} children, {:?} bytes)",
-                      self.algorithm(),
-                      self.common.children.as_ref().map(
-                          |cont| cont.children().len()),
-                      self.common.body.as_ref().map(|body| body.len()));
+                      self.algo(),
+                      self.children().count(),
+                      self.body().len());
         }
 
         let o = stream::Message::new(o);
-        let mut o =
-            stream::Compressor::new_naked(o, self.algorithm(), None, 0)?;
+        let mut o = stream::Compressor::new_naked(
+            o, self.algo(), Default::default(), 0)?;
 
         // Serialize the packets.
-        if let Some(ref children) = self.common.children {
-            for p in children.children() {
-                p.serialize(&mut o)?;
-            }
+        for p in self.children() {
+            p.serialize(&mut o)?;
         }
 
         // Append the data.
-        if let Some(ref data) = self.common.body {
-            o.write_all(data)?;
-        }
-
+        o.write_all(self.body())?;
         o.finalize()
     }
 }
@@ -1698,11 +1719,8 @@ impl Serialize for CompressedData {
 impl NetLength for CompressedData {
     fn net_len(&self) -> usize {
         let inner_length =
-            self.common.children.as_ref().map(|children| {
-                children.packets.iter().map(|p| p.serialized_len())
-                    .sum()
-            }).unwrap_or(0)
-            + self.common.body.as_ref().map(|body| body.len()).unwrap_or(0);
+            self.children().map(|p| p.serialized_len()).sum::<usize>()
+            + self.body().len();
 
         // Worst case, the data gets larger.  Account for that.
         let inner_length = inner_length + cmp::max(inner_length / 2, 128);
@@ -1738,6 +1756,7 @@ impl Serialize for PKESK {
     fn serialize(&self, o: &mut dyn std::io::Write) -> Result<()> {
         match self {
             &PKESK::V3(ref p) => p.serialize(o),
+            PKESK::__Nonexhaustive => unreachable!(),
         }
     }
 }
@@ -1746,12 +1765,14 @@ impl SerializeInto for PKESK {
     fn serialized_len(&self) -> usize {
         match self {
             &PKESK::V3(ref p) => p.serialized_len(),
+            PKESK::__Nonexhaustive => unreachable!(),
         }
     }
 
     fn serialize_into(&self, buf: &mut [u8]) -> Result<usize> {
         match self {
             &PKESK::V3(ref p) => generic_serialize_into(p, buf),
+            PKESK::__Nonexhaustive => unreachable!(),
         }
     }
 }
@@ -1791,6 +1812,7 @@ impl Serialize for SKESK {
         match self {
             &SKESK::V4(ref s) => s.serialize(o),
             &SKESK::V5(ref s) => s.serialize(o),
+            SKESK::__Nonexhaustive => unreachable!(),
         }
     }
 }
@@ -1800,6 +1822,7 @@ impl NetLength for SKESK {
         match self {
             &SKESK::V4(ref s) => s.net_len(),
             &SKESK::V5(ref s) => s.net_len(),
+            SKESK::__Nonexhaustive => unreachable!(),
         }
     }
 }
@@ -1809,6 +1832,7 @@ impl SerializeInto for SKESK {
         match self {
             &SKESK::V4(ref s) => s.serialized_len(),
             &SKESK::V5(ref s) => s.serialized_len(),
+            SKESK::__Nonexhaustive => unreachable!(),
         }
     }
 
@@ -1816,6 +1840,7 @@ impl SerializeInto for SKESK {
         match self {
             &SKESK::V4(ref s) => generic_serialize_into(s, buf),
             &SKESK::V5(ref s) => generic_serialize_into(s, buf),
+            SKESK::__Nonexhaustive => unreachable!(),
         }
     }
 }
@@ -1900,15 +1925,13 @@ impl Serialize for SEIP {
     /// To construct an encrypted message, use
     /// `serialize::stream::Encryptor`.
     fn serialize(&self, o: &mut dyn std::io::Write) -> Result<()> {
-        if let Some(ref _children) = self.common.children {
+        if self.children().next().is_some() {
             return Err(Error::InvalidOperation(
                 "Cannot encrypt, use serialize::stream::Encryptor".into())
                        .into());
         } else {
             o.write_all(&[self.version()])?;
-            if let Some(ref body) = self.common.body {
-                o.write_all(&body[..])?;
-            }
+            o.write_all(self.body())?;
         }
 
         Ok(())
@@ -1918,13 +1941,13 @@ impl Serialize for SEIP {
 impl NetLength for SEIP {
     fn net_len(&self) -> usize {
         1 // Version.
-            + self.common.body.as_ref().map(|b| b.len()).unwrap_or(0)
+            + self.body().len()
     }
 }
 
 impl SerializeInto for SEIP {
     fn serialized_len(&self) -> usize {
-        if self.common.children.is_some() {
+        if self.children().next().is_some() {
             0 // XXX
         } else {
             self.gross_len()
@@ -1938,7 +1961,7 @@ impl SerializeInto for SEIP {
 
 impl Serialize for MDC {
     fn serialize(&self, o: &mut dyn std::io::Write) -> Result<()> {
-        o.write_all(self.hash())?;
+        o.write_all(self.digest())?;
         Ok(())
     }
 }
@@ -1963,6 +1986,7 @@ impl Serialize for AED {
     fn serialize(&self, o: &mut dyn std::io::Write) -> Result<()> {
         match self {
             &AED::V1(ref p) => p.serialize(o),
+            AED::__Nonexhaustive => unreachable!(),
         }
     }
 }
@@ -1971,12 +1995,14 @@ impl SerializeInto for AED {
     fn serialized_len(&self) -> usize {
         match self {
             &AED::V1(ref p) => p.serialized_len(),
+            AED::__Nonexhaustive => unreachable!(),
         }
     }
 
     fn serialize_into(&self, buf: &mut [u8]) -> Result<usize> {
         match self {
             &AED::V1(ref p) => p.serialize_into(buf),
+            AED::__Nonexhaustive => unreachable!(),
         }
     }
 }
@@ -2003,16 +2029,13 @@ impl Serialize for AED1 {
     /// To construct an encrypted message, use
     /// `serialize::stream::Encryptor`.
     fn serialize(&self, o: &mut dyn std::io::Write) -> Result<()> {
-        if let Some(ref _children) = self.common.children {
+        if self.children().next().is_some() {
             return Err(Error::InvalidOperation(
                 "Cannot encrypt, use serialize::stream::Encryptor".into())
                        .into());
         } else {
             self.serialize_headers(o)?;
-
-            if let Some(ref body) = self.common.body {
-                o.write_all(&body[..])?;
-            }
+            o.write_all(self.body())?;
         }
 
         Ok(())
@@ -2021,12 +2044,12 @@ impl Serialize for AED1 {
 
 impl NetLength for AED1 {
     fn net_len(&self) -> usize {
-        if self.common.children.is_some() {
+        if self.children().next().is_some() {
             0
         } else {
             4 // Headers.
                 + self.iv().len()
-                + self.common.body.as_ref().map(|b| b.len()).unwrap_or(0)
+                + self.body().len()
         }
     }
 }
@@ -2080,6 +2103,7 @@ impl Serialize for Packet {
             &Packet::SEIP(ref p) => p.serialize(o),
             &Packet::MDC(ref p) => p.serialize(o),
             &Packet::AED(ref p) => p.serialize(o),
+            Packet::__Nonexhaustive => unreachable!(),
         }
     }
 
@@ -2121,6 +2145,7 @@ impl Serialize for Packet {
             &Packet::SEIP(ref p) => p.export(o),
             &Packet::MDC(ref p) => p.export(o),
             &Packet::AED(ref p) => p.export(o),
+            Packet::__Nonexhaustive => unreachable!(),
         }
     }
 }
@@ -2146,6 +2171,7 @@ impl NetLength for Packet {
             &Packet::SEIP(ref p) => p.net_len(),
             &Packet::MDC(ref p) => p.net_len(),
             &Packet::AED(ref p) => p.net_len(),
+            Packet::__Nonexhaustive => unreachable!(),
         }
     }
 }
@@ -2171,6 +2197,7 @@ impl SerializeInto for Packet {
             &Packet::SEIP(ref p) => p.serialized_len(),
             &Packet::MDC(ref p) => p.serialized_len(),
             &Packet::AED(ref p) => p.serialized_len(),
+            Packet::__Nonexhaustive => unreachable!(),
         })
             + 1 // CTB.
             + BodyLength::Full(self.net_len() as u32).serialized_len()
@@ -2194,7 +2221,8 @@ impl SerializeInto for Packet {
 ///
 /// [`openpgp::Packet`]: ../enum.Packet.html
 /// [`packet::Signature`]: ../packet/enum.Signature.html
-pub enum PacketRef<'a> {
+#[allow(dead_code)]
+enum PacketRef<'a> {
     /// Unknown packet.
     Unknown(&'a packet::Unknown),
     /// Signature packet.
@@ -2420,7 +2448,7 @@ impl Serialize for PacketPile {
     /// Exports a serialized version of the specified `PacketPile` to `o`.
     fn export(&self, o: &mut dyn std::io::Write) -> Result<()> {
         for p in self.children() {
-            dbg!(dbg!(p).export(o))?;
+            p.export(o)?;
         }
 
         Ok(())
@@ -2465,28 +2493,11 @@ impl SerializeInto for Message {
         self.deref().export_into(buf)
     }
 }
-
-impl Serialize for autocrypt::AutocryptHeader {
-    fn serialize(&self, o: &mut dyn std::io::Write) -> Result<()> {
-        if self.key.is_none() {
-            return Err(Error::InvalidOperation("No key".into()).into());
-        }
-
-        for attr in self.attributes.iter() {
-            write!(o, "{}={}; ", attr.key, attr.value)?;
-        }
-
-        let mut buf = Vec::new();
-        self.key.as_ref().unwrap().serialize(&mut buf)?;
-        write!(o, "keydata={} ", base64::encode(&buf))?;
-        Ok(())
-    }
-}
 
 #[cfg(test)]
 mod test {
     use super::*;
-    use crate::constants::CompressionAlgorithm;
+    use crate::types::CompressionAlgorithm;
     use crate::parse::to_unknown_packet;
     use crate::parse::PacketParserBuilder;
     use crate::parse::Parse;
@@ -2535,16 +2546,8 @@ mod test {
         let expected = to_unknown_packet(expected).unwrap();
         let got = to_unknown_packet(got).unwrap();
 
-        let expected_body = if let Some(ref data) = expected.common.body {
-            &data[..]
-        } else {
-            &b""[..]
-        };
-        let got_body = if let Some(ref data) = got.common.body {
-            &data[..]
-        } else {
-            &b""[..]
-        };
+        let expected_body = expected.body();
+        let got_body = got.body();
 
         let mut fail = false;
         if expected.tag() != got.tag() {
@@ -2691,7 +2694,7 @@ mod test {
             // 1. Read the message into a local buffer.
             let data = crate::tests::message(filename);
 
-            // 2. Do a shallow parse of the messsage.  In other words,
+            // 2. Do a shallow parse of the message.  In other words,
             // never recurse so that the resulting message only
             // contains the top-level packets.  Any containers will
             // have their raw content stored in packet.content.
@@ -2721,14 +2724,14 @@ mod test {
                     eprintln!("Orig:");
                     let p = pile.children().next().unwrap();
                     eprintln!("{:?}", p);
-                    let body = &p.body.as_ref().unwrap()[..];
+                    let body = p.body().unwrap();
                     eprintln!("Body: {}", body.len());
                     eprintln!("{}", binary_pp(body));
 
                     eprintln!("Reparsed:");
                     let p = pile2.children().next().unwrap();
                     eprintln!("{:?}", p);
-                    let body = &p.body.as_ref().unwrap()[..];
+                    let body = p.body().unwrap();
                     eprintln!("Body: {}", body.len());
                     eprintln!("{}", binary_pp(body));
 
@@ -2744,7 +2747,7 @@ mod test {
     // reparse them, and make sure we get the same result.
     #[test]
     fn serialize_test_3() {
-        use crate::constants::DataFormat::Text as T;
+        use crate::types::DataFormat::Text as T;
 
         // serialize_test_1 and serialize_test_2 parse a byte stream.
         // This tests creates the message, and then serializes and
@@ -2934,18 +2937,18 @@ mod test {
 
     #[test]
     fn export_signature() {
-        use crate::tpk::TPKBuilder;
+        use crate::cert::CertBuilder;
 
-        let (tpk, _) = TPKBuilder::new().generate().unwrap();
-        let mut keypair = tpk.primary().key().clone().mark_parts_secret()
-            .into_keypair().unwrap();
+        let (cert, _) = CertBuilder::new().generate().unwrap();
+        let mut keypair = cert.primary_key().key().clone().mark_parts_secret()
+            .unwrap().into_keypair().unwrap();
         let uid = UserID::from("foo");
 
         // Make a signature w/o an exportable certification subpacket.
         let sig = uid.bind(
-            &mut keypair, &tpk,
-            signature::Builder::new(SignatureType::GenericCertificate),
-            None, None).unwrap();
+            &mut keypair, &cert,
+            signature::Builder::new(SignatureType::GenericCertification))
+            .unwrap();
 
         // The signature is exportable.  Try to export it in
         // various ways.
@@ -2967,10 +2970,9 @@ mod test {
 
         // Make a signature that is explicitly marked as exportable.
         let sig = uid.bind(
-            &mut keypair, &tpk,
-            signature::Builder::new(SignatureType::GenericCertificate)
-                .set_exportable_certification(true).unwrap(),
-            None, None).unwrap();
+            &mut keypair, &cert,
+            signature::Builder::new(SignatureType::GenericCertification)
+                .set_exportable_certification(true).unwrap()).unwrap();
 
         // The signature is exportable.  Try to export it in
         // various ways.
@@ -2992,10 +2994,9 @@ mod test {
 
         // Make a non-exportable signature.
         let sig = uid.bind(
-            &mut keypair, &tpk,
-            signature::Builder::new(SignatureType::GenericCertificate)
-                .set_exportable_certification(false).unwrap(),
-            None, None).unwrap();
+            &mut keypair, &cert,
+            signature::Builder::new(SignatureType::GenericCertification)
+                .set_exportable_certification(false).unwrap()).unwrap();
 
         // The signature is not exportable.  Try to export it in
         // various ways.
