@@ -53,10 +53,10 @@ impl<'a, C: 'a> Stackable<'a, C> for BZ<'a, C> {
     fn mount(&mut self, _new: BoxStack<'a, C>) {
         unreachable!("Only implemented by Signer")
     }
-    fn inner_mut(&mut self) -> Option<&mut Stackable<'a, C>> {
+    fn inner_mut(&mut self) -> Option<&mut dyn Stackable<'a, C>> {
         Some(self.inner.inner.get_mut())
     }
-    fn inner_ref(&self) -> Option<&Stackable<'a, C>> {
+    fn inner_ref(&self) -> Option<&dyn Stackable<'a, C>> {
         Some(self.inner.inner.get_ref())
     }
     fn cookie_set(&mut self, cookie: C) -> C {
