@@ -17,48 +17,67 @@ mod for_each_artifact {
                 let mut v = Vec::new();
                 p.serialize(&mut v)?;
                 let q = openpgp::Packet::from_bytes(&v)?;
-                assert_eq!(p, &q, "roundtripping {:?} failed", src);
-                let w = p.to_vec().unwrap();
-                assert_eq!(v, w,
-                           "Serialize and SerializeInto disagree on {:?}", p);
+                if p != &q {
+                    return Err(failure::format_err!(
+                        "assertion failed: p == q\np = {:?}\nq = {:?}", p, q));
+                }
+                let w = p.to_vec()?;
+                if v != w {
+                    return Err(failure::format_err!(
+                        "assertion failed: v == w\nv = {:?}\nw = {:?}", v, w));
+                }
                 Ok(())
             })
         }).unwrap();
     }
 
     #[test]
-    fn tpk_roundtrip() {
+    fn cert_roundtrip() {
         for_all_files(&test_data_dir(), |src| {
-            let p = if let Ok(tpk) = openpgp::TPK::from_file(src) {
-                tpk
+            let p = if let Ok(cert) = openpgp::Cert::from_file(src) {
+                cert
             } else {
-                // Ignore non-TPK files.
+                // Ignore non-Cert files.
                 return Ok(());
             };
 
             let mut v = Vec::new();
             p.as_tsk().serialize(&mut v)?;
-            let q = openpgp::TPK::from_bytes(&v)?;
+            let q = openpgp::Cert::from_bytes(&v)?;
             if p != q {
                 eprintln!("roundtripping {:?} failed", src);
 
-                let p : Vec<openpgp::Packet> = p.clone().into_packets().collect();
-                let q : Vec<openpgp::Packet> = q.clone().into_packets().collect();
+                let p_: Vec<_> = p.clone().into_packets().collect();
+                let q_: Vec<_> = q.clone().into_packets().collect();
                 eprintln!("original: {} packets; roundtripped: {} packets",
-                          p.len(), q.len());
-                for (i, (p, q)) in p.iter().zip(q.iter()).enumerate() {
+                          p_.len(), q_.len());
+
+                for (i, (p, q)) in p_.iter().zip(q_.iter()).enumerate() {
                     if p != q {
                         eprintln!("First difference at packet {}:\nOriginal: {:?}\nNew: {:?}",
                                   i, p, q);
                         break;
                     }
                 }
+
+                eprintln!("This is the recovered cert:\n{}",
+                          String::from_utf8_lossy(
+                              &q.armored().to_vec().unwrap()));
             }
             assert_eq!(p, q, "roundtripping {:?} failed", src);
 
             let w = p.as_tsk().to_vec().unwrap();
             assert_eq!(v, w,
                        "Serialize and SerializeInto disagree on {:?}", p);
+
+            // Check that Cert::into_packets() and Cert::to_vec()
+            // agree.
+            let v = p.to_vec()?;
+            let mut buf = Vec::new();
+            for p in p.clone().into_packets() {
+                p.serialize(&mut buf)?;
+            }
+            assert_eq!(buf, v);
             Ok(())
         }).unwrap();
     }
