@@ -5,7 +5,7 @@ use std::cmp;
 use std::fmt;
 use std::io;
 use std::rc::Rc;
-use time::Duration;
+use std::time::Duration;
 
 use capnp::capability::Promise;
 use capnp;
@@ -25,7 +25,7 @@ use tokio_core::reactor::{Handle, Timeout};
 use tokio_core;
 use tokio_io::io::ReadHalf;
 
-use crate::openpgp::{self, TPK, KeyID, Fingerprint};
+use crate::openpgp::{self, Cert, KeyID, Fingerprint};
 use crate::openpgp::parse::Parse;
 use crate::openpgp::serialize::Serialize;
 use sequoia_core as core;
@@ -47,30 +47,30 @@ mod log;
 
 /// Minimum sleep time.
 fn min_sleep_time() -> Duration {
-    Duration::minutes(5)
+    Duration::new(5 * 60, 0)
 }
 
 /// Interval after which all keys should be refreshed once.
 fn refresh_interval() -> Duration {
-    Duration::weeks(1)
+    Duration::new(1 * 7 * 24 * 60 * 60, 0)
 }
 
 /// Returns a value from the uniform distribution over [0, 2*d).
 ///
 /// This function is used to randomize key refresh times.
 fn random_duration(d: Duration) -> Duration {
-    let s = Uniform::from(0..2 * d.num_seconds())
+    let s = Uniform::from(0..2 * d.as_secs())
         .sample(&mut thread_rng());
-    Duration::seconds(s)
+    Duration::new(s, 0)
 }
 
 /* Entry point.  */
 
 /// Makes backends.
 pub fn factory(descriptor: ipc::Descriptor, handle: Handle)
-               -> Result<Box<ipc::Handler>> {
+               -> Result<Box<dyn ipc::Handler>> {
     Backend::new(descriptor, handle)
-        .map(|b| -> Box<ipc::Handler> { Box::new(b) })
+        .map(|b| -> Box<dyn ipc::Handler> { Box::new(b) })
 }
 
 struct Backend {
@@ -149,12 +149,12 @@ impl node::Server for NodeServer {
 
         // XXX maybe check ephemeral and use in-core sqlite db
 
-        let store = sry!(StoreServer::open(self.c.clone(),
+        let mapping = sry!(MappingServer::open(self.c.clone(),
                                            pry!(params.get_realm()),
                                            pry!(params.get_network_policy()).into(),
                                            pry!(params.get_name())));
         pry!(pry!(results.get().get_result()).set_ok(
-            node::store::ToClient::new(store).into_client::<capnp_rpc::Server>()));
+            node::mapping::ToClient::new(mapping).into_client::<capnp_rpc::Server>()));
         Promise::ok(())
     }
 
@@ -164,9 +164,9 @@ impl node::Server for NodeServer {
             -> Promise<(), capnp::Error> {
         bind_results!(results);
         let prefix = pry!(pry!(params.get()).get_realm_prefix());
-        let iter = StoreIterServer::new(self.c.clone(), prefix);
+        let iter = MappingIterServer::new(self.c.clone(), prefix);
         pry!(pry!(results.get().get_result()).set_ok(
-            node::store_iter::ToClient::new(iter).into_client::<capnp_rpc::Server>()));
+            node::mapping_iter::ToClient::new(iter).into_client::<capnp_rpc::Server>()));
         Promise::ok(())
     }
 
@@ -197,7 +197,7 @@ impl node::Server for NodeServer {
               mut results: node::ImportResults)
               -> Promise<(), capnp::Error> {
         bind_results!(results);
-        let new = sry!(TPK::from_bytes(&pry!(pry!(params.get()).get_key())));
+        let new = sry!(Cert::from_bytes(&pry!(pry!(params.get()).get_key())));
         let fp = new.fingerprint();
         let key_id = sry!(KeyServer::lookup_or_create(&self.c, &fp));
         let key = KeyServer::new(self.c.clone(), key_id);
@@ -261,14 +261,14 @@ impl node::Server for NodeServer {
     }
 }
 
-struct StoreServer {
+struct MappingServer {
     c: Rc<Connection>,
     id: ID,
 }
 
-impl Query for StoreServer {
+impl Query for MappingServer {
     fn table_name() -> &'static str {
-        "stores"
+        "mappings"
     }
 
     fn id(&self) -> ID {
@@ -281,7 +281,7 @@ impl Query for StoreServer {
 
     fn slug(&self) -> String {
         self.c.query_row(
-            "SELECT realm, name FROM stores WHERE id = ?1",
+            "SELECT realm, name FROM mappings WHERE id = ?1",
             &[&self.id], |row| -> rusqlite::Result<String> {
                 Ok(format!("{}:{}",
                            row.get::<_, String>(0)?,
@@ -293,9 +293,9 @@ impl Query for StoreServer {
     }
 }
 
-impl StoreServer {
-    fn new(c: Rc<Connection>, id: ID) -> StoreServer {
-        StoreServer{c: c, id: id}
+impl MappingServer {
+    fn new(c: Rc<Connection>, id: ID) -> MappingServer {
+        MappingServer{c: c, id: id}
     }
 
     fn open(c: Rc<Connection>, realm: &str, policy: core::NetworkPolicy, name: &str)
@@ -305,22 +305,22 @@ impl StoreServer {
         let p: u8 = (&policy).into();
 
         c.execute(
-            "INSERT OR IGNORE INTO stores (realm, network_policy, name) VALUES (?1, ?2, ?3)",
-            &[&realm as &ToSql, &p, &name])?;
-        let (id, store_policy): (ID, i64) = c.query_row(
-            "SELECT id, network_policy FROM stores WHERE realm = ?1 AND name = ?2",
+            "INSERT OR IGNORE INTO mappings (realm, network_policy, name) VALUES (?1, ?2, ?3)",
+            &[&realm as &dyn ToSql, &p, &name])?;
+        let (id, mapping_policy): (ID, i64) = c.query_row(
+            "SELECT id, network_policy FROM mappings WHERE realm = ?1 AND name = ?2",
             &[&realm, &name],
             |row| Ok((row.get(0)?, row.get(1)?)))?;
 
         // We cannot implement FromSql and friends for
         // core::NetworkPolicy, hence we need to do it by foot.
-        if store_policy < 0 || store_policy > 3 {
+        if mapping_policy < 0 || mapping_policy > 3 {
             return Err(super::Error::ProtocolError.into());
         }
-        let store_policy = core::NetworkPolicy::from(store_policy as u8);
+        let mapping_policy = core::NetworkPolicy::from(mapping_policy as u8);
 
-        if store_policy != policy {
-            return Err(core::Error::NetworkPolicyViolation(store_policy)
+        if mapping_policy != policy {
+            return Err(core::Error::NetworkPolicyViolation(mapping_policy)
                        .into());
         }
 
@@ -328,10 +328,10 @@ impl StoreServer {
     }
 }
 
-impl node::store::Server for StoreServer {
+impl node::mapping::Server for MappingServer {
     fn add(&mut self,
-           params: node::store::AddParams,
-           mut results: node::store::AddResults)
+           params: node::mapping::AddParams,
+           mut results: node::mapping::AddResults)
            -> Promise<(), capnp::Error> {
         bind_results!(results);
         let params = pry!(params.get());
@@ -346,9 +346,9 @@ impl node::store::Server for StoreServer {
         if created {
             sry!(log::message(
                 &self.c,
-                log::Refers::to().store(self.id).binding(binding_id).key(key_id),
+                log::Refers::to().mapping(self.id).binding(binding_id).key(key_id),
                 &self.slug(),
-                &format!("New binding {} -> {}", label, fp.to_keyid())));
+                &format!("New binding {} -> {}", label, KeyID::from(fp))));
         }
 
 
@@ -360,16 +360,16 @@ impl node::store::Server for StoreServer {
     }
 
     fn lookup(&mut self,
-              params: node::store::LookupParams,
-              mut results: node::store::LookupResults)
+              params: node::mapping::LookupParams,
+              mut results: node::mapping::LookupResults)
               -> Promise<(), capnp::Error> {
         bind_results!(results);
         let label = pry!(pry!(params.get()).get_label());
 
         let binding_id: ID = sry!(
             self.c.query_row(
-                "SELECT id FROM bindings WHERE store = ?1 AND label = ?2",
-                &[&self.id as &ToSql, &label], |row| row.get(0)));
+                "SELECT id FROM bindings WHERE mapping = ?1 AND label = ?2",
+                &[&self.id as &dyn ToSql, &label], |row| row.get(0)));
 
         pry!(pry!(results.get().get_result()).set_ok(
             node::binding::ToClient::new(
@@ -379,8 +379,8 @@ impl node::store::Server for StoreServer {
     }
 
     fn lookup_by_subkeyid(&mut self,
-                          params: node::store::LookupBySubkeyidParams,
-                          mut results: node::store::LookupBySubkeyidResults)
+                          params: node::mapping::LookupBySubkeyidParams,
+                          mut results: node::mapping::LookupBySubkeyidResults)
                           -> Promise<(), capnp::Error> {
         bind_results!(results);
         let keyid = pry!(params.get()).get_keyid();
@@ -400,32 +400,32 @@ impl node::store::Server for StoreServer {
     }
 
     fn delete(&mut self,
-              _: node::store::DeleteParams,
-              mut results: node::store::DeleteResults)
+              _: node::mapping::DeleteParams,
+              mut results: node::mapping::DeleteResults)
               -> Promise<(), capnp::Error> {
         bind_results!(results);
-        sry!(self.c.execute("DELETE FROM stores WHERE id = ?1",
+        sry!(self.c.execute("DELETE FROM mappings WHERE id = ?1",
                                      &[&self.id]));
         Promise::ok(())
     }
 
     fn iter(&mut self,
-            _: node::store::IterParams,
-            mut results: node::store::IterResults)
+            _: node::mapping::IterParams,
+            mut results: node::mapping::IterResults)
             -> Promise<(), capnp::Error> {
         bind_results!(results);
-        let iter = BindingIterServer::new(self.c.clone(), self.id);
+        let iter = BundleIterServer::new(self.c.clone(), self.id);
         pry!(pry!(results.get().get_result()).set_ok(
             node::binding_iter::ToClient::new(iter).into_client::<capnp_rpc::Server>()));
         Promise::ok(())
     }
 
     fn log(&mut self,
-           _: node::store::LogParams,
-           mut results: node::store::LogResults)
+           _: node::mapping::LogParams,
+           mut results: node::mapping::LogResults)
            -> Promise<(), capnp::Error> {
         bind_results!(results);
-        let iter = log::IterServer::new(self.c.clone(), log::Selector::Store(self.id));
+        let iter = log::IterServer::new(self.c.clone(), log::Selector::Mapping(self.id));
         pry!(pry!(results.get().get_result()).set_ok(
             node::log_iter::ToClient::new(iter).into_client::<capnp_rpc::Server>()));
         Promise::ok(())
@@ -454,12 +454,12 @@ impl BindingServer {
     ///
     /// On success, the id of the binding and the key is returned, and
     /// whether or not the entry was just created.
-    fn lookup_or_create(c: &Connection, store: ID, label: &str, fp: &Fingerprint)
+    fn lookup_or_create(c: &Connection, mapping: ID, label: &str, fp: &Fingerprint)
                         -> Result<(ID, ID, bool)> {
         let key_id = KeyServer::lookup_or_create(c, fp)?;
         if let Ok((binding, key)) = c.query_row(
-            "SELECT id, key FROM bindings WHERE store = ?1 AND label = ?2",
-            &[&store as &ToSql, &label],
+            "SELECT id, key FROM bindings WHERE mapping = ?1 AND label = ?2",
+            &[&mapping as &dyn ToSql, &label],
             |row| -> rusqlite::Result<(ID, ID)> {
                 Ok((row.get(0)?, row.get(1)?))
             })
@@ -471,9 +471,9 @@ impl BindingServer {
             }
         } else {
             let r = c.execute(
-                "INSERT INTO bindings (store, label, key, created)
+                "INSERT INTO bindings (mapping, label, key, created)
                  VALUES (?, ?, ?, ?)",
-                &[&store as &ToSql, &label, &key_id, &Timestamp::now()]);
+                &[&mapping as &dyn ToSql, &label, &key_id, &Timestamp::now()]);
 
             // Some other mutator might race us to the insertion.
             match r {
@@ -481,8 +481,8 @@ impl BindingServer {
                     // We lost.  Retry the lookup.
                     rusqlite::ErrorCode::ConstraintViolation => {
                         let (binding, key): (ID, ID) = c.query_row(
-                            "SELECT id, key FROM bindings WHERE store = ?1 AND label = ?2",
-                            &[&store as &ToSql, &label],
+                            "SELECT id, key FROM bindings WHERE mapping = ?1 AND label = ?2",
+                            &[&mapping as &dyn ToSql, &label],
                             |row| Ok((row.get(0)?, row.get(1)?)))?;
                         if key == key_id {
                             Ok((binding, key_id, false))
@@ -556,7 +556,7 @@ impl node::binding::Server for BindingServer {
         let force = pry!(params.get()).get_force();
 
         // This is the key to import.
-        let mut new = sry!(TPK::from_bytes(&pry!(pry!(params.get()).get_key())));
+        let mut new = sry!(Cert::from_bytes(&pry!(pry!(params.get()).get_key())));
 
         // Check in the database for the current key.
         let key_id = sry!(self.key_id());
@@ -566,9 +566,9 @@ impl node::binding::Server for BindingServer {
                 &[&key_id],
                 |row| Ok((row.get(0)?, row.get(1).ok()))));
 
-        // If we found one, convert it to TPK.
+        // If we found one, convert it to Cert.
         let current = if let Some(current) = key {
-            let current = sry!(TPK::from_bytes(&current));
+            let current = sry!(Cert::from_bytes(&current));
             if current.fingerprint().to_hex() != fingerprint {
                 // Inconsistent database.
                 fail!(node::Error::SystemError);
@@ -602,7 +602,7 @@ impl node::binding::Server for BindingServer {
         sry!(new.serialize(&mut blob));
 
         sry!(self.c.execute("UPDATE keys SET key = ?1 WHERE id = ?2",
-                            &[&blob as &ToSql, &key_id]));
+                            &[&blob as &dyn ToSql, &key_id]));
         sry!(KeyServer::reindex_subkeys(&self.c, key_id, &new));
 
         pry!(pry!(results.get().get_result()).set_ok(&blob[..]));
@@ -633,14 +633,14 @@ impl node::binding::Server for BindingServer {
                            encryption_first = coalesce(encryption_first, ?2),
                            encryption_last = ?2
                        WHERE id = ?1",
-                      &[&self.id as &ToSql, &now]));
+                      &[&self.id as &dyn ToSql, &now]));
         sry!(self.c
              .execute("UPDATE keys
                        SET encryption_count = encryption_count + 1,
                            encryption_first = coalesce(encryption_first, ?2),
                            encryption_last = ?2
                        WHERE id = ?1",
-                      &[&key as &ToSql, &now]));
+                      &[&key as &dyn ToSql, &now]));
 
         sry!(self.query_stats( pry!(results.get().get_result()).init_ok()));
         Promise::ok(())
@@ -660,14 +660,14 @@ impl node::binding::Server for BindingServer {
                            verification_first = coalesce(verification_first, ?2),
                            verification_last = ?2
                        WHERE id = ?1",
-                      &[&self.id as &ToSql, &now]));
+                      &[&self.id as &dyn ToSql, &now]));
         sry!(self.c
              .execute("UPDATE keys
                        SET verification_count = verification_count + 1,
                            verification_first = coalesce(verification_first, ?2),
                            verification_last = ?2
                        WHERE id = ?1",
-                      &[&key as &ToSql, &now]));
+                      &[&key as &dyn ToSql, &now]));
 
         sry!(self.query_stats( pry!(results.get().get_result()).init_ok()));
         Promise::ok(())
@@ -745,7 +745,7 @@ impl KeyServer {
         } else {
             let r = c.execute(
                 "INSERT INTO keys (fingerprint, created, update_at) VALUES (?1, ?2, ?2)",
-                &[&fp as &ToSql, &Timestamp::now()]);
+                &[&fp as &dyn ToSql, &Timestamp::now()]);
 
             // Some other mutator might race us to the insertion.
             match r {
@@ -766,8 +766,8 @@ impl KeyServer {
 
     /// Merges other into this key updating the database.
     ///
-    /// Returnes the merged key as blob.
-    fn merge(&self, other: TPK) -> Result<Vec<u8>> {
+    /// Returns the merged key as blob.
+    fn merge(&self, other: Cert) -> Result<Vec<u8>> {
         let mut new = other;
 
         // Get the current key from the database.
@@ -779,7 +779,7 @@ impl KeyServer {
 
         // If there was a key stored there, merge it.
         if let Some(current) = key {
-            let current = TPK::from_bytes(&current)?;
+            let current = Cert::from_bytes(&current)?;
 
             if current.fingerprint().to_hex() != fingerprint {
                 // Inconsistent database.
@@ -798,21 +798,21 @@ impl KeyServer {
         new.serialize(&mut blob)?;
 
         self.c.execute("UPDATE keys SET key = ?1 WHERE id = ?2",
-                       &[&blob as &ToSql, &self.id])?;
+                       &[&blob as &dyn ToSql, &self.id])?;
         KeyServer::reindex_subkeys(&self.c, self.id, &new)?;
 
         Ok(blob)
     }
 
     /// Keeps the mapping of (sub)KeyIDs to keys up-to-date.
-    fn reindex_subkeys(c: &Connection, key_id: ID, tpk: &TPK) -> Result<()> {
-        for (_, _, key) in tpk.keys_all() {
-            let keyid = key.keyid().as_u64()
+    fn reindex_subkeys(c: &Connection, key_id: ID, cert: &Cert) -> Result<()> {
+        for ka in cert.keys() {
+            let keyid = ka.key().keyid().as_u64()
                 .expect("computed keyid is valid");
 
             let r = c.execute(
                 "INSERT INTO key_by_keyid (keyid, key) VALUES (?1, ?2)",
-                &[&(keyid as i64) as &ToSql, &key_id]);
+                &[&(keyid as i64) as &dyn ToSql, &key_id]);
 
             // The mapping might already be present.  This is not an error.
             match r {
@@ -837,7 +837,7 @@ impl KeyServer {
         self.c.execute("UPDATE keys
                         SET updated = ?2, update_at = ?3
                         WHERE id = ?1",
-                       &[&self.id as &ToSql, &Timestamp::now(),
+                       &[&self.id as &dyn ToSql, &Timestamp::now(),
                          &(Timestamp::now() + next)])?;
         Ok(())
     }
@@ -849,7 +849,7 @@ impl KeyServer {
         self.c.execute("UPDATE keys
                         SET update_at = ?2
                         WHERE id = ?1",
-                       &[&self.id as &ToSql,
+                       &[&self.id as &dyn ToSql,
                          &(Timestamp::now() + next)])?;
         Ok(())
     }
@@ -863,8 +863,8 @@ impl KeyServer {
         c.query_row(
             "SELECT keys.update_at FROM keys
                  JOIN bindings on keys.id = bindings.key
-                 JOIN stores on stores.id = bindings.store
-                 WHERE stores.network_policy = ?1
+                 JOIN mappings on mappings.id = bindings.mapping
+                 WHERE mappings.network_policy = ?1
                  ORDER BY keys.update_at LIMIT 1",
             &[&network_policy_u8], |row| -> rusqlite::Result<Timestamp> {
                 row.get(0)
@@ -873,17 +873,17 @@ impl KeyServer {
 
     /// Returns the number of keys using the given policy.
     fn need_update(c: &Rc<Connection>, network_policy: core::NetworkPolicy)
-                   -> Result<i32> {
+                   -> Result<u32> {
         let network_policy_u8 = u8::from(&network_policy);
 
         let count: i64 = c.query_row(
             "SELECT COUNT(*) FROM keys
                  JOIN bindings on keys.id = bindings.key
-                 JOIN stores on stores.id = bindings.store
-                 WHERE stores.network_policy >= ?1",
+                 JOIN mappings on mappings.id = bindings.mapping
+                 WHERE mappings.network_policy >= ?1",
             &[&network_policy_u8], |row| row.get(0))?;
         assert!(count >= 0);
-        Ok(count as i32)
+        Ok(count as u32)
     }
 
     /// Helper for `update`.
@@ -899,11 +899,11 @@ impl KeyServer {
         let (id, fingerprint): (ID, String) = c.query_row(
             "SELECT keys.id, keys.fingerprint FROM keys
                  JOIN bindings on keys.id = bindings.key
-                 JOIN stores on stores.id = bindings.store
-                 WHERE stores.network_policy >= ?1
+                 JOIN mappings on mappings.id = bindings.mapping
+                 WHERE mappings.network_policy >= ?1
                    AND keys.update_at < ?2
                  ORDER BY keys.update_at LIMIT 1",
-            &[&network_policy_u8 as &ToSql, &Timestamp::now()],
+            &[&network_policy_u8 as &dyn ToSql, &Timestamp::now()],
             |row| Ok((row.get(0)?, row.get(1)?)))?;
         let fingerprint = openpgp::Fingerprint::from_hex(&fingerprint)
             .map_err(|_| node::Error::SystemError)?;
@@ -913,14 +913,14 @@ impl KeyServer {
         let keyserver = net::KeyServer::keys_openpgp_org(&ctx)?;
 
         Ok((KeyServer::new(c.clone(), id),
-            fingerprint.to_keyid(),
+            fingerprint.into(),
             keyserver))
     }
 
     /// Updates the key that was least recently updated.
     fn update(c: &Rc<Connection>,
               network_policy: core::NetworkPolicy)
-              -> Box<Future<Item=Duration, Error=failure::Error> + 'static> {
+              -> Box<dyn Future<Item=Duration, Error=failure::Error> + 'static> {
         let (key, id, mut keyserver)
             = match Self::update_helper(c, network_policy) {
             Ok((key, id, keyserver)) => (key, id, keyserver),
@@ -935,12 +935,12 @@ impl KeyServer {
         if at <= now {
             Box::new(
                 keyserver.get(&id)
-                    .then(move |tpk| {
+                    .then(move |cert| {
                         let next = Self::need_update(&c, network_policy)
                             .map(|c| refresh_interval() / c)
                             .unwrap_or(min_sleep_time());
 
-                        if let Err(e) = tpk.map(|t| key.merge(t)) {
+                        if let Err(e) = cert.map(|t| key.merge(t)) {
                             key.error("Update unsuccessful",
                                       &format!("{:?}", e), next / 2)
                                 .unwrap_or(());
@@ -953,7 +953,8 @@ impl KeyServer {
                     }))
         } else {
             assert!(at > now);
-            Box::new(future::ok(cmp::max(min_sleep_time(), at - now)))
+            Box::new(future::ok(cmp::max(min_sleep_time(),
+                                         now.duration_since(at).unwrap())))
         }
     }
 
@@ -969,10 +970,7 @@ impl KeyServer {
             Self::update(&c, network_policy)
                 .then(move |d| {
                     let d = d.unwrap_or(min_sleep_time());
-                     Timeout::new(
-                         ::std::time::Duration::new(random_duration(d)
-                                                    .num_seconds() as u64, 0),
-                         &h1)
+                     Timeout::new(random_duration(d), &h1)
                      .unwrap() // XXX: May fail if the eventloop expired.
                      .then(move |timeout| {
                          if timeout.is_ok() {
@@ -1007,7 +1005,7 @@ impl Query for KeyServer {
             &[&self.id], |row| -> rusqlite::Result<String> { row.get(0) })
             .ok()
             .and_then(|fp| Fingerprint::from_hex(&fp).ok())
-            .map(|fp| fp.to_keyid().to_string())
+            .map(|fp| KeyID::from(fp).to_string())
             .unwrap_or(
                 format!("{}::{}", Self::table_name(), self.id())
             )
@@ -1024,9 +1022,9 @@ impl node::key::Server for KeyServer {
         Promise::ok(())
     }
 
-    fn tpk(&mut self,
-           _: node::key::TpkParams,
-           mut results: node::key::TpkResults)
+    fn cert(&mut self,
+           _: node::key::CertParams,
+           mut results: node::key::CertResults)
            -> Promise<(), capnp::Error> {
         bind_results!(results);
         let key: Vec<u8> = sry!(
@@ -1043,7 +1041,7 @@ impl node::key::Server for KeyServer {
               mut results: node::key::ImportResults)
               -> Promise<(), capnp::Error> {
         bind_results!(results);
-        let new = sry!(TPK::from_bytes(&pry!(pry!(params.get()).get_key())));
+        let new = sry!(Cert::from_bytes(&pry!(pry!(params.get()).get_key())));
         let blob = sry!(self.merge(new));
         pry!(pry!(results.get().get_result()).set_ok(&blob[..]));
         Promise::ok(())
@@ -1125,30 +1123,30 @@ trait Query {
 
 /* Iterators.  */
 
-struct StoreIterServer {
+struct MappingIterServer {
     c: Rc<Connection>,
     prefix: String,
     n: ID,
 }
 
-impl StoreIterServer {
+impl MappingIterServer {
     fn new(c: Rc<Connection>, prefix: &str) -> Self {
-        StoreIterServer{c: c, prefix: String::from(prefix) + "%", n: ID::null()}
+        MappingIterServer{c: c, prefix: String::from(prefix) + "%", n: ID::null()}
     }
 }
 
-impl node::store_iter::Server for StoreIterServer {
+impl node::mapping_iter::Server for MappingIterServer {
     fn next(&mut self,
-            _: node::store_iter::NextParams,
-            mut results: node::store_iter::NextResults)
+            _: node::mapping_iter::NextParams,
+            mut results: node::mapping_iter::NextResults)
             -> Promise<(), capnp::Error> {
         bind_results!(results);
         let (id, realm, name, network_policy): (ID, String, String, i64) =
             sry!(self.c.query_row(
-                 "SELECT id, realm, name, network_policy FROM stores
+                 "SELECT id, realm, name, network_policy FROM mappings
                       WHERE id > ?1 AND realm like ?2
                       ORDER BY id LIMIT 1",
-                &[&self.n as &ToSql, &self.prefix],
+                &[&self.n as &dyn ToSql, &self.prefix],
                 |row| Ok((row.get(0)?, row.get(1)?, row.get(2)?, row.get(3)?))));
 
         // We cannot implement FromSql and friends for
@@ -1162,26 +1160,26 @@ impl node::store_iter::Server for StoreIterServer {
         entry.set_realm(&realm);
         entry.set_name(&name);
         entry.set_network_policy(network_policy.into());
-        entry.set_store(node::store::ToClient::new(
-            StoreServer::new(self.c.clone(), id)).into_client::<capnp_rpc::Server>());
+        entry.set_mapping(node::mapping::ToClient::new(
+            MappingServer::new(self.c.clone(), id)).into_client::<capnp_rpc::Server>());
         self.n = id;
         Promise::ok(())
     }
 }
 
-struct BindingIterServer {
+struct BundleIterServer {
     c: Rc<Connection>,
-    store_id: ID,
+    mapping_id: ID,
     n: ID,
 }
 
-impl BindingIterServer {
-    fn new(c: Rc<Connection>, store_id: ID) -> Self {
-        BindingIterServer{c: c, store_id: store_id, n: ID::null()}
+impl BundleIterServer {
+    fn new(c: Rc<Connection>, mapping_id: ID) -> Self {
+        BundleIterServer{c: c, mapping_id: mapping_id, n: ID::null()}
     }
 }
 
-impl node::binding_iter::Server for BindingIterServer {
+impl node::binding_iter::Server for BundleIterServer {
     fn next(&mut self,
             _: node::binding_iter::NextParams,
             mut results: node::binding_iter::NextResults)
@@ -1191,9 +1189,9 @@ impl node::binding_iter::Server for BindingIterServer {
             sry!(self.c.query_row(
                  "SELECT bindings.id, bindings.label, keys.fingerprint FROM bindings
                       JOIN keys ON bindings.key = keys.id
-                      WHERE bindings.id > ?1 AND bindings.store = ?2
+                      WHERE bindings.id > ?1 AND bindings.mapping = ?2
                       ORDER BY bindings.id LIMIT 1",
-                &[&self.n, &self.store_id],
+                &[&self.n, &self.mapping_id],
                 |row| Ok((row.get(0)?, row.get(1)?, row.get(2)?))));
 
         let mut entry = pry!(results.get().get_result()).init_ok();
@@ -1250,7 +1248,7 @@ impl fmt::Debug for node::Error {
                    &node::Error::NotFound => "NotFound",
                    &node::Error::Conflict => "Conflict",
                    &node::Error::SystemError => "SystemError",
-                   &node::Error::MalformedTPK => "MalformedTPK",
+                   &node::Error::MalformedCert => "MalformedCert",
                    &node::Error::MalformedFingerprint => "MalformedFingerprint",
                    &node::Error::NetworkPolicyViolationOffline =>
                        "NetworkPolicyViolation(Offline)",
@@ -1283,8 +1281,8 @@ impl From<failure::Error> for node::Error {
     fn from(e: failure::Error) -> Self {
         if let Some(e) = e.downcast_ref::<openpgp::Error>() {
             return match e {
-                &openpgp::Error::MalformedTPK(_) =>
-                    node::Error::MalformedTPK,
+                &openpgp::Error::MalformedCert(_) =>
+                    node::Error::MalformedCert,
                 _ => node::Error::SystemError,
             }
         }
@@ -1335,8 +1333,8 @@ impl From<failure::Error> for node::Error {
 impl From<openpgp::Error> for node::Error {
     fn from(e: openpgp::Error) -> Self {
         match e {
-            openpgp::Error::MalformedTPK(_) =>
-                node::Error::MalformedTPK,
+            openpgp::Error::MalformedCert(_) =>
+                node::Error::MalformedCert,
             _ => node::Error::SystemError,
         }
     }
@@ -1371,7 +1369,7 @@ CREATE TABLE version (
 
 INSERT INTO version (id, version) VALUES (1, 1);
 
-CREATE TABLE stores (
+CREATE TABLE mappings (
     id INTEGER PRIMARY KEY,
     realm TEXT NOT NULL,
     network_policy INTEGER NOT NULL,
@@ -1380,7 +1378,7 @@ CREATE TABLE stores (
 
 CREATE TABLE bindings (
     id INTEGER PRIMARY KEY,
-    store INTEGER NOT NULL,
+    mapping INTEGER NOT NULL,
     label TEXT NOT NULL,
     key INTEGER NOT NULL,
 
@@ -1394,8 +1392,8 @@ CREATE TABLE bindings (
     verification_first INTEGER NULL,
     verification_last INTEGER NULL,
 
-    UNIQUE(store, label),
-    FOREIGN KEY (store) REFERENCES stores(id) ON DELETE CASCADE,
+    UNIQUE(mapping, label),
+    FOREIGN KEY (mapping) REFERENCES mappings(id) ON DELETE CASCADE,
     FOREIGN KEY (key) REFERENCES keys(id) ON DELETE CASCADE);
 
 CREATE TABLE keys (
@@ -1428,13 +1426,13 @@ CREATE TABLE log (
     id INTEGER PRIMARY KEY,
     timestamp INTEGER NOT NULL,
     level INTEGER NOT NULL,
-    store INTEGER NULL,
+    mapping INTEGER NULL,
     binding INTEGER NULL,
     key INTEGER NULL,
     slug TEXT NOT NULL,
     message TEXT NOT NULL,
     error TEXT NULL,
-    FOREIGN KEY (store) REFERENCES stores(id) ON DELETE CASCADE,
+    FOREIGN KEY (mapping) REFERENCES mappings(id) ON DELETE CASCADE,
     FOREIGN KEY (binding) REFERENCES bindings(id) ON DELETE CASCADE,
     FOREIGN KEY (key) REFERENCES keys(id) ON DELETE CASCADE);
 ";
