@@ -1,30 +1,57 @@
 use std::io::{self, Read};
-use time;
 
 extern crate sequoia_openpgp as openpgp;
-use self::openpgp::constants::SymmetricAlgorithm;
-use self::openpgp::conversions::hex;
+use self::openpgp::types::{Duration, Timestamp, SymmetricAlgorithm};
+use self::openpgp::fmt::hex;
 use self::openpgp::crypto::mpis;
 use self::openpgp::{Packet, Result};
 use self::openpgp::packet::prelude::*;
-use self::openpgp::packet::ctb::CTB;
-use self::openpgp::packet::{Header, BodyLength, Signature};
+use self::openpgp::packet::header::CTB;
+use self::openpgp::packet::{Header, header::BodyLength, Signature};
 use self::openpgp::packet::signature::subpacket::{Subpacket, SubpacketValue};
-use self::openpgp::crypto::{SessionKey, s2k::S2K};
+use self::openpgp::crypto::{SessionKey, S2K};
 use self::openpgp::parse::{map::Map, Parse, PacketParserResult};
 
-use super::TIMEFMT;
-
 #[derive(Debug)]
 pub enum Kind {
     Message {
         encrypted: bool,
     },
     Keyring,
-    TPK,
+    Cert,
     Unknown,
 }
 
+/// Converts sequoia_openpgp types for rendering.
+pub trait Convert<T> {
+    /// Performs the conversion.
+    fn convert(self) -> T;
+}
+
+impl Convert<chrono::Duration> for std::time::Duration {
+    fn convert(self) -> chrono::Duration {
+        chrono::Duration::seconds(self.as_secs() as i64)
+    }
+}
+
+impl Convert<chrono::Duration> for Duration {
+    fn convert(self) -> chrono::Duration {
+        chrono::Duration::seconds(self.as_secs() as i64)
+    }
+}
+
+impl Convert<chrono::DateTime<chrono::offset::Utc>> for std::time::SystemTime {
+    fn convert(self) -> chrono::DateTime<chrono::offset::Utc> {
+        chrono::DateTime::<chrono::offset::Utc>::from(self)
+    }
+}
+
+impl Convert<chrono::DateTime<chrono::offset::Utc>> for Timestamp {
+    fn convert(self) -> chrono::DateTime<chrono::offset::Utc> {
+        std::time::SystemTime::from(self).convert()
+    }
+}
+
 pub fn dump<W>(input: &mut dyn io::Read, output: &mut dyn io::Write,
                mpis: bool, hex: bool, sk: Option<&SessionKey>,
                width: W)
@@ -125,8 +152,8 @@ pub fn dump<W>(input: &mut dyn io::Read, output: &mut dyn io::Write,
             Ok(Kind::Message {
                 encrypted: message_encrypted,
             })
-        } else if eof.is_tpk().is_ok() {
-            Ok(Kind::TPK)
+        } else if eof.is_cert().is_ok() {
+            Ok(Kind::Cert)
         } else if eof.is_keyring().is_ok() {
             Ok(Kind::Keyring)
         } else {
@@ -238,10 +265,16 @@ impl PacketDumper {
                   -> Result<()> {
         use self::openpgp::Packet::*;
 
+        if let Some(tag) = p.kind() {
+            write!(output, "{}", tag)?;
+        } else {
+            write!(output, "Unknown Packet")?;
+        }
+
         if let Some(h) = header {
-            write!(output, "{} CTB, {}: ",
-                   if let CTB::Old(_) = h.ctb { "Old" } else { "New" },
-                   match h.length {
+            write!(output, ", {} CTB, {}",
+                   if let CTB::Old(_) = h.ctb() { "old" } else { "new" },
+                   match h.length() {
                        BodyLength::Full(n) =>
                            format!("{} bytes", n),
                        BodyLength::Partial(n) =>
@@ -250,22 +283,24 @@ impl PacketDumper {
                            "indeterminate length".into(),
                    })?;
         }
+        writeln!(output)?;
 
         fn dump_key<P, R>(pd: &PacketDumper,
-                          output: &mut dyn io::Write, i: &str, p: &Packet,
+                          output: &mut dyn io::Write, i: &str,
                           k: &Key<P, R>)
             -> Result<()>
             where P: key::KeyParts,
                   R: key::KeyRole,
         {
-            writeln!(output, "{}", p.tag())?;
             writeln!(output, "{}  Version: {}", i, k.version())?;
             writeln!(output, "{}  Creation time: {}", i,
-                     time::strftime(TIMEFMT, k.creation_time()).unwrap())?;
+                     k.creation_time().convert())?;
             writeln!(output, "{}  Pk algo: {}", i, k.pk_algo())?;
             if let Some(bits) = k.mpis().bits() {
                 writeln!(output, "{}  Pk size: {} bits", i, bits)?;
             }
+            writeln!(output, "{}  Fingerprint: {}", i, k.fingerprint())?;
+            writeln!(output, "{}  KeyID: {}", i, k.keyid())?;
             if pd.mpis {
                 writeln!(output, "{}", i)?;
                 writeln!(output, "{}  Public Key:", i)?;
@@ -386,36 +421,34 @@ impl PacketDumper {
 
         match p {
             Unknown(ref u) => {
-                writeln!(output, "Unknown Packet")?;
                 writeln!(output, "{}  Tag: {}", i, u.tag())?;
                 writeln!(output, "{}  Error: {}", i, u.error())?;
             },
 
-            PublicKey(ref k) => dump_key(self, output, i, p, k)?,
-            PublicSubkey(ref k) => dump_key(self, output, i, p, k)?,
-            SecretKey(ref k) => dump_key(self, output, i, p, k)?,
-            SecretSubkey(ref k) => dump_key(self, output, i, p, k)?,
+            PublicKey(ref k) => dump_key(self, output, i, k)?,
+            PublicSubkey(ref k) => dump_key(self, output, i, k)?,
+            SecretKey(ref k) => dump_key(self, output, i, k)?,
+            SecretSubkey(ref k) => dump_key(self, output, i, k)?,
 
             Signature(ref s) => {
-                writeln!(output, "Signature Packet")?;
                 writeln!(output, "{}  Version: {}", i, s.version())?;
                 writeln!(output, "{}  Type: {}", i, s.typ())?;
                 writeln!(output, "{}  Pk algo: {}", i, s.pk_algo())?;
                 writeln!(output, "{}  Hash algo: {}", i, s.hash_algo())?;
                 if s.hashed_area().iter().count() > 0 {
                     writeln!(output, "{}  Hashed area:", i)?;
-                    for (_, _, pkt) in s.hashed_area().iter() {
+                    for pkt in s.hashed_area().iter() {
                         self.dump_subpacket(output, i, pkt, s)?;
                     }
                 }
                 if s.unhashed_area().iter().count() > 0 {
                     writeln!(output, "{}  Unhashed area:", i)?;
-                    for (_, _, pkt) in s.unhashed_area().iter() {
+                    for pkt in s.unhashed_area().iter() {
                         self.dump_subpacket(output, i, pkt, s)?;
                     }
                 }
-                writeln!(output, "{}  Hash prefix: {}", i,
-                         hex::encode(s.hash_prefix()))?;
+                writeln!(output, "{}  Digest prefix: {}", i,
+                         hex::encode(s.digest_prefix()))?;
                 write!(output, "{}  Level: {} ", i, s.level())?;
                 match s.level() {
                     0 => writeln!(output, "(signature over data)")?,
@@ -470,7 +503,6 @@ impl PacketDumper {
             },
 
             OnePassSig(ref o) => {
-                writeln!(output, "One-Pass Signature Packet")?;
                 writeln!(output, "{}  Version: {}", i, o.version())?;
                 writeln!(output, "{}  Type: {}", i, o.typ())?;
                 writeln!(output, "{}  Pk algo: {}", i, o.pk_algo())?;
@@ -480,19 +512,16 @@ impl PacketDumper {
             },
 
             Trust(ref p) => {
-                writeln!(output, "Trust Packet")?;
                 writeln!(output, "{}  Value: {}", i, hex::encode(p.value()))?;
             },
 
             UserID(ref u) => {
-                writeln!(output, "User ID Packet")?;
                 writeln!(output, "{}  Value: {}", i,
                          String::from_utf8_lossy(u.value()))?;
             },
 
             UserAttribute(ref u) => {
                 use self::openpgp::packet::user_attribute::{Subpacket, Image};
-                writeln!(output, "User Attribute Packet")?;
 
                 for subpacket in u.subpackets() {
                     match subpacket {
@@ -522,11 +551,9 @@ impl PacketDumper {
             },
 
             Marker(_) => {
-                writeln!(output, "Marker Packet")?;
             },
 
             Literal(ref l) => {
-                writeln!(output, "Literal Data Packet")?;
                 writeln!(output, "{}  Format: {}", i, l.format())?;
                 if let Some(filename) = l.filename() {
                     writeln!(output, "{}  Filename: {}", i,
@@ -534,17 +561,15 @@ impl PacketDumper {
                 }
                 if let Some(timestamp) = l.date() {
                     writeln!(output, "{}  Timestamp: {}", i,
-                             time::strftime(TIMEFMT, timestamp).unwrap())?;
+                             timestamp.convert())?;
                 }
             },
 
             CompressedData(ref c) => {
-                writeln!(output, "Compressed Data Packet")?;
-                writeln!(output, "{}  Algorithm: {}", i, c.algorithm())?;
+                writeln!(output, "{}  Algorithm: {}", i, c.algo())?;
             },
 
             PKESK(ref p) => {
-                writeln!(output, "Public-key Encrypted Session Key Packet")?;
                 writeln!(output, "{}  Version: {}", i, p.version())?;
                 writeln!(output, "{}  Recipient: {}", i, p.recipient())?;
                 writeln!(output, "{}  Pk algo: {}", i, p.pk_algo())?;
@@ -586,7 +611,6 @@ impl PacketDumper {
             },
 
             SKESK(ref s) => {
-                writeln!(output, "Symmetric-key Encrypted Session Key Packet")?;
                 writeln!(output, "{}  Version: {}", i, s.version())?;
                 match s {
                     self::openpgp::packet::SKESK::V4(ref s) => {
@@ -616,30 +640,32 @@ impl PacketDumper {
                         writeln!(output, "{}  Digest: {}", i,
                                  hex::encode(s.aead_digest()))?;
                     },
+
+                    self::openpgp::packet::SKESK::__Nonexhaustive =>
+                        unreachable!(),
                 }
             },
 
             SEIP(ref s) => {
-                writeln!(output, "Encrypted and Integrity Protected Data Packet")?;
                 writeln!(output, "{}  Version: {}", i, s.version())?;
             },
 
             MDC(ref m) => {
-                writeln!(output, "Modification Detection Code Packet")?;
-                writeln!(output, "{}  Hash: {}",
-                         i, hex::encode(m.hash()))?;
-                writeln!(output, "{}  Computed hash: {}",
-                         i, hex::encode(m.computed_hash()))?;
+                writeln!(output, "{}  Digest: {}",
+                         i, hex::encode(m.digest()))?;
+                writeln!(output, "{}  Computed digest: {}",
+                         i, hex::encode(m.computed_digest()))?;
             },
 
             AED(ref a) => {
-                writeln!(output, "AEAD Encrypted Data Packet")?;
                 writeln!(output, "{}  Version: {}", i, a.version())?;
                 writeln!(output, "{}  Symmetric algo: {}", i, a.symmetric_algo())?;
                 writeln!(output, "{}  AEAD: {}", i, a.aead())?;
                 writeln!(output, "{}  Chunk size: {}", i, a.chunk_size())?;
                 writeln!(output, "{}  IV: {}", i, hex::encode(a.iv()))?;
             },
+
+            __Nonexhaustive => unreachable!(),
         }
 
         if let Some(fields) = additional_fields {
@@ -652,13 +678,13 @@ impl PacketDumper {
             writeln!(output, "{}", i)?;
             let mut hd = hex::Dumper::new(output, self.indentation_for_hexdump(
                 i, map.iter()
-                    .map(|f| if f.name == "body" { 16 } else { f.name.len() })
+                    .map(|f| if f.name() == "body" { 16 } else { f.name().len() })
                     .max()
                     .expect("we always have one entry")));
 
             for field in map.iter() {
-                if field.name == "body" {
-                    hd.write_labeled(field.data, |offset, data| {
+                if field.name() == "body" {
+                    hd.write_labeled(field.data(), |offset, data| {
                         let mut l = String::new();
                         for _ in 0..offset {
                             l.push(' ');
@@ -675,7 +701,7 @@ impl PacketDumper {
                         Some(l)
                     })?;
                 } else {
-                    hd.write(field.data, field.name)?;
+                    hd.write(field.data(), field.name())?;
                 }
             }
 
@@ -689,7 +715,7 @@ impl PacketDumper {
     }
 
     fn dump_subpacket(&self, output: &mut dyn io::Write, i: &str,
-                      s: Subpacket, sig: &Signature)
+                      s: &Subpacket, sig: &Signature)
                       -> Result<()> {
         use self::SubpacketValue::*;
 
@@ -700,26 +726,20 @@ impl PacketDumper {
             Ok(())
         };
 
-        match s.value {
-            Unknown(ref b) => {
-                writeln!(output, "{}    {:?}{}:", i, s.tag,
-                         if s.critical { " (critical)" } else { "" })?;
-                hexdump_unknown(output, b)?;
+        match s.value() {
+            Unknown { body, .. } => {
+                writeln!(output, "{}    {:?}{}:", i, s.tag(),
+                         if s.critical() { " (critical)" } else { "" })?;
+                hexdump_unknown(output, body)?;
             },
-            Invalid(ref b) => {
-                writeln!(output, "{}    {:?}{}:", i, s.tag,
-                         if s.critical { " (critical)" } else { "" })?;
-                hexdump_unknown(output, b)?;
-            },
-            SignatureCreationTime(ref t) =>
+            SignatureCreationTime(t) =>
                 write!(output, "{}    Signature creation time: {}", i,
-                       time::strftime(TIMEFMT, t).unwrap())?,
-            SignatureExpirationTime(ref t) =>
+                       (*t).convert())?,
+            SignatureExpirationTime(t) =>
                 write!(output, "{}    Signature expiration time: {} ({})",
-                       i, t,
+                       i, t.convert(),
                        if let Some(creation) = sig.signature_creation_time() {
-                           time::strftime(TIMEFMT, &(creation + *t))
-                               .unwrap()
+                           (creation + t.clone().into()).convert().to_string()
                        } else {
                            " (no Signature Creation Time subpacket)".into()
                        })?,
@@ -733,8 +753,9 @@ impl PacketDumper {
                        String::from_utf8_lossy(r))?,
             Revocable(r) =>
                 write!(output, "{}    Revocable: {}", i, r)?,
-            KeyExpirationTime(ref t) =>
-                write!(output, "{}    Key expiration time: {}", i, t)?,
+            KeyExpirationTime(t) =>
+                write!(output, "{}    Key expiration time: {}", i,
+                       t.convert())?,
             PreferredSymmetricAlgorithms(ref c) =>
                 write!(output, "{}    Symmetric algo preferences: {}", i,
                        c.iter().map(|c| format!("{:?}", c))
@@ -791,20 +812,23 @@ impl PacketDumper {
                        .collect::<Vec<String>>().join(", "))?,
             IntendedRecipient(ref fp) =>
                 write!(output, "{}    Intended Recipient: {}", i, fp)?,
+            __Nonexhaustive => unreachable!(),
         }
 
-        match s.value {
-            Unknown(_) | Invalid(_) => (),
+        match s.value() {
+            Unknown { .. } => (),
             EmbeddedSignature(ref sig) => {
-                if s.critical {
+                if s.critical() {
                     write!(output, " (critical)")?;
                 }
                 writeln!(output)?;
                 let indent = format!("{}      ", i);
-                self.dump_packet(output, &indent, None, sig, None, None)?;
+                write!(output, "{}", indent)?;
+                self.dump_packet(output, &indent, None, &sig.clone().into(),
+                                 None, None)?;
             },
             _ => {
-                if s.critical {
+                if s.critical() {
                     write!(output, " (critical)")?;
                 }
                 writeln!(output)?;
