@@ -3,17 +3,16 @@ use std::cmp::Ordering;
 use std::collections::{HashMap, HashSet};
 use std::fs::File;
 use std::io::{self, Write};
-use time;
+use std::time::SystemTime;
 use rpassword;
 
 extern crate sequoia_openpgp as openpgp;
 use sequoia_core::Context;
-use crate::openpgp::constants::{
+use crate::openpgp::types::{
     CompressionAlgorithm,
-    DataFormat,
 };
 use crate::openpgp::crypto;
-use crate::openpgp::{TPK, KeyID, Result};
+use crate::openpgp::{Cert, KeyID, Result};
 use crate::openpgp::packet::prelude::*;
 use crate::openpgp::parse::{
     Parse,
@@ -31,34 +30,30 @@ use crate::openpgp::serialize::padding::{
     Padder,
     padme,
 };
+use crate::openpgp::policy::Policy;
 extern crate sequoia_store as store;
 
-mod decrypt;
+pub mod decrypt;
 pub use self::decrypt::decrypt;
 mod sign;
 pub use self::sign::sign;
-mod dump;
+pub mod dump;
+use dump::Convert;
 pub use self::dump::dump;
 mod inspect;
 pub use self::inspect::inspect;
 pub mod key;
 
-const TIMEFMT: &'static str = "%Y-%m-%dT%H:%M";
-
-fn tm2str(t: &time::Tm) -> String {
-    time::strftime(TIMEFMT, t).expect("TIMEFMT is correct")
-}
-
-/// Returns suitable signing keys from a given list of TPKs.
-fn get_signing_keys(tpks: &[openpgp::TPK])
-    -> Result<Vec<crypto::KeyPair<
-           openpgp::packet::key::UnspecifiedRole>>>
+/// Returns suitable signing keys from a given list of Certs.
+fn get_signing_keys(certs: &[openpgp::Cert], p: &dyn Policy,
+                    timestamp: Option<SystemTime>)
+    -> Result<Vec<crypto::KeyPair>>
 {
     let mut keys = Vec::new();
-    'next_tpk: for tsk in tpks {
-        for key in tsk.keys_valid()
-            .signing_capable()
-            .map(|k| k.2)
+    'next_cert: for tsk in certs {
+        for key in tsk.keys().with_policy(p, timestamp).alive().revoked(false)
+            .for_signing()
+            .map(|ka| ka.key())
         {
             if let Some(secret) = key.secret() {
                 let unencrypted = match secret {
@@ -74,7 +69,7 @@ fn get_signing_keys(tpks: &[openpgp::TPK])
 
                 keys.push(crypto::KeyPair::new(key.clone(), unencrypted)
                           .unwrap());
-                break 'next_tpk;
+                break 'next_cert;
             }
         }
 
@@ -85,17 +80,18 @@ fn get_signing_keys(tpks: &[openpgp::TPK])
     Ok(keys)
 }
 
-pub fn encrypt(store: &mut store::Store,
-               input: &mut io::Read, output: &mut io::Write,
+pub fn encrypt(policy: &dyn Policy,
+               mapping: &mut store::Mapping,
+               input: &mut dyn io::Read, output: &mut dyn io::Write,
                npasswords: usize, recipients: Vec<&str>,
-               mut tpks: Vec<openpgp::TPK>, signers: Vec<openpgp::TPK>,
-               mode: KeyFlags,
-               compression: &str)
+               mut certs: Vec<openpgp::Cert>, signers: Vec<openpgp::Cert>,
+               mode: openpgp::types::KeyFlags, compression: &str,
+               time: Option<SystemTime>)
                -> Result<()> {
     for r in recipients {
-        tpks.push(store.lookup(r).context("No such key found")?.tpk()?);
+        certs.push(mapping.lookup(r).context("No such key found")?.cert()?);
     }
-    let mut passwords = Vec::with_capacity(npasswords);
+    let mut passwords: Vec<crypto::Password> = Vec::with_capacity(npasswords);
     for n in 0..npasswords {
         let nprompt = format!("Enter password {}: ", n + 1);
         passwords.push(rpassword::read_password_from_tty(Some(
@@ -106,62 +102,79 @@ pub fn encrypt(store: &mut store::Store,
             }))?.into());
     }
 
-    let mut signers = get_signing_keys(&signers)?;
+    if certs.len() + passwords.len() == 0 {
+        return Err(failure::format_err!(
+            "Neither recipient nor password given"));
+    }
+
+    let mut signers = get_signing_keys(&signers, policy, time)?;
 
     // Build a vector of references to hand to Signer.
-    let recipients: Vec<&openpgp::TPK> = tpks.iter().collect();
+    let recipients: Vec<&openpgp::Cert> = certs.iter().collect();
 
     // Build a vector of recipients to hand to Encryptor.
     let mut recipient_subkeys: Vec<Recipient> = Vec::new();
-    for tpk in tpks.iter() {
+    for cert in certs.iter() {
         let mut count = 0;
-        for (_, _, key) in tpk.keys_valid().key_flags(mode.clone()) {
+        for key in cert.keys().with_policy(policy, None).alive().revoked(false)
+            .key_flags(&mode).map(|ka| ka.key())
+        {
             recipient_subkeys.push(key.into());
             count += 1;
         }
         if count == 0 {
             return Err(failure::format_err!(
-                "Key {} has no suitable encryption key", tpk));
+                "Key {} has no suitable encryption key", cert));
         }
     }
 
-    let passwords_: Vec<&openpgp::crypto::Password> =
-        passwords.iter().collect();
-
     // Stream an OpenPGP message.
     let message = Message::new(output);
 
     // We want to encrypt a literal data packet.
-    let mut sink = Encryptor::new(message,
-                                  &passwords_,
-                                  recipient_subkeys,
-                                  None, None)
+    let mut encryptor = if let Some(p) = passwords.pop() {
+        Encryptor::with_password(message, p)
+    } else {
+        Encryptor::for_recipient(message, recipient_subkeys.pop().unwrap())
+    };
+    for p in passwords {
+        encryptor = encryptor.add_password(p);
+    }
+    for r in recipient_subkeys {
+        encryptor = encryptor.add_recipient(r);
+    }
+
+    let mut sink = encryptor.build()
         .context("Failed to create encryptor")?;
 
     match compression {
         "none" => (),
         "pad" => sink = Padder::new(sink, padme)?,
-        "zip" =>
-            sink = Compressor::new(sink, CompressionAlgorithm::Zip, None)?,
-        "zlib" =>
-            sink = Compressor::new(sink, CompressionAlgorithm::Zlib, None)?,
-        "bzip2" =>
-            sink = Compressor::new(sink, CompressionAlgorithm::BZip2, None)?,
+        "zip" => sink =
+            Compressor::new(sink).algo(CompressionAlgorithm::Zip).build()?,
+        "zlib" => sink =
+            Compressor::new(sink).algo(CompressionAlgorithm::Zlib).build()?,
+        "bzip2" => sink =
+            Compressor::new(sink).algo(CompressionAlgorithm::BZip2).build()?,
         _ => unreachable!("all possible choices are handled")
     }
 
     // Optionally sign message.
     if ! signers.is_empty() {
-        sink = Signer::with_intended_recipients(
-            sink,
-            signers.iter_mut().map(|s| -> &mut dyn crypto::Signer<_> { s })
-                .collect(),
-            &recipients,
-            None)?;
+        let mut signer = Signer::new(sink, signers.pop().unwrap());
+        for s in signers {
+            signer = signer.add_signer(s);
+            if let Some(time) = time {
+                signer = signer.creation_time(time);
+            }
+        }
+        for r in recipients {
+            signer = signer.add_intended_recipient(r);
+        }
+        sink = signer.build()?;
     }
 
-    let mut literal_writer = LiteralWriter::new(sink, DataFormat::Binary,
-                                                None, None)
+    let mut literal_writer = LiteralWriter::new(sink).build()
         .context("Failed to create literal writer")?;
 
     // Finally, copy stdin to our writer stack to encrypt the data.
@@ -176,9 +189,9 @@ pub fn encrypt(store: &mut store::Store,
 
 struct VHelper<'a> {
     ctx: &'a Context,
-    store: &'a mut store::Store,
+    mapping: &'a mut store::Mapping,
     signatures: usize,
-    tpks: Option<Vec<TPK>>,
+    certs: Option<Vec<Cert>>,
     labels: HashMap<KeyID, String>,
     trusted: HashSet<KeyID>,
     good_signatures: usize,
@@ -186,17 +199,18 @@ struct VHelper<'a> {
     unknown_checksums: usize,
     bad_signatures: usize,
     bad_checksums: usize,
+    broken_signatures: usize,
 }
 
 impl<'a> VHelper<'a> {
-    fn new(ctx: &'a Context, store: &'a mut store::Store, signatures: usize,
-           tpks: Vec<TPK>)
+    fn new(ctx: &'a Context, mapping: &'a mut store::Mapping, signatures: usize,
+           certs: Vec<Cert>)
            -> Self {
         VHelper {
             ctx: ctx,
-            store: store,
+            mapping: mapping,
             signatures: signatures,
-            tpks: Some(tpks),
+            certs: Some(certs),
             labels: HashMap::new(),
             trusted: HashSet::new(),
             good_signatures: 0,
@@ -204,6 +218,7 @@ impl<'a> VHelper<'a> {
             unknown_checksums: 0,
             bad_signatures: 0,
             bad_checksums: 0,
+            broken_signatures: 0,
         }
     }
 
@@ -224,6 +239,7 @@ impl<'a> VHelper<'a> {
         p(&mut dirty, "unknown checksum", self.unknown_checksums);
         p(&mut dirty, "bad signature", self.bad_signatures);
         p(&mut dirty, "bad checksum", self.bad_checksums);
+        p(&mut dirty, "broken signatures", self.broken_signatures);
         if dirty {
             eprintln!(".");
         }
@@ -233,14 +249,41 @@ impl<'a> VHelper<'a> {
         use self::VerificationResult::*;
         for result in results {
             let (issuer, level) = match result {
-                GoodChecksum(ref sig, ..) => (sig.get_issuer(), sig.level()),
-                MissingKey(ref sig) => (sig.get_issuer(), sig.level()),
-                BadChecksum(ref sig) => (sig.get_issuer(), sig.level()),
+                GoodChecksum { sig, ka, .. } =>
+                    (ka.key().keyid(), sig.level()),
+                NotAlive { sig, .. } =>
+                    (sig.get_issuers().get(0)
+                         .map(|i| i.into())
+                         .unwrap_or(KeyID::wildcard()),
+                     sig.level()),
+                MissingKey { sig } => {
+                    let issuer = sig.get_issuers().get(0)
+                        .expect("missing key checksum has an issuer")
+                        .to_string();
+                    let what = match sig.level() {
+                        0 => "checksum".into(),
+                        n => format!("level {} notarizing checksum", n),
+                    };
+                    eprintln!("No key to check {} from {}", what, issuer);
+                    self.unknown_checksums += 1;
+                    continue;
+                }
+                Error { sig, error } => {
+                    let issuer = sig.get_issuers().get(0)
+                        .expect("key has an issuer")
+                        .to_string();
+                    let what = match sig.level() {
+                        0 => "checksum".into(),
+                        n => format!("level {} notarizing checksum", n),
+                    };
+                    eprintln!("Error verifying {} from {}: {}",
+                              what, issuer, error);
+                    self.broken_signatures += 1;
+                    continue;
+                }
             };
 
-            let trusted = issuer.as_ref().map(|i| {
-                self.trusted.contains(&i)
-            }).unwrap_or(false);
+            let trusted = self.trusted.contains(&issuer);
             let what = match (level == 0, trusted) {
                 (true,  true)  => "signature".into(),
                 (false, true)  => format!("level {} notarization", level),
@@ -249,74 +292,61 @@ impl<'a> VHelper<'a> {
                     format!("level {} notarizing checksum", level),
             };
 
+            let issuer_str = issuer.to_string();
+            let label = self.labels.get(&issuer).unwrap_or(&issuer_str);
             match result {
-                GoodChecksum(..) => {
-                    let issuer = issuer
-                        .expect("good checksum has an issuer");
-                    let issuer_str = format!("{}", issuer);
-                    eprintln!("Good {} from {}", what,
-                              self.labels.get(&issuer).unwrap_or(
-                                  &issuer_str));
+                GoodChecksum { .. } => {
+                    eprintln!("Good {} from {}", what, label);
                     if trusted {
                         self.good_signatures += 1;
                     } else {
                         self.good_checksums += 1;
                     }
                 },
-                MissingKey(_) => {
-                    let issuer = issuer
-                        .expect("missing key checksum has an issuer");
-                    eprintln!("No key to check {} from {}", what, issuer);
-                    self.unknown_checksums += 1;
-                },
-                BadChecksum(_) => {
-                    if let Some(issuer) = issuer {
-                        let issuer_str = format!("{}", issuer);
-                        eprintln!("Bad {} from {}", what,
-                                  self.labels.get(&issuer).unwrap_or(
-                                      &issuer_str));
-                    } else {
-                        eprintln!("Bad {} without issuer information",
-                                  what);
-                    }
+                NotAlive { .. } => {
+                    eprintln!("Good checksum, but not alive: {} from {}",
+                              what, label);
                     if trusted {
                         self.bad_signatures += 1;
                     } else {
                         self.bad_checksums += 1;
                     }
                 },
+                MissingKey { .. } => unreachable!("handled above"),
+                Error { .. } => unreachable!("handled above"),
             }
         }
     }
 }
 
 impl<'a> VerificationHelper for VHelper<'a> {
-    fn get_public_keys(&mut self, ids: &[KeyID]) -> Result<Vec<TPK>> {
-        let mut tpks = self.tpks.take().unwrap();
-        let seen: HashSet<_> = tpks.iter()
-            .flat_map(|tpk| {
-                // Even if a key is revoked or expired, we can still
-                // use it to verify a message.
-                tpk.keys_all().map(|(_, _, key)| key.fingerprint().to_keyid())
+    fn get_public_keys(&mut self, ids: &[openpgp::KeyHandle]) -> Result<Vec<Cert>> {
+        let mut certs = self.certs.take().unwrap();
+        // Get all keys.
+        let seen: HashSet<_> = certs.iter()
+            .flat_map(|cert| {
+                cert.keys().map(|ka| ka.key().fingerprint().into())
             }).collect();
 
         // Explicitly provided keys are trusted.
         self.trusted = seen.clone();
 
-        // Try to get missing TPKs from the store.
-        for id in ids.iter().filter(|i| !seen.contains(i)) {
+        // Try to get missing Certs from the mapping.
+        for id in ids.iter().map(|i| KeyID::from(i))
+            .filter(|i| !seen.contains(i))
+        {
             let _ =
-                self.store.lookup_by_subkeyid(id)
+                self.mapping.lookup_by_subkeyid(&id)
                 .and_then(|binding| {
                     self.labels.insert(id.clone(), binding.label()?);
 
-                    // Keys from our store are trusted.
+                    // Keys from our mapping are trusted.
                     self.trusted.insert(id.clone());
 
-                    binding.tpk()
+                    binding.cert()
                 })
-                .and_then(|tpk| {
-                    tpks.push(tpk);
+                .and_then(|cert| {
+                    certs.push(cert);
                     Ok(())
                 });
         }
@@ -324,23 +354,25 @@ impl<'a> VerificationHelper for VHelper<'a> {
         // Update seen.
         let seen = self.trusted.clone();
 
-        // Try to get missing TPKs from the pool.
-        for id in ids.iter().filter(|i| !seen.contains(i)) {
+        // Try to get missing Certs from the pool.
+        for id in ids.iter().map(|i| KeyID::from(i.clone()))
+            .filter(|i| !seen.contains(i))
+        {
             let _ =
-                store::Pool::lookup_by_subkeyid(self.ctx, id)
+                store::Store::lookup_by_subkeyid(self.ctx, &id)
                 .and_then(|key| {
                     // Keys from the pool are NOT trusted.
-                    key.tpk()
+                    key.cert()
                 })
-                .and_then(|tpk| {
-                    tpks.push(tpk);
+                .and_then(|cert| {
+                    certs.push(cert);
                     Ok(())
                 });
         }
-        Ok(tpks)
+        Ok(certs)
     }
 
-    fn check(&mut self, structure: &MessageStructure) -> Result<()> {
+    fn check(&mut self, structure: MessageStructure) -> Result<()> {
         for layer in structure.iter() {
             match layer {
                 MessageLayer::Compression { algo } =>
@@ -367,17 +399,18 @@ impl<'a> VerificationHelper for VHelper<'a> {
     }
 }
 
-pub fn verify(ctx: &Context, store: &mut store::Store,
-              input: &mut io::Read,
-              detached: Option<&mut io::Read>,
-              output: &mut io::Write,
-              signatures: usize, tpks: Vec<TPK>)
+pub fn verify(ctx: &Context, policy: &dyn Policy,
+              mapping: &mut store::Mapping,
+              input: &mut dyn io::Read,
+              detached: Option<&mut dyn io::Read>,
+              output: &mut dyn io::Write,
+              signatures: usize, certs: Vec<Cert>)
               -> Result<()> {
-    let helper = VHelper::new(ctx, store, signatures, tpks);
+    let helper = VHelper::new(ctx, mapping, signatures, certs);
     let mut verifier = if let Some(dsig) = detached {
-        DetachedVerifier::from_reader(dsig, input, helper, None)?
+        DetachedVerifier::from_reader(policy, dsig, input, helper, None)?
     } else {
-        Verifier::from_reader(input, helper, None)?
+        Verifier::from_reader(policy, input, helper, None)?
     };
 
     io::copy(&mut verifier, output)
@@ -393,7 +426,7 @@ pub fn verify(ctx: &Context, store: &mut store::Store,
     Ok(())
 }
 
-pub fn split(input: &mut io::Read, prefix: &str)
+pub fn split(input: &mut dyn io::Read, prefix: &str)
              -> Result<()> {
     // We (ab)use the mapping feature to create byte-accurate dumps of
     // nested packets.
@@ -407,16 +440,17 @@ pub fn split(input: &mut io::Read, prefix: &str)
     while let PacketParserResult::Some(pp) = ppr {
         if let Some(ref map) = pp.map() {
             let filename = format!(
-                "{}{}--{:?}", prefix,
+                "{}{}--{}{:?}", prefix,
                 pos.iter().map(|n| format!("{}", n))
                     .collect::<Vec<String>>().join("-"),
+                pp.packet.kind().map(|_| "").unwrap_or("Unknown-"),
                 pp.packet.tag());
             let mut sink = File::create(filename)
                 .context("Failed to create output file")?;
 
             // Write all the bytes.
             for field in map.iter() {
-                sink.write_all(field.data)?;
+                sink.write_all(field.data())?;
             }
         }
 
@@ -439,17 +473,17 @@ pub fn split(input: &mut io::Read, prefix: &str)
 }
 
 /// Joins the given files.
-pub fn join(inputs: Option<clap::Values>, output: &mut io::Write)
+pub fn join(inputs: Option<clap::Values>, output: &mut dyn io::Write)
             -> Result<()> {
     /// Writes a bit-accurate copy of all top-level packets in PPR to
     /// OUTPUT.
-    fn copy(mut ppr: PacketParserResult, output: &mut io::Write)
+    fn copy(mut ppr: PacketParserResult, output: &mut dyn io::Write)
             -> Result<()> {
         while let PacketParserResult::Some(pp) = ppr {
             // We (ab)use the mapping feature to create byte-accurate
             // copies.
             for field in pp.map().expect("must be mapped").iter() {
-                output.write_all(field.data)?;
+                output.write_all(field.data())?;
             }
 
             ppr = pp.next()?.1;
@@ -473,24 +507,24 @@ pub fn join(inputs: Option<clap::Values>, output: &mut io::Write)
     Ok(())
 }
 
-pub fn store_print_stats(store: &store::Store, label: &str) -> Result<()> {
+pub fn mapping_print_stats(mapping: &store::Mapping, label: &str) -> Result<()> {
     fn print_stamps(st: &store::Stamps) -> Result<()> {
         println!("{} messages using this key", st.count);
         if let Some(t) = st.first {
-            println!("    First: {}", tm2str(&time::at(t)));
+            println!("    First: {}", t.convert());
         }
         if let Some(t) = st.last {
-            println!("    Last: {}", tm2str(&time::at(t)));
+            println!("    Last: {}", t.convert());
         }
         Ok(())
     }
 
     fn print_stats(st: &store::Stats) -> Result<()> {
         if let Some(t) = st.created {
-            println!("  Created: {}", tm2str(&time::at(t)));
+            println!("  Created: {}", t.convert());
         }
         if let Some(t) = st.updated {
-            println!("  Updated: {}", tm2str(&time::at(t)));
+            println!("  Updated: {}", t.convert());
         }
         print!("  Encrypted ");
         print_stamps(&st.encryption)?;
@@ -499,7 +533,7 @@ pub fn store_print_stats(store: &store::Store, label: &str) -> Result<()> {
         Ok(())
     }
 
-    let binding = store.lookup(label)?;
+    let binding = mapping.lookup(label)?;
     println!("Binding {:?}", label);
     print_stats(&binding.stats().context("Failed to get stats")?)?;
     let key = binding.key().context("Failed to get key")?;
