@@ -7,36 +7,41 @@ extern crate failure;
 extern crate prettytable;
 extern crate rpassword;
 extern crate tempfile;
-extern crate termsize;
-extern crate time;
+extern crate crossterm;
 extern crate itertools;
 extern crate tokio_core;
 
+use crossterm::terminal;
 use failure::ResultExt;
 use prettytable::{Table, Cell, Row};
 use std::fs::{File, OpenOptions};
-use std::io;
+use std::io::{self, Write};
 use std::path::{Path, PathBuf};
 use std::process::exit;
+use chrono::{DateTime, offset::Utc};
 
 extern crate sequoia_openpgp as openpgp;
 extern crate sequoia_core;
 extern crate sequoia_net;
-extern crate sequoia_store;
+extern crate sequoia_store as store;
 
-use crate::openpgp::{armor, autocrypt, Fingerprint, TPK};
-use crate::openpgp::conversions::hex;
-use crate::openpgp::packet::KeyFlags;
+use crate::openpgp::{armor, Fingerprint, Cert};
+use sequoia_autocrypt as autocrypt;
+use crate::openpgp::fmt::hex;
+use crate::openpgp::types::KeyFlags;
 use crate::openpgp::parse::Parse;
 use crate::openpgp::serialize::Serialize;
+use crate::openpgp::cert::CertParser;
+use crate::openpgp::policy::StandardPolicy as P;
 use sequoia_core::{Context, NetworkPolicy};
 use sequoia_net::{KeyServer, wkd};
-use sequoia_store::{Store, LogIter};
+use store::{Mapping, LogIter};
 
 mod sq_cli;
 mod commands;
+use commands::dump::Convert;
 
-fn open_or_stdin(f: Option<&str>) -> Result<Box<io::Read>, failure::Error> {
+fn open_or_stdin(f: Option<&str>) -> Result<Box<dyn io::Read>, failure::Error> {
     match f {
         Some(f) => Ok(Box::new(File::open(f)
                                .context("Failed to open input file")?)),
@@ -45,7 +50,7 @@ fn open_or_stdin(f: Option<&str>) -> Result<Box<io::Read>, failure::Error> {
 }
 
 fn create_or_stdout(f: Option<&str>, force: bool)
-    -> Result<Box<io::Write>, failure::Error> {
+    -> Result<Box<dyn io::Write>, failure::Error> {
     match f {
         None => Ok(Box::new(io::stdout())),
         Some(p) if p == "-" => Ok(Box::new(io::stdout())),
@@ -66,38 +71,109 @@ fn create_or_stdout(f: Option<&str>, force: bool)
     }
 }
 
-fn load_tpks<'a, I>(files: I) -> openpgp::Result<Vec<TPK>>
+// XXX: This is a candidate for inclusion in the library.
+enum Writer<T: Write> {
+    Binary {
+        inner: T,
+    },
+    Armored {
+        inner: openpgp::armor::Writer<T>,
+    },
+}
+
+impl<T: Write> From<T> for Writer<T> {
+    fn from(inner: T) -> Self {
+        Writer::Binary { inner }
+    }
+}
+
+impl<T: Write> From<openpgp::armor::Writer<T>> for Writer<T> {
+    fn from(inner: openpgp::armor::Writer<T>) -> Self {
+        Writer::Armored { inner }
+    }
+}
+
+impl<T: Write> Writer<T> {
+    pub fn armor(self, kind: openpgp::armor::Kind, headers: &[(&str, &str)])
+                 -> openpgp::Result<Self>
+    {
+        match self {
+            Writer::Binary { inner } =>
+                Ok(openpgp::armor::Writer::new(inner, kind, headers)?
+                   .into()),
+            Writer::Armored { .. } =>
+                Err(openpgp::Error::InvalidOperation("already armored".into())
+                    .into()),
+        }
+    }
+
+    pub fn finalize(self) -> std::io::Result<T> {
+        match self {
+            Writer::Binary { inner } => Ok(inner),
+            Writer::Armored { inner } => inner.finalize(),
+        }
+    }
+}
+
+impl<T: Write> Write for Writer<T> {
+    fn write(&mut self, buf: &[u8]) -> std::io::Result<usize> {
+        match self {
+            Writer::Binary { inner } => inner.write(buf),
+            Writer::Armored { inner } => inner.write(buf),
+        }
+    }
+    fn flush(&mut self) -> std::io::Result<()> {
+        match self {
+            Writer::Binary { inner } => inner.flush(),
+            Writer::Armored { inner } => inner.flush(),
+        }
+    }
+}
+
+fn create_or_stdout_pgp(f: Option<&str>, force: bool,
+                        binary: bool, kind: armor::Kind)
+    -> Result<Writer<Box<dyn Write>>, failure::Error>
+{
+    let sink = create_or_stdout(f, force)?;
+    let mut sink = Writer::from(sink);
+    if ! binary {
+        sink = sink.armor(kind, &[])?;
+    }
+    Ok(sink)
+}
+
+fn load_certs<'a, I>(files: I) -> openpgp::Result<Vec<Cert>>
     where I: Iterator<Item=&'a str>
 {
-    let mut tpks = vec![];
+    let mut certs = vec![];
     for f in files {
-        tpks.push(TPK::from_file(f)
+        certs.push(Cert::from_file(f)
                   .context(format!("Failed to load key from file {:?}", f))?);
     }
-    Ok(tpks)
+    Ok(certs)
 }
 
 /// Serializes a keyring, adding descriptive headers if armored.
-fn serialize_keyring(mut output: &mut io::Write, tpks: &[TPK], binary: bool)
+fn serialize_keyring(mut output: &mut dyn io::Write, certs: &[Cert], binary: bool)
                      -> openpgp::Result<()> {
     // Handle the easy options first.  No armor no cry:
     if binary {
-        for tpk in tpks {
-            tpk.serialize(&mut output)?;
+        for cert in certs {
+            cert.serialize(&mut output)?;
         }
         return Ok(());
     }
 
-    // Just one TPK?  Ez:
-    if tpks.len() == 1 {
-        return tpks[0].armored().serialize(&mut output);
+    // Just one Cert?  Ez:
+    if certs.len() == 1 {
+        return certs[0].armored().serialize(&mut output);
     }
 
     // Otherwise, collect the headers first:
     let mut headers = Vec::new();
-    for (i, tpk) in tpks.iter().enumerate() {
+    for (i, cert) in certs.iter().enumerate() {
         headers.push(format!("Key #{}", i));
-        headers.append(&mut tpk.armor_headers());
+        headers.append(&mut cert.armor_headers());
     }
 
     let headers: Vec<_> = headers.iter()
@@ -106,9 +182,10 @@ fn serialize_keyring(mut output: &mut io::Write, tpks: &[TPK], binary: bool)
     let mut output = armor::Writer::new(&mut output,
                                         armor::Kind::PublicKey,
                                         &headers)?;
-    for tpk in tpks {
-        tpk.serialize(&mut output)?;
+    for cert in certs {
+        cert.serialize(&mut output)?;
     }
+    output.finalize()?;
     Ok(())
 }
 
@@ -136,9 +213,11 @@ fn help_warning(arg: &str) {
 }
 
 fn real_main() -> Result<(), failure::Error> {
+    let policy = &P::new();
+
     let matches = sq_cli::build().get_matches();
 
-    let policy = match matches.value_of("policy") {
+    let network_policy = match matches.value_of("policy") {
         None => NetworkPolicy::Encrypted,
         Some("offline") => NetworkPolicy::Offline,
         Some("anonymized") => NetworkPolicy::Anonymized,
@@ -150,8 +229,8 @@ fn real_main() -> Result<(), failure::Error> {
         },
     };
     let force = matches.is_present("force");
-    let (realm_name, store_name) = {
-        let s = matches.value_of("store").expect("has a default value");
+    let (realm_name, mapping_name) = {
+        let s = matches.value_of("mapping").expect("has a default value");
         if let Some(i) = s.find('/') {
             (&s[..i], &s[i+1..])
         } else {
@@ -159,7 +238,7 @@ fn real_main() -> Result<(), failure::Error> {
         }
     };
     let mut builder = Context::configure()
-        .network_policy(policy);
+        .network_policy(network_policy);
     if let Some(dir) = matches.value_of("home") {
         builder = builder.home(dir);
     }
@@ -172,56 +251,61 @@ fn real_main() -> Result<(), failure::Error> {
             let mut output = create_or_stdout(m.value_of("output"), force)?;
             let signatures: usize =
                 m.value_of("signatures").unwrap_or("0").parse()?;
-            let tpks = m.values_of("public-key-file")
-                .map(load_tpks)
+            let certs = m.values_of("sender-cert-file")
+                .map(load_certs)
                 .unwrap_or(Ok(vec![]))?;
             let secrets = m.values_of("secret-key-file")
-                .map(load_tpks)
+                .map(load_certs)
                 .unwrap_or(Ok(vec![]))?;
-            let mut store = Store::open(&ctx, realm_name, store_name)
-                .context("Failed to open the store")?;
-            commands::decrypt(&ctx, &mut store,
+            let mut mapping = Mapping::open(&ctx, realm_name, mapping_name)
+                .context("Failed to open the mapping")?;
+            commands::decrypt(&ctx, policy, &mut mapping,
                               &mut input, &mut output,
-                              signatures, tpks, secrets,
+                              signatures, certs, secrets,
                               m.is_present("dump-session-key"),
                               m.is_present("dump"), m.is_present("hex"))?;
         },
         ("encrypt",  Some(m)) => {
             let mut input = open_or_stdin(m.value_of("input"))?;
-            let mut output = create_or_stdout(m.value_of("output"), force)?;
-            let mut output = if ! m.is_present("binary") {
-                Box::new(armor::Writer::new(&mut output,
-                                            armor::Kind::Message,
-                                            &[])?)
-            } else {
-                output
-            };
-            let mut store = Store::open(&ctx, realm_name, store_name)
-                .context("Failed to open the store")?;
+            let mut output =
+                create_or_stdout_pgp(m.value_of("output"), force,
+                                     m.is_present("binary"),
+                                     armor::Kind::Message)?;
+            let mut mapping = Mapping::open(&ctx, realm_name, mapping_name)
+                .context("Failed to open the mapping")?;
             let recipients = m.values_of("recipient")
                 .map(|r| r.collect())
                 .unwrap_or(vec![]);
-            let additional_tpks = m.values_of("recipient-key-file")
-                .map(load_tpks)
+            let additional_certs = m.values_of("recipient-key-file")
+                .map(load_certs)
                 .unwrap_or(Ok(vec![]))?;
             let additional_secrets = m.values_of("signer-key-file")
-                .map(load_tpks)
+                .map(load_certs)
                 .unwrap_or(Ok(vec![]))?;
             let mode = match m.value_of("mode").expect("has default") {
                 "rest" => KeyFlags::default()
-                    .set_encrypt_at_rest(true),
+                    .set_storage_encryption(true),
                 "transport" => KeyFlags::default()
-                    .set_encrypt_for_transport(true),
+                    .set_transport_encryption(true),
                 "all" => KeyFlags::default()
-                    .set_encrypt_at_rest(true)
-                    .set_encrypt_for_transport(true),
+                    .set_storage_encryption(true)
+                    .set_transport_encryption(true),
                 _ => unreachable!("uses possible_values"),
             };
-            commands::encrypt(&mut store, &mut input, &mut output,
+            let time = if let Some(time) = m.value_of("time") {
+                Some(parse_iso8601(time, chrono::NaiveTime::from_hms(0, 0, 0))
+                         .context(format!("Bad value passed to --time: {:?}",
+                                          time))?.into())
+            } else {
+                None
+            };
+            commands::encrypt(policy, &mut mapping, &mut input, &mut output,
                               m.occurrences_of("symmetric") as usize,
-                              recipients, additional_tpks, additional_secrets,
+                              recipients, additional_certs, additional_secrets,
                               mode,
-                              m.value_of("compression").expect("has default"))?;
+                              m.value_of("compression").expect("has default"),
+                              time.into())?;
+            output.finalize()?;
         },
         ("sign",  Some(m)) => {
             let mut input = open_or_stdin(m.value_of("input"))?;
@@ -231,10 +315,17 @@ fn real_main() -> Result<(), failure::Error> {
             let append = m.is_present("append");
             let notarize = m.is_present("notarize");
             let secrets = m.values_of("secret-key-file")
-                .map(load_tpks)
+                .map(load_certs)
                 .unwrap_or(Ok(vec![]))?;
-            commands::sign(&mut input, output, secrets, detached, binary,
-                           append, notarize, force)?;
+            let time = if let Some(time) = m.value_of("time") {
+                Some(parse_iso8601(time, chrono::NaiveTime::from_hms(0, 0, 0))
+                         .context(format!("Bad value passed to --time: {:?}",
+                                          time))?.into())
+            } else {
+                None
+            };
+            commands::sign(policy, &mut input, output, secrets, detached, binary,
+                           append, notarize, time, force)?;
         },
         ("verify",  Some(m)) => {
             let mut input = open_or_stdin(m.value_of("input"))?;
@@ -246,22 +337,24 @@ fn real_main() -> Result<(), failure::Error> {
             };
             let signatures: usize =
                 m.value_of("signatures").unwrap_or("0").parse()?;
-            let tpks = m.values_of("public-key-file")
-                .map(load_tpks)
+            let certs = m.values_of("sender-cert-file")
+                .map(load_certs)
                 .unwrap_or(Ok(vec![]))?;
-            let mut store = Store::open(&ctx, realm_name, store_name)
-                .context("Failed to open the store")?;
-            commands::verify(&ctx, &mut store, &mut input,
-                             detached.as_mut().map(|r| r as &mut io::Read),
-                             &mut output, signatures, tpks)?;
+            let mut mapping = Mapping::open(&ctx, realm_name, mapping_name)
+                .context("Failed to open the mapping")?;
+            commands::verify(&ctx, policy, &mut mapping, &mut input,
+                             detached.as_mut().map(|r| r as &mut dyn io::Read),
+                             &mut output, signatures, certs)?;
         },
 
         ("enarmor",  Some(m)) => {
             let mut input = open_or_stdin(m.value_of("input"))?;
-            let mut output = create_or_stdout(m.value_of("output"), force)?;
-            let kind = parse_armor_kind(m.value_of("kind"));
-            let mut filter = armor::Writer::new(&mut output, kind, &[])?;
-            io::copy(&mut input, &mut filter)?;
+            let mut output =
+                create_or_stdout_pgp(m.value_of("output"), force,
+                                     false,
+                                     parse_armor_kind(m.value_of("kind")))?;
+            io::copy(&mut input, &mut output)?;
+            output.finalize()?;
         },
         ("dearmor",  Some(m)) => {
             let mut input = open_or_stdin(m.value_of("input"))?;
@@ -273,33 +366,31 @@ fn real_main() -> Result<(), failure::Error> {
             match m.subcommand() {
                 ("decode",  Some(m)) => {
                     let input = open_or_stdin(m.value_of("input"))?;
-                    let mut output = create_or_stdout(m.value_of("output"), force)?;
+                    let mut output =
+                        create_or_stdout_pgp(m.value_of("output"), force,
+                                             true,
+                                             armor::Kind::PublicKey)?;
                     let ac = autocrypt::AutocryptHeaders::from_reader(input)?;
                     for h in &ac.headers {
-                        if let Some(ref tpk) = h.key {
-                            let mut filter = armor::Writer::new(
-                                &mut output, armor::Kind::PublicKey, &[])?;
-                            tpk.serialize(&mut filter)?;
+                        if let Some(ref cert) = h.key {
+                            cert.serialize(&mut output)?;
                         }
                     }
+                    output.finalize()?;
                 },
                 ("encode-sender",  Some(m)) => {
                     let input = open_or_stdin(m.value_of("input"))?;
                     let mut output = create_or_stdout(m.value_of("output"),
                                                       force)?;
-                    let tpk = TPK::from_reader(input)?;
+                    let cert = Cert::from_reader(input)?;
                     let addr = m.value_of("address").map(|a| a.to_string())
                         .or_else(|| {
-                            if let Some(Ok(Some(a))) =
-                                tpk.userids().nth(0).map(|u| u.userid().address())
-                            {
-                                Some(a)
-                            } else {
-                                None
-                            }
+                            cert.primary_userid(policy, None)
+                                .map(|ca| ca.userid().to_string())
                         });
                     let ac = autocrypt::AutocryptHeader::new_sender(
-                        &tpk,
+                        policy,
+                        &cert,
                         &addr.ok_or(failure::err_msg(
                             "No well-formed primary userid found, use \
                              --address to specify one"))?,
@@ -313,7 +404,7 @@ fn real_main() -> Result<(), failure::Error> {
 
         ("inspect",  Some(m)) => {
             let mut output = create_or_stdout(m.value_of("output"), force)?;
-            commands::inspect(m, &mut output)?;
+            commands::inspect(m, policy, &mut output)?;
         },
 
         ("packet", Some(m)) => match m.subcommand() {
@@ -326,11 +417,30 @@ fn real_main() -> Result<(), failure::Error> {
                     } else {
                         None
                     };
-                let width = termsize::get().map(|s| s.cols as usize);
+                let width = terminal::size().ok().map(|(cols, _)| cols as usize);
                 commands::dump(&mut input, &mut output,
                                m.is_present("mpis"), m.is_present("hex"),
                                session_key.as_ref(), width)?;
             },
+
+            ("decrypt",  Some(m)) => {
+                let mut input = open_or_stdin(m.value_of("input"))?;
+                let mut output =
+                    create_or_stdout_pgp(m.value_of("output"), force,
+                                         m.is_present("binary"),
+                                         armor::Kind::Message)?;
+                let secrets = m.values_of("secret-key-file")
+                    .map(load_certs)
+                    .unwrap_or(Ok(vec![]))?;
+                let mut mapping = Mapping::open(&ctx, realm_name, mapping_name)
+                    .context("Failed to open the mapping")?;
+                commands::decrypt::decrypt_unwrap(
+                    &ctx, policy, &mut mapping,
+                    &mut input, &mut output,
+                    secrets, m.is_present("dump-session-key"))?;
+                output.finalize()?;
+            },
+
             ("split",  Some(m)) => {
                 let mut input = open_or_stdin(m.value_of("input"))?;
                 let prefix =
@@ -350,14 +460,12 @@ fn real_main() -> Result<(), failure::Error> {
                 commands::split(&mut input, &prefix)?;
             },
             ("join",  Some(m)) => {
-                let output = create_or_stdout(m.value_of("output"), force)?;
-                let mut output = if ! m.is_present("binary") {
-                    let kind = parse_armor_kind(m.value_of("kind"));
-                    Box::new(armor::Writer::new(output, kind, &[])?)
-                } else {
-                    output
-                };
+                let mut output =
+                    create_or_stdout_pgp(m.value_of("output"), force,
+                                         m.is_present("binary"),
+                                         parse_armor_kind(m.value_of("kind")))?;
                 commands::join(m.values_of("input"), &mut output)?;
+                output.finalize()?;
             },
             _ => unreachable!(),
         },
@@ -382,79 +490,79 @@ fn real_main() -> Result<(), failure::Error> {
                     let id = id.unwrap();
 
                     let mut output = create_or_stdout(m.value_of("output"), force)?;
-                    let tpk = core.run(ks.get(&id))
+                    let cert = core.run(ks.get(&id))
                         .context("Failed to retrieve key")?;
                     if ! m.is_present("binary") {
-                        tpk.armored().serialize(&mut output)
+                        cert.armored().serialize(&mut output)
                     } else {
-                        tpk.serialize(&mut output)
+                        cert.serialize(&mut output)
                     }.context("Failed to serialize key")?;
                 },
                 ("send",  Some(m)) => {
                     let mut input = open_or_stdin(m.value_of("input"))?;
-                    let tpk = TPK::from_reader(&mut input).
+                    let cert = Cert::from_reader(&mut input).
                         context("Malformed key")?;
 
-                    core.run(ks.send(&tpk))
+                    core.run(ks.send(&cert))
                         .context("Failed to send key to server")?;
                 },
                 _ => unreachable!(),
             }
         },
-        ("store",  Some(m)) => {
-            let store = Store::open(&ctx, realm_name, store_name)
-                .context("Failed to open the store")?;
+        ("mapping",  Some(m)) => {
+            let mapping = Mapping::open(&ctx, realm_name, mapping_name)
+                .context("Failed to open the mapping")?;
 
             match m.subcommand() {
                 ("list",  Some(_)) => {
-                    list_bindings(&store, realm_name, store_name)?;
+                    list_bindings(&mapping, realm_name, mapping_name)?;
                 },
                 ("add",  Some(m)) => {
                     let fp = Fingerprint::from_hex(m.value_of("fingerprint").unwrap())
                         .expect("Malformed fingerprint");
-                    store.add(m.value_of("label").unwrap(), &fp)?;
+                    mapping.add(m.value_of("label").unwrap(), &fp)?;
                 },
                 ("import",  Some(m)) => {
                     let label = m.value_of("label").unwrap();
                     help_warning(label);
                     let mut input = open_or_stdin(m.value_of("input"))?;
-                    let tpk = TPK::from_reader(&mut input)?;
-                    store.import(label, &tpk)?;
+                    let cert = Cert::from_reader(&mut input)?;
+                    mapping.import(label, &cert)?;
                 },
                 ("export",  Some(m)) => {
-                    let tpk = store.lookup(m.value_of("label").unwrap())?.tpk()?;
+                    let cert = mapping.lookup(m.value_of("label").unwrap())?.cert()?;
                     let mut output = create_or_stdout(m.value_of("output"), force)?;
                     if m.is_present("binary") {
-                        tpk.serialize(&mut output)?;
+                        cert.serialize(&mut output)?;
                     } else {
-                        tpk.armored().serialize(&mut output)?;
+                        cert.armored().serialize(&mut output)?;
                     }
                 },
                 ("delete",  Some(m)) => {
-                    if m.is_present("label") == m.is_present("the-store") {
-                        eprintln!("Please specify either a label or --the-store.");
+                    if m.is_present("label") == m.is_present("the-mapping") {
+                        eprintln!("Please specify either a label or --the-mapping.");
                         exit(1);
                     }
 
-                    if m.is_present("the-store") {
-                        store.delete().context("Failed to delete the store")?;
+                    if m.is_present("the-mapping") {
+                        mapping.delete().context("Failed to delete the mapping")?;
                     } else {
-                        let binding = store.lookup(m.value_of("label").unwrap())
+                        let binding = mapping.lookup(m.value_of("label").unwrap())
                             .context("Failed to get key")?;
                         binding.delete().context("Failed to delete the binding")?;
                     }
                 },
                 ("stats",  Some(m)) => {
-                    commands::store_print_stats(&store,
+                    commands::mapping_print_stats(&mapping,
                                                 m.value_of("label").unwrap())?;
                 },
                 ("log",  Some(m)) => {
                     if m.is_present("label") {
-                        let binding = store.lookup(m.value_of("label").unwrap())
+                        let binding = mapping.lookup(m.value_of("label").unwrap())
                             .context("No such key")?;
                         print_log(binding.log().context("Failed to get log")?, false);
                     } else {
-                        print_log(store.log().context("Failed to get log")?, true);
+                        print_log(mapping.log().context("Failed to get log")?, true);
                     }
                 },
                 _ => unreachable!(),
@@ -462,13 +570,13 @@ fn real_main() -> Result<(), failure::Error> {
         },
         ("list",  Some(m)) => {
             match m.subcommand() {
-                ("stores",  Some(m)) => {
+                ("mappings",  Some(m)) => {
                     let mut table = Table::new();
                     table.set_format(*prettytable::format::consts::FORMAT_NO_LINESEP_WITH_TITLE);
                     table.set_titles(row!["realm", "name", "network policy"]);
 
                     for (realm, name, network_policy, _)
-                        in Store::list(&ctx, m.value_of("prefix").unwrap_or(""))? {
+                        in Mapping::list(&ctx, m.value_of("prefix").unwrap_or(""))? {
                             table.add_row(Row::new(vec![
                                 Cell::new(&realm),
                                 Cell::new(&name),
@@ -479,9 +587,9 @@ fn real_main() -> Result<(), failure::Error> {
                     table.printstd();
                 },
                 ("bindings",  Some(m)) => {
-                    for (realm, name, _, store)
-                        in Store::list(&ctx, m.value_of("prefix").unwrap_or(""))? {
-                            list_bindings(&store, &realm, &name)?;
+                    for (realm, name, _, mapping)
+                        in Mapping::list(&ctx, m.value_of("prefix").unwrap_or(""))? {
+                            list_bindings(&mapping, &realm, &name)?;
                         }
                 },
                 ("keys",  Some(_)) => {
@@ -489,13 +597,13 @@ fn real_main() -> Result<(), failure::Error> {
                     table.set_format(*prettytable::format::consts::FORMAT_NO_LINESEP_WITH_TITLE);
                     table.set_titles(row!["fingerprint", "updated", "status"]);
 
-                    for (fingerprint, key) in Store::list_keys(&ctx)? {
+                    for (fingerprint, key) in store::Store::list_keys(&ctx)? {
                             let stats = key.stats()
                                 .context("Failed to get key stats")?;
                             table.add_row(Row::new(vec![
                                 Cell::new(&fingerprint.to_string()),
-                                if let Some(ref t) = stats.updated {
-                                    Cell::new(&format_time(t))
+                                if let Some(t) = stats.updated {
+                                    Cell::new(&t.convert().to_string())
                                 } else {
                                     Cell::new("")
                                 },
@@ -506,7 +614,7 @@ fn real_main() -> Result<(), failure::Error> {
                     table.printstd();
                 },
                 ("log",  Some(_)) => {
-                    print_log(Store::server_log(&ctx)?, true);
+                    print_log(store::Store::server_log(&ctx)?, true);
                 },
                 _ => unreachable!(),
             }
@@ -532,7 +640,7 @@ fn real_main() -> Result<(), failure::Error> {
                     // stderr and exit.
                     // Because it might be created a WkdServer struct, not
                     // doing it for now.
-                    let tpks = core.run(wkd::get(&email_address))?;
+                    let certs = core.run(wkd::get(&email_address))?;
                     // ```text
                     //     The HTTP GET method MUST return the binary representation of the
                     //     OpenPGP key for the given mail address.
@@ -541,9 +649,28 @@ fn real_main() -> Result<(), failure::Error> {
                     // But to keep the parallelism with `store export` and `keyserver get`,
                     // The output is armored if not `--binary` option is given.
                     let mut output = create_or_stdout(m.value_of("output"), force)?;
-                    serialize_keyring(&mut output, &tpks,
+                    serialize_keyring(&mut output, &certs,
                                       m.is_present("binary"))?;
                 },
+                ("generate", Some(m)) => {
+                    let domain = m.value_of("domain").unwrap();
+                    let f = open_or_stdin(m.value_of("input"))?;
+                    let base_path =
+                        m.value_of("base_directory").expect("required");
+                    let variant = if m.is_present("direct_method") {
+                        wkd::Variant::Direct
+                    } else {
+                        wkd::Variant::Advanced
+                    };
+                    let parser = CertParser::from_reader(f)?;
+                    let certs: Vec<Cert> = parser.filter_map(|cert| cert.ok())
+                        .collect();
+                    for cert in certs {
+                        wkd::insert(&base_path, domain, variant, &cert)
+                            .context(format!("Failed to generate the WKD in \
+                                              {}.", base_path))?;
+                    }
+                },
                 _ => unreachable!(),
             }
         },
@@ -553,18 +680,20 @@ fn real_main() -> Result<(), failure::Error> {
     return Ok(())
 }
 
-fn list_bindings(store: &Store, realm: &str, name: &str) -> Result<(), failure::Error> {
-    if store.iter()?.count() == 0 {
-        println!("No label-key bindings in the \"{}/{}\" store.", realm, name);
+fn list_bindings(mapping: &Mapping, realm: &str, name: &str)
+                 -> Result<(), failure::Error> {
+    if mapping.iter()?.count() == 0 {
+        println!("No label-key bindings in the \"{}/{}\" mapping.",
+                 realm, name);
         return Ok(());
     }
 
-    println!("Realm: {:?}, store: {:?}:", realm, name);
+    println!("Realm: {:?}, mapping: {:?}:", realm, name);
 
     let mut table = Table::new();
     table.set_format(*prettytable::format::consts::FORMAT_NO_LINESEP_WITH_TITLE);
     table.set_titles(row!["label", "fingerprint"]);
-    for (label, fingerprint, _) in store.iter()? {
+    for (label, fingerprint, _) in mapping.iter()? {
         table.add_row(Row::new(vec![
             Cell::new(&label),
             Cell::new(&fingerprint.to_string())]));
@@ -583,7 +712,7 @@ fn print_log(iter: LogIter, with_slug: bool) {
     table.set_titles(head);
 
     for entry in iter {
-        let mut row = row![&format_time(&entry.timestamp),
+        let mut row = row![&entry.timestamp.convert().to_string(),
                            &entry.short()];
         if with_slug {
             row.insert_cell(1, Cell::new(&entry.slug));
@@ -594,9 +723,76 @@ fn print_log(iter: LogIter, with_slug: bool) {
     table.printstd();
 }
 
-fn format_time(t: &time::Timespec) -> String {
-    time::strftime("%F %H:%M", &time::at(*t))
-    .unwrap() // Only parse errors can happen.
+/// Parses the given string depicting a ISO 8601 timestamp.
+fn parse_iso8601(s: &str, pad_date_with: chrono::NaiveTime)
+                 -> failure::Fallible<DateTime<Utc>>
+{
+    // If you modify this function this function, synchronize the
+    // changes with the copy in sqv.rs!
+    for f in &[
+        "%Y-%m-%dT%H:%M:%S%#z",
+        "%Y-%m-%dT%H:%M:%S",
+        "%Y-%m-%dT%H:%M%#z",
+        "%Y-%m-%dT%H:%M",
+        "%Y-%m-%dT%H%#z",
+        "%Y-%m-%dT%H",
+        "%Y%m%dT%H%M%S%#z",
+        "%Y%m%dT%H%M%S",
+        "%Y%m%dT%H%M%#z",
+        "%Y%m%dT%H%M",
+        "%Y%m%dT%H%#z",
+        "%Y%m%dT%H",
+    ] {
+        if f.ends_with("%#z") {
+            if let Ok(d) = DateTime::parse_from_str(s, *f) {
+                return Ok(d.into());
+            }
+        } else {
+            if let Ok(d) = chrono::NaiveDateTime::parse_from_str(s, *f) {
+                return Ok(DateTime::from_utc(d, Utc));
+            }
+        }
+    }
+    for f in &[
+        "%Y-%m-%d",
+        "%Y-%m",
+        "%Y-%j",
+        "%Y%m%d",
+        "%Y%m",
+        "%Y%j",
+        "%Y",
+    ] {
+        if let Ok(d) = chrono::NaiveDate::parse_from_str(s, *f) {
+            return Ok(DateTime::from_utc(d.and_time(pad_date_with), Utc));
+        }
+    }
+    Err(failure::format_err!("Malformed ISO8601 timestamp: {}", s))
+}
+
+#[test]
+fn test_parse_iso8601() {
+    let z = chrono::NaiveTime::from_hms(0, 0, 0);
+    parse_iso8601("2017-03-04T13:25:35Z", z).unwrap();
+    parse_iso8601("2017-03-04T13:25:35+08:30", z).unwrap();
+    parse_iso8601("2017-03-04T13:25:35", z).unwrap();
+    parse_iso8601("2017-03-04T13:25Z", z).unwrap();
+    parse_iso8601("2017-03-04T13:25", z).unwrap();
+    // parse_iso8601("2017-03-04T13Z", z).unwrap(); // XXX: chrono doesn't like
+    // parse_iso8601("2017-03-04T13", z).unwrap(); // ditto
+    parse_iso8601("2017-03-04", z).unwrap();
+    // parse_iso8601("2017-03", z).unwrap(); // ditto
+    parse_iso8601("2017-031", z).unwrap();
+    parse_iso8601("20170304T132535Z", z).unwrap();
+    parse_iso8601("20170304T132535+0830", z).unwrap();
+    parse_iso8601("20170304T132535", z).unwrap();
+    parse_iso8601("20170304T1325Z", z).unwrap();
+    parse_iso8601("20170304T1325", z).unwrap();
+    // parse_iso8601("20170304T13Z", z).unwrap(); // ditto
+    // parse_iso8601("20170304T13", z).unwrap(); // ditto
+    parse_iso8601("20170304", z).unwrap();
+    // parse_iso8601("201703", z).unwrap(); // ditto
+    parse_iso8601("2017031", z).unwrap();
+    // parse_iso8601("2017", z).unwrap(); // ditto
 }
 
 fn main() {
