@@ -14,11 +14,11 @@ pub fn build() -> App<'static, 'static> {
         .arg(Arg::with_name("home").value_name("DIRECTORY")
              .long("home")
              .help("Sets the home directory to use"))
-        .arg(Arg::with_name("store").value_name("STORE")
-             .long("store")
-             .short("s")
+        .arg(Arg::with_name("mapping").value_name("MAPPING")
+             .long("mapping")
+             .short("m")
              .default_value("org.sequoia-pgp.contacts/default")
-             .help("Sets the realm and store to use"))
+             .help("Sets the realm and mapping to use"))
         .arg(Arg::with_name("policy").value_name("NETWORK-POLICY")
              .long("policy")
              .short("p")
@@ -42,13 +42,14 @@ pub fn build() -> App<'static, 'static> {
                          .long("signatures")
                          .short("n")
                          .takes_value(true))
-                    .arg(Arg::with_name("public-key-file")
-                         .long("public-key-file")
+                    .arg(Arg::with_name("sender-cert-file")
+                         .long("sender-cert-file")
                          .multiple(true)
                          .takes_value(true)
-                         .value_name("TPK-FILE")
+                         .value_name("CERT-FILE")
                          .number_of_values(1)
-                         .help("Public key to verify with, given as a file \
+                         .help("The sender's certificate verify signatures \
+                                with, given as a file \
                                 (can be given multiple times)"))
                     .arg(Arg::with_name("secret-key-file")
                          .long("secret-key-file")
@@ -94,7 +95,7 @@ pub fn build() -> App<'static, 'static> {
                          .long("recipient-key-file")
                          .multiple(true)
                          .takes_value(true)
-                         .value_name("TPK-FILE")
+                         .value_name("CERT-FILE")
                          .number_of_values(1)
                          .help("Recipient to encrypt for, given as a file \
                                 (can be given multiple times)"))
@@ -128,7 +129,13 @@ pub fn build() -> App<'static, 'static> {
                          .possible_values(&["none", "pad", "zip", "zlib",
                                             "bzip2"])
                          .default_value("pad")
-                         .help("Selects compression scheme to use")))
+                         .help("Selects compression scheme to use"))
+                    .arg(Arg::with_name("time").value_name("TIME")
+                         .long("time")
+                         .short("t")
+                         .help("Chooses keys valid at the specified time and \
+                                sets the signature's creation time"))
+        )
 
         .subcommand(SubCommand::with_name("sign")
                     .display_order(25)
@@ -163,7 +170,12 @@ pub fn build() -> App<'static, 'static> {
                          .value_name("TSK-FILE")
                          .number_of_values(1)
                          .help("Secret key to sign with, given as a file \
-                                (can be given multiple times)")))
+                                (can be given multiple times)"))
+                    .arg(Arg::with_name("time").value_name("TIME")
+                         .long("time")
+                         .short("t")
+                         .help("Chooses keys valid at the specified time and \
+                                sets the signature's creation time")))
         .subcommand(SubCommand::with_name("verify")
                     .display_order(26)
                     .about("Verifies a message")
@@ -184,13 +196,14 @@ pub fn build() -> App<'static, 'static> {
                          .long("signatures")
                          .short("n")
                          .takes_value(true))
-                    .arg(Arg::with_name("public-key-file")
-                         .long("public-key-file")
+                    .arg(Arg::with_name("sender-cert-file")
+                         .long("sender-cert-file")
                          .multiple(true)
                          .takes_value(true)
-                         .value_name("TPK-FILE")
+                         .value_name("CERT-FILE")
                          .number_of_values(1)
-                         .help("Public key to verify with, given as a file \
+                         .help("The sender's certificate verify signatures \
+                                with, given as a file \
                                 (can be given multiple times)")))
         .subcommand(SubCommand::with_name("enarmor")
                     .about("Applies ASCII Armor to a file")
@@ -220,7 +233,8 @@ pub fn build() -> App<'static, 'static> {
                     .about("Autocrypt support")
                     .setting(AppSettings::SubcommandRequiredElseHelp)
                     .subcommand(SubCommand::with_name("decode")
-                                .about("Converts Autocrypt-encoded keys to OpenPGP TPKs")
+                                .about("Converts Autocrypt-encoded keys to \
+                                        OpenPGP Certificates")
                                 .arg(Arg::with_name("input").value_name("FILE")
                                      .help("Sets the input file to use"))
                                 .arg(Arg::with_name("output").value_name("FILE")
@@ -228,7 +242,8 @@ pub fn build() -> App<'static, 'static> {
                                      .short("o")
                                      .help("Sets the output file to use")))
                     .subcommand(SubCommand::with_name("encode-sender")
-                                .about("Encodes the senders' OpenPGP TPKs into \
+                                .about("Encodes the sender's OpenPGP \
+                                        Certificates into \
                                         an Autocrypt header")
                                 .arg(Arg::with_name("input").value_name("FILE")
                                      .help("Sets the input file to use"))
@@ -284,12 +299,12 @@ pub fn build() -> App<'static, 'static> {
                                 .about("Sends a key")
                                 .arg(Arg::with_name("input").value_name("FILE")
                                      .help("Sets the input file to use"))))
-        .subcommand(SubCommand::with_name("store")
+        .subcommand(SubCommand::with_name("mapping")
                     .display_order(30)
-                    .about("Interacts with key stores")
+                    .about("Interacts with key mappings")
                     .setting(AppSettings::SubcommandRequiredElseHelp)
                     .subcommand(SubCommand::with_name("list")
-                                .about("Lists keys in the store"))
+                                .about("Lists keys in the mapping"))
                     .subcommand(SubCommand::with_name("add")
                                 .about("Add a key identified by fingerprint")
                                 .arg(Arg::with_name("label").value_name("LABEL")
@@ -319,10 +334,10 @@ pub fn build() -> App<'static, 'static> {
                                      .short("B")
                                      .help("Don't ASCII-armor encode the OpenPGP data")))
                     .subcommand(SubCommand::with_name("delete")
-                                .about("Deletes bindings or stores")
-                                .arg(Arg::with_name("the-store")
-                                     .long("the-store")
-                                     .help("Delete the selected store (change with --store)"))
+                                .about("Deletes bindings or mappings")
+                                .arg(Arg::with_name("the-mapping")
+                                     .long("the-mapping")
+                                     .help("Delete the selected mapping (change with --mapping)"))
                                 .arg(Arg::with_name("label")
                                      .value_name("LABEL")
                                      .help("Delete binding with this label")))
@@ -337,16 +352,16 @@ pub fn build() -> App<'static, 'static> {
                                      .value_name("LABEL")
                                      .help("List messages related to this label"))))
         .subcommand(SubCommand::with_name("list")
-                    .about("Lists key stores and known keys")
+                    .about("Lists key mappings and known keys")
                     .setting(AppSettings::SubcommandRequiredElseHelp)
-                    .subcommand(SubCommand::with_name("stores")
-                                .about("Lists key stores")
+                    .subcommand(SubCommand::with_name("mappings")
+                                .about("Lists key mappings")
                                 .arg(Arg::with_name("prefix").value_name("PREFIX")
-                                     .help("List only stores with the given realm prefix")))
+                                     .help("List only mappings with the given realm prefix")))
                     .subcommand(SubCommand::with_name("bindings")
-                                .about("Lists all bindings in all key stores")
+                                .about("Lists all bindings in all key mappings")
                                 .arg(Arg::with_name("prefix").value_name("PREFIX")
-                                     .help("List only bindings from stores with the given realm prefix")))
+                                     .help("List only bindings from mappings with the given realm prefix")))
                     .subcommand(SubCommand::with_name("keys")
                                 .about("Lists all keys in the common key pool"))
                     .subcommand(SubCommand::with_name("log")
@@ -362,12 +377,16 @@ pub fn build() -> App<'static, 'static> {
                              .value_name("EMAIL")
                              .long("userid")
                              .short("u")
-                             .help("Primary user ID"))
+                             .multiple(true)
+                             .number_of_values(1)
+                             .takes_value(true)
+                             .help("Add userid to the key \
+                                    (can be given multiple times)"))
                         .arg(Arg::with_name("cipher-suite")
                              .value_name("CIPHER-SUITE")
                              .long("cipher-suite")
                              .short("c")
-                             .possible_values(&["rsa3k", "cv25519"])
+                             .possible_values(&["rsa3k", "rsa4k", "cv25519"])
                              .default_value("cv25519")
                              .help("Cryptographic algorithms used for the key."))
                         .arg(Arg::with_name("with-password")
@@ -397,9 +416,10 @@ pub fn build() -> App<'static, 'static> {
                                .args(&["can-encrypt", "cannot-encrypt"]))
                         .arg(Arg::with_name("can-encrypt").value_name("PURPOSE")
                              .long("can-encrypt")
-                             .possible_values(&["transport", "rest", "all"])
+                             .possible_values(&["transport", "storage",
+                                                "universal"])
                              .help("The key has an encryption-capable subkey \
-                                    (default)"))
+                                    (default: universal)"))
                         .arg(Arg::with_name("cannot-encrypt")
                              .long("cannot-encrypt")
                              .help("The key will not be able to encrypt data"))
@@ -441,6 +461,36 @@ pub fn build() -> App<'static, 'static> {
                                      .long("hex")
                                      .short("x")
                                      .help("Print a hexdump")))
+
+                    .subcommand(SubCommand::with_name("decrypt")
+                                .display_order(10)
+                                .about("Decrypts an OpenPGP message, dumping \
+                                        the content of the encryption \
+                                        container without further processing")
+                                .arg(Arg::with_name("input").value_name("FILE")
+                                     .help("Sets the input file to use"))
+                                .arg(Arg::with_name("output").value_name("FILE")
+                                     .long("output")
+                                     .short("o")
+                                     .help("Sets the output file to use"))
+                                .arg(Arg::with_name("binary")
+                                     .long("binary")
+                                     .short("B")
+                                     .help("Don't ASCII-armor encode the \
+                                            OpenPGP data"))
+                                .arg(Arg::with_name("secret-key-file")
+                                     .long("secret-key-file")
+                                     .multiple(true)
+                                     .takes_value(true)
+                                     .value_name("TSK-FILE")
+                                     .number_of_values(1)
+                                     .help("Secret key to decrypt with, given \
+                                            as a file \
+                                            (can be given multiple times)"))
+                                .arg(Arg::with_name("dump-session-key")
+                                     .long("dump-session-key")
+                                     .help("Prints the session key to stderr")))
+
                     .subcommand(SubCommand::with_name("split")
                                 .about("Splits a message into OpenPGP packets")
                                 .arg(Arg::with_name("input").value_name("FILE")
@@ -489,17 +539,41 @@ pub fn build() -> App<'static, 'static> {
                                             obtain the WKD URI.")))
                     .subcommand(SubCommand::with_name("get")
                                 .about("Writes to the standard output the \
-                                        TPK retrieved \
+                                        Cert retrieved \
                                         from a Web Key Directory, given an \
                                         email address")
                                 .arg(Arg::with_name("input")
                                     .value_name("EMAIL_ADDRESS")
                                     .required(true)
                                     .help("The email address from which to \
-                                            obtain the TPK from a WKD."))
+                                            obtain the Cert from a WKD."))
                                 .arg(Arg::with_name("binary")
                                     .long("binary")
                                     .short("B")
                                     .help("Don't ASCII-armor encode the OpenPGP data")))
+                    .subcommand(SubCommand::with_name("generate")
+                                .about("Generates a Web Key Directory \
+                                        for the given domain and keys.  \
+                                        If the WKD exists, the new \
+                                        keys will be inserted and it \
+                                        is updated and existing ones \
+                                        will be updated.")
+                                .arg(Arg::with_name("base_directory")
+                                     .value_name("BASE-DIRECTORY")
+                                     .required(true)
+                                     .help("The location to write the WKD to"))
+                                .arg(Arg::with_name("domain")
+                                    .value_name("DOMAIN")
+                                    .help("The domain for the WKD.")
+                                    .required(true))
+                                .arg(Arg::with_name("input")
+                                    .value_name("KEYRING")
+                                    .help("The keyring file with the keys to add to the WKD."))
+                                .arg(Arg::with_name("direct_method")
+                                     .long("direct_method")
+                                     .short("d")
+                                     .help("Use the direct method. \
+                                            [default: advanced method]"))
+                    )
         )
 }
