@@ -7,14 +7,14 @@ extern crate tempfile;
 use tempfile::TempDir;
 
 extern crate sequoia_openpgp as openpgp;
-use crate::openpgp::{Packet, PacketPile, TPK};
+use crate::openpgp::{Packet, PacketPile, Cert};
 use crate::openpgp::crypto::KeyPair;
 use crate::openpgp::packet::key::SecretKeyMaterial;
-use crate::openpgp::constants::{CompressionAlgorithm, DataFormat, SignatureType};
+use crate::openpgp::types::{CompressionAlgorithm, SignatureType};
 use crate::openpgp::parse::Parse;
 use crate::openpgp::serialize::stream::{Message, Signer, Compressor, LiteralWriter};
 
-fn p(filename: &str) -> String {
+fn artifact(filename: &str) -> String {
     format!("../openpgp/tests/data/{}", filename)
 }
 
@@ -30,10 +30,10 @@ fn sq_sign() {
               &tmp_dir.path().to_string_lossy(),
               "sign",
               "--secret-key-file",
-              &p("keys/dennis-simon-anton-private.pgp"),
+              &artifact("keys/dennis-simon-anton-private.pgp"),
               "--output",
               &sig.to_string_lossy(),
-              &p("messages/a-cypherpunks-manifesto.txt")])
+              &artifact("messages/a-cypherpunks-manifesto.txt")])
         .unwrap();
 
     // Check that the content is sane.
@@ -66,8 +66,8 @@ fn sq_sign() {
             &["--home",
               &tmp_dir.path().to_string_lossy(),
               "verify",
-              "--public-key-file",
-              &p("keys/dennis-simon-anton.pgp"),
+              "--sender-cert-file",
+              &artifact("keys/dennis-simon-anton.pgp"),
               &sig.to_string_lossy()])
         .unwrap();
 }
@@ -84,10 +84,10 @@ fn sq_sign_append() {
               &tmp_dir.path().to_string_lossy(),
               "sign",
               "--secret-key-file",
-              &p("keys/dennis-simon-anton-private.pgp"),
+              &artifact("keys/dennis-simon-anton-private.pgp"),
               "--output",
               &sig0.to_string_lossy(),
-              &p("messages/a-cypherpunks-manifesto.txt")])
+              &artifact("messages/a-cypherpunks-manifesto.txt")])
         .unwrap();
 
     // Check that the content is sane.
@@ -120,8 +120,8 @@ fn sq_sign_append() {
             &["--home",
               &tmp_dir.path().to_string_lossy(),
               "verify",
-              "--public-key-file",
-              &p("keys/dennis-simon-anton.pgp"),
+              "--sender-cert-file",
+              &artifact("keys/dennis-simon-anton.pgp"),
               &sig0.to_string_lossy()])
         .unwrap();
 
@@ -134,7 +134,7 @@ fn sq_sign_append() {
               "sign",
               "--append",
               "--secret-key-file",
-              &p("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
+              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
               "--output",
               &sig1.to_string_lossy(),
               &sig0.to_string_lossy()])
@@ -183,8 +183,8 @@ fn sq_sign_append() {
             &["--home",
               &tmp_dir.path().to_string_lossy(),
               "verify",
-              "--public-key-file",
-              &p("keys/dennis-simon-anton.pgp"),
+              "--sender-cert-file",
+              &artifact("keys/dennis-simon-anton.pgp"),
               &sig1.to_string_lossy()])
         .unwrap();
     Assert::cargo_binary("sq")
@@ -192,8 +192,8 @@ fn sq_sign_append() {
             &["--home",
               &tmp_dir.path().to_string_lossy(),
               "verify",
-              "--public-key-file",
-              &p("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp"),
+              "--sender-cert-file",
+              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp"),
               &sig1.to_string_lossy()])
         .unwrap();
 }
@@ -201,30 +201,31 @@ fn sq_sign_append() {
 #[test]
 #[allow(unreachable_code)]
 fn sq_sign_append_on_compress_then_sign() {
+    use crate::openpgp::policy::StandardPolicy as P;
+
+    let p = &P::new();
     let tmp_dir = TempDir::new().unwrap();
     let sig0 = tmp_dir.path().join("sig0");
 
     // This is quite an odd scheme, so we need to create such a
     // message by foot.
-    let tsk = TPK::from_file(&p("keys/dennis-simon-anton-private.pgp"))
+    let tsk = Cert::from_file(&artifact("keys/dennis-simon-anton-private.pgp"))
         .unwrap();
-    let key = tsk.keys_all().signing_capable().nth(0).unwrap().2;
+    let key = tsk.keys().with_policy(p, None).for_signing().nth(0).unwrap().key();
     let sec = match key.secret() {
         Some(SecretKeyMaterial::Unencrypted(ref u)) => u.clone(),
         _ => unreachable!(),
     };
-    let mut keypair = KeyPair::new(key.clone(), sec).unwrap();
+    let keypair = KeyPair::new(key.clone(), sec).unwrap();
     let signer = Signer::new(Message::new(File::create(&sig0).unwrap()),
-                             vec![&mut keypair], None)
-        .unwrap();
-    let compressor = Compressor::new(signer, CompressionAlgorithm::Uncompressed,
-                                     None)
-        .unwrap();
-    let mut literal = LiteralWriter::new(compressor, DataFormat::Binary, None,
-                                         None)
+                             keypair).build().unwrap();
+    let compressor = Compressor::new(signer)
+        .algo(CompressionAlgorithm::Uncompressed)
+        .build().unwrap();
+    let mut literal = LiteralWriter::new(compressor).build()
         .unwrap();
     io::copy(
-        &mut File::open(&p("messages/a-cypherpunks-manifesto.txt")).unwrap(),
+        &mut File::open(&artifact("messages/a-cypherpunks-manifesto.txt")).unwrap(),
         &mut literal)
         .unwrap();
     literal.finalize()
@@ -257,8 +258,8 @@ fn sq_sign_append_on_compress_then_sign() {
             &["--home",
               &tmp_dir.path().to_string_lossy(),
               "verify",
-              "--public-key-file",
-              &p("keys/dennis-simon-anton.pgp"),
+              "--sender-cert-file",
+              &artifact("keys/dennis-simon-anton.pgp"),
               &sig0.to_string_lossy()])
         .unwrap();
 
@@ -271,7 +272,7 @@ fn sq_sign_append_on_compress_then_sign() {
               "sign",
               "--append",
               "--secret-key-file",
-              &p("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
+              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
               "--output",
               &sig1.to_string_lossy(),
               &sig0.to_string_lossy()])
@@ -324,8 +325,8 @@ fn sq_sign_append_on_compress_then_sign() {
             &["--home",
               &tmp_dir.path().to_string_lossy(),
               "verify",
-              "--public-key-file",
-              &p("keys/dennis-simon-anton.pgp"),
+              "--sender-cert-file",
+              &artifact("keys/dennis-simon-anton.pgp"),
               &sig0.to_string_lossy()])
         .unwrap();
     Assert::cargo_binary("sq")
@@ -333,8 +334,8 @@ fn sq_sign_append_on_compress_then_sign() {
             &["--home",
               &tmp_dir.path().to_string_lossy(),
               "verify",
-              "--public-key-file",
-              &p("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp"),
+              "--sender-cert-file",
+              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp"),
               &sig0.to_string_lossy()])
         .unwrap();
 }
@@ -352,10 +353,10 @@ fn sq_sign_detached() {
               "sign",
               "--detached",
               "--secret-key-file",
-              &p("keys/dennis-simon-anton-private.pgp"),
+              &artifact("keys/dennis-simon-anton-private.pgp"),
               "--output",
               &sig.to_string_lossy(),
-              &p("messages/a-cypherpunks-manifesto.txt")])
+              &artifact("messages/a-cypherpunks-manifesto.txt")])
         .unwrap();
 
     // Check that the content is sane.
@@ -377,11 +378,11 @@ fn sq_sign_detached() {
             &["--home",
               &tmp_dir.path().to_string_lossy(),
               "verify",
-              "--public-key-file",
-              &p("keys/dennis-simon-anton.pgp"),
+              "--sender-cert-file",
+              &artifact("keys/dennis-simon-anton.pgp"),
               "--detached",
               &sig.to_string_lossy(),
-              &p("messages/a-cypherpunks-manifesto.txt")])
+              &artifact("messages/a-cypherpunks-manifesto.txt")])
         .unwrap();
 }
 
@@ -398,10 +399,10 @@ fn sq_sign_detached_append() {
               "sign",
               "--detached",
               "--secret-key-file",
-              &p("keys/dennis-simon-anton-private.pgp"),
+              &artifact("keys/dennis-simon-anton-private.pgp"),
               "--output",
               &sig.to_string_lossy(),
-              &p("messages/a-cypherpunks-manifesto.txt")])
+              &artifact("messages/a-cypherpunks-manifesto.txt")])
         .unwrap();
 
     // Check that the content is sane.
@@ -423,11 +424,11 @@ fn sq_sign_detached_append() {
             &["--home",
               &tmp_dir.path().to_string_lossy(),
               "verify",
-              "--public-key-file",
-              &p("keys/dennis-simon-anton.pgp"),
+              "--sender-cert-file",
+              &artifact("keys/dennis-simon-anton.pgp"),
               "--detached",
               &sig.to_string_lossy(),
-              &p("messages/a-cypherpunks-manifesto.txt")])
+              &artifact("messages/a-cypherpunks-manifesto.txt")])
         .unwrap();
 
     // Check that we don't blindly overwrite signatures.
@@ -438,10 +439,10 @@ fn sq_sign_detached_append() {
               "sign",
               "--detached",
               "--secret-key-file",
-              &p("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
+              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
               "--output",
               &sig.to_string_lossy(),
-              &p("messages/a-cypherpunks-manifesto.txt")])
+              &artifact("messages/a-cypherpunks-manifesto.txt")])
         .fails()
         .unwrap();
 
@@ -454,10 +455,10 @@ fn sq_sign_detached_append() {
               "--detached",
               "--append",
               "--secret-key-file",
-              &p("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
+              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
               "--output",
               &sig.to_string_lossy(),
-              &p("messages/a-cypherpunks-manifesto.txt")])
+              &artifact("messages/a-cypherpunks-manifesto.txt")])
         .unwrap();
 
     // Check that the content is sane.
@@ -484,22 +485,22 @@ fn sq_sign_detached_append() {
             &["--home",
               &tmp_dir.path().to_string_lossy(),
               "verify",
-              "--public-key-file",
-              &p("keys/dennis-simon-anton.pgp"),
+              "--sender-cert-file",
+              &artifact("keys/dennis-simon-anton.pgp"),
               "--detached",
               &sig.to_string_lossy(),
-              &p("messages/a-cypherpunks-manifesto.txt")])
+              &artifact("messages/a-cypherpunks-manifesto.txt")])
         .unwrap();
     Assert::cargo_binary("sq")
         .with_args(
             &["--home",
               &tmp_dir.path().to_string_lossy(),
               "verify",
-              "--public-key-file",
-              &p("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp"),
+              "--sender-cert-file",
+              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp"),
               "--detached",
               &sig.to_string_lossy(),
-              &p("messages/a-cypherpunks-manifesto.txt")])
+              &artifact("messages/a-cypherpunks-manifesto.txt")])
         .unwrap();
 
     // Finally, check that we don't truncate the file if something
@@ -513,10 +514,10 @@ fn sq_sign_detached_append() {
               "--append",
               "--secret-key-file",
               // Not a private key => signing will fail.
-              &p("keys/erika-corinna-daniela-simone-antonia-nistp521.pgp"),
+              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp521.pgp"),
               "--output",
               &sig.to_string_lossy(),
-              &p("messages/a-cypherpunks-manifesto.txt")])
+              &artifact("messages/a-cypherpunks-manifesto.txt")])
         .fails()
         .unwrap();
 
@@ -551,10 +552,10 @@ fn sq_sign_append_a_notarization() {
               "sign",
               "--append",
               "--secret-key-file",
-              &p("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
+              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
               "--output",
               &sig0.to_string_lossy(),
-              &p("messages/signed-1-notarized-by-ed25519.pgp")])
+              &artifact("messages/signed-1-notarized-by-ed25519.pgp")])
         .unwrap();
 
     // Check that the content is sane.
@@ -612,8 +613,8 @@ fn sq_sign_append_a_notarization() {
             &["--home",
               &tmp_dir.path().to_string_lossy(),
               "verify",
-              "--public-key-file",
-              &p("keys/neal.pgp"),
+              "--sender-cert-file",
+              &artifact("keys/neal.pgp"),
               &sig0.to_string_lossy()])
         .unwrap();
     Assert::cargo_binary("sq")
@@ -621,8 +622,8 @@ fn sq_sign_append_a_notarization() {
             &["--home",
               &tmp_dir.path().to_string_lossy(),
               "verify",
-              "--public-key-file",
-              &p("keys/emmelie-dorothea-dina-samantha-awina-ed25519.pgp"),
+              "--sender-cert-file",
+              &artifact("keys/emmelie-dorothea-dina-samantha-awina-ed25519.pgp"),
               &sig0.to_string_lossy()])
         .unwrap();
     Assert::cargo_binary("sq")
@@ -630,8 +631,8 @@ fn sq_sign_append_a_notarization() {
             &["--home",
               &tmp_dir.path().to_string_lossy(),
               "verify",
-              "--public-key-file",
-              &p("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp"),
+              "--sender-cert-file",
+              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp"),
               &sig0.to_string_lossy()])
         .unwrap();
 }
@@ -649,10 +650,10 @@ fn sq_sign_notarize() {
               "sign",
               "--notarize",
               "--secret-key-file",
-              &p("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
+              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
               "--output",
               &sig0.to_string_lossy(),
-              &p("messages/signed-1.gpg")])
+              &artifact("messages/signed-1.gpg")])
         .unwrap();
 
     // Check that the content is sane.
@@ -698,8 +699,8 @@ fn sq_sign_notarize() {
             &["--home",
               &tmp_dir.path().to_string_lossy(),
               "verify",
-              "--public-key-file",
-              &p("keys/neal.pgp"),
+              "--sender-cert-file",
+              &artifact("keys/neal.pgp"),
               &sig0.to_string_lossy()])
         .unwrap();
     Assert::cargo_binary("sq")
@@ -707,8 +708,8 @@ fn sq_sign_notarize() {
             &["--home",
               &tmp_dir.path().to_string_lossy(),
               "verify",
-              "--public-key-file",
-              &p("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp"),
+              "--sender-cert-file",
+              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp"),
               &sig0.to_string_lossy()])
         .unwrap();
 }
@@ -726,10 +727,10 @@ fn sq_sign_notarize_a_notarization() {
               "sign",
               "--notarize",
               "--secret-key-file",
-              &p("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
+              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
               "--output",
               &sig0.to_string_lossy(),
-              &p("messages/signed-1-notarized-by-ed25519.pgp")])
+              &artifact("messages/signed-1-notarized-by-ed25519.pgp")])
         .unwrap();
 
     // Check that the content is sane.
@@ -787,8 +788,8 @@ fn sq_sign_notarize_a_notarization() {
             &["--home",
               &tmp_dir.path().to_string_lossy(),
               "verify",
-              "--public-key-file",
-              &p("keys/neal.pgp"),
+              "--sender-cert-file",
+              &artifact("keys/neal.pgp"),
               &sig0.to_string_lossy()])
         .unwrap();
     Assert::cargo_binary("sq")
@@ -796,8 +797,8 @@ fn sq_sign_notarize_a_notarization() {
             &["--home",
               &tmp_dir.path().to_string_lossy(),
               "verify",
-              "--public-key-file",
-              &p("keys/emmelie-dorothea-dina-samantha-awina-ed25519.pgp"),
+              "--sender-cert-file",
+              &artifact("keys/emmelie-dorothea-dina-samantha-awina-ed25519.pgp"),
               &sig0.to_string_lossy()])
         .unwrap();
     Assert::cargo_binary("sq")
@@ -805,8 +806,8 @@ fn sq_sign_notarize_a_notarization() {
             &["--home",
               &tmp_dir.path().to_string_lossy(),
               "verify",
-              "--public-key-file",
-              &p("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp"),
+              "--sender-cert-file",
+              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp"),
               &sig0.to_string_lossy()])
         .unwrap();
 }
